@@ -1,6 +1,6 @@
 /*
 	This file is part of Warzone 2100.
-	Copyright (C) 2005-2008  Warzone Resurrection Project
+	Copyright (C) 2005-2009  Warzone Resurrection Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -21,6 +21,8 @@
 #include <locale.h>
 #include <physfs.h>
 
+#include "string_ext.h"
+
 
 /* Always use fallbacks on Windows */
 #if defined(WZ_OS_WIN)
@@ -45,15 +47,15 @@ static const struct
 	{ "cs", N_("Czech"), LANG_CZECH, SUBLANG_DEFAULT },
 	{ "da", N_("Danish"), LANG_DANISH, SUBLANG_DEFAULT },
 	{ "de", N_("German"), LANG_GERMAN, SUBLANG_DEFAULT },
-	{ "en", N_("English"), LANG_ENGLISH, SUBLANG_DEFAULT },
+//	{ "en", N_("English"), LANG_ENGLISH, SUBLANG_DEFAULT },
 	{ "en_GB", N_("English (United Kingdom)"), LANG_ENGLISH, SUBLANG_DEFAULT },
 	{ "es", N_("Spanish"), LANG_SPANISH, SUBLANG_DEFAULT },
-	{ "eu", N_("Basque"), LANG_BASQUE, SUBLANG_DEFAULT },
+//	{ "eu", N_("Basque"), LANG_BASQUE, SUBLANG_DEFAULT },
 	{ "fi", N_("Finnish"), LANG_FINNISH, SUBLANG_DEFAULT },
 	{ "fr", N_("French"), LANG_FRENCH, SUBLANG_DEFAULT },
 	/* Our Frisian translation is the "West Frisian" variation of it. This
-	 * variation is mostly spoken in the Dutch province Friesland and has
-	 * ISO 639-3 code "fry".
+	 * variation is mostly spoken in the Dutch province Friesland (Fryslân
+	 * in Frisian) and has ISO 639-3 code "fry".
 	 *
 	 * FIXME: We should really use a sub-language code for this. E.g.
 	 *        fy_XX.
@@ -61,26 +63,28 @@ static const struct
 	{ "fy", N_("Frisian"), LANG_FRISIAN, SUBLANG_FRISIAN_NETHERLANDS },
 	{ "ga", N_("Irish"), LANG_IRISH, SUBLANG_DEFAULT },
 	{ "it", N_("Italian"), LANG_ITALIAN, SUBLANG_DEFAULT },
+//	{ "la", N_("Latin"), LANG_LATIN, SUBLANG_DEFAULT },
 	{ "lt", N_("Lithuanian"), LANG_LITHUANIAN, SUBLANG_DEFAULT },
-	{ "lv", N_("Latvian"), LANG_LATVIAN, SUBLANG_DEFAULT },
+//	{ "lv", N_("Latvian"), LANG_LATVIAN, SUBLANG_DEFAULT },
 	{ "nb", N_("Norwegian"), LANG_NORWEGIAN, SUBLANG_DEFAULT },
-	{ "nn", N_("Norwegian (Nynorsk)"), LANG_NORWEGIAN, SUBLANG_NORWEGIAN_NYNORSK },
+//	{ "nn", N_("Norwegian (Nynorsk)"), LANG_NORWEGIAN, SUBLANG_NORWEGIAN_NYNORSK },
 	{ "nl", N_("Dutch"), LANG_DUTCH, SUBLANG_DEFAULT },
 	{ "pl", N_("Polish"), LANG_POLISH, SUBLANG_DEFAULT },
 	{ "pt_BR", N_("Brazilian Portuguese"), LANG_PORTUGUESE, SUBLANG_PORTUGUESE_BRAZILIAN },
-	{ "pt", N_("Portuegese"), LANG_PORTUGUESE, SUBLANG_DEFAULT },
+	{ "pt", N_("Portuguese"), LANG_PORTUGUESE, SUBLANG_DEFAULT },
 	{ "ro", N_("Romanian"), LANG_ROMANIAN, SUBLANG_DEFAULT },
 	{ "ru", N_("Russian"), LANG_RUSSIAN, SUBLANG_DEFAULT },
 	{ "sl", N_("Slovenian"), LANG_SLOVENIAN, SUBLANG_DEFAULT },
 #if (WINVER >= 0x0600)
-	{ "sv_SE", N_("Swedish (Sweden)"), LANG_SWEDISH, SUBLANG_SWEDISH_SWEDEN },
+//	{ "sv_SE", N_("Swedish (Sweden)"), LANG_SWEDISH, SUBLANG_SWEDISH_SWEDEN },
 #else
-	{ "sv_SE", N_("Swedish (Sweden)"), LANG_SWEDISH, SUBLANG_SWEDISH },
+//	{ "sv_SE", N_("Swedish (Sweden)"), LANG_SWEDISH, SUBLANG_SWEDISH },
 #endif
-	{ "sv", N_("Swedish"), LANG_SWEDISH, SUBLANG_DEFAULT },
-	{ "tr", N_("Turkish"), LANG_TURKISH, SUBLANG_DEFAULT },
-	{ "uz", N_("Uzbek (Cyrillic)"), LANG_UZBEK, SUBLANG_UZBEK_CYRILLIC },
+//	{ "sv", N_("Swedish"), LANG_SWEDISH, SUBLANG_DEFAULT },
+//	{ "tr", N_("Turkish"), LANG_TURKISH, SUBLANG_DEFAULT },
+//	{ "uz", N_("Uzbek (Cyrillic)"), LANG_UZBEK, SUBLANG_UZBEK_CYRILLIC },
 	{ "zh_CN", N_("Simplified Chinese"), LANG_CHINESE, SUBLANG_CHINESE_SIMPLIFIED },
+	{ "zh_TW", N_("Traditional Chinese"), LANG_CHINESE, SUBLANG_CHINESE_TRADITIONAL },
 #  endif
 };
 #else
@@ -96,15 +100,15 @@ static const struct
 	{ "cs", N_("Czech"), "cs.UTF-8", "cs" },
 	{ "da", N_("Danish"), "da_DK.UTF-8", "da_DK" },
 	{ "de", N_("German"), "de_DE.UTF-8", "de_DE" },
-	{ "en", N_("English"), "en_US.UTF-8", "en_US" },
+//	{ "en", N_("English"), "en_US.UTF-8", "en_US" },
 	{ "en_GB", N_("English (United Kingdom)"), "en_GB.UTF-8", "en_GB" },
 	{ "es", N_("Spanish"), "es.UTF-8", "es" },
-	{ "eu", N_("Basque"), "eu.UTF-8", "eu" },
+//	{ "eu", N_("Basque"), "eu.UTF-8", "eu" },
 	{ "fi", N_("Finnish"), "fi.UTF-8", "fi" },
 	{ "fr", N_("French"), "fr_FR.UTF-8", "fr_FR" },
 	/* Our Frisian translation is the "West Frisian" variation of it. This
-	 * variation is mostly spoken in the Dutch province Friesland and has
-	 * ISO 639-3 code "fry".
+	 * variation is mostly spoken in the Dutch province Friesland (Fryslân
+	 * in Frisian) and has ISO 639-3 code "fry".
 	 *
 	 * FIXME: We should really use a sub-language code for this. E.g.
 	 *        fy_XX.
@@ -114,21 +118,22 @@ static const struct
 	{ "it", N_("Italian"), "it_IT.UTF-8", "it_IT" },
 	{ "la", N_("Latin"), "la.UTF-8", "la" },
 	{ "lt", N_("Lithuanian"), "lt.UTF-8", "lt" },
-	{ "lv", N_("Latvian"), "lv.UTF-8", "lv" },
+//	{ "lv", N_("Latvian"), "lv.UTF-8", "lv" },
 	{ "nb", N_("Norwegian"), "nb_NO.UTF-8", "nb_NO" },
-	{ "nn", N_("Norwegian (Nynorsk)"), "nn.UTF-8", "nn" },
+//	{ "nn", N_("Norwegian (Nynorsk)"), "nn.UTF-8", "nn" },
 	{ "nl", N_("Dutch"), "nl_NL.UTF-8", "nl_NL" },
 	{ "pl", N_("Polish"), "pl.UTF-8", "pl" },
 	{ "pt_BR", N_("Brazilian Portuguese"), "pt_BR.UTF-8", "pt_BR" },
-	{ "pt", N_("Portuegese"), "pt_PT.UTF-8", "pt_PT" },
+	{ "pt", N_("Portuguese"), "pt_PT.UTF-8", "pt_PT" },
 	{ "ro", N_("Romanian"), "ro.UTF-8", "ro" },
 	{ "ru", N_("Russian"), "ru_RU.UTF-8", "ru_RU" },
 	{ "sl", N_("Slovenian"), "sl.UTF-8", "sl" },
-	{ "sv_SE", N_("Swedish (Sweden)"), "sv_SE.UTF-8", "sv_SE" },
-	{ "sv", N_("Swedish"), "sv.UTF-8", "sv" },
-	{ "tr", N_("Turkish"), "tr.UTF-8", "tr" },
-	{ "uz", N_("Uzbek (Cyrillic)"), "uz.UTF-8", "uz" },
+//	{ "sv_SE", N_("Swedish (Sweden)"), "sv_SE.UTF-8", "sv_SE" },
+//	{ "sv", N_("Swedish"), "sv.UTF-8", "sv" },
+//	{ "tr", N_("Turkish"), "tr.UTF-8", "tr" },
+//	{ "uz", N_("Uzbek (Cyrillic)"), "uz.UTF-8", "uz" },
 	{ "zh_CN", N_("Simplified Chinese"), "zh_CN.UTF-8", "zh_CN" },
+	{ "zh_TW", N_("Traditional Chinese"), "zh_TW.UTF-8", "zh_TW" },
 #  endif
 };
 #endif
@@ -168,7 +173,7 @@ const char *getLanguage(void)
 #else
 const char *getLanguage(void)
 {
-	static char language[4] = { '\0' }; // ISO639 language code has to fit in!
+	static char language[6] = { '\0' }; // large enough for xx_YY
 	const char *localeName = setlocale(LC_MESSAGES, NULL);
 	char *delim = NULL;
 
@@ -179,18 +184,23 @@ const char *getLanguage(void)
 
 	sstrcpy(language, localeName);
 
-	delim = strchr(language, '_');
-
-	if ( !delim )
-	{
-		delim = strchr(language, '.');
-	}
-
-	if ( delim ) // Cut after '_' or '.'
+	// cut anything after a '.' to get rid of the encoding part
+	delim = strchr(language, '.');
+	if (delim)
 	{
 		*delim = '\0';
 	}
 
+	// if language is xx_XX, cut the _XX part
+	delim = strchr(language, '_');
+	if (delim)
+	{
+		if (!strncasecmp(language, delim + 1, 2))
+		{
+			*delim = '\0';
+		}
+	}
+
 	return language;
 }
 #endif
@@ -209,8 +219,7 @@ const char* getLanguageName(void)
 		}
 	}
 
-	ASSERT(false, "getLanguageName: Unknown language");
-	return NULL;
+	return language;
 }
 
 
@@ -221,9 +230,13 @@ static BOOL setLocaleWindows(USHORT usPrimaryLanguage, USHORT usSubLanguage)
 	BOOL success = SUCCEEDED( SetThreadLocale( MAKELCID( MAKELANGID(usPrimaryLanguage, usSubLanguage), SORT_DEFAULT ) ) );
 
 	if (!success)
-		debug(LOG_ERROR, "Failed to set locale to \"%d\"", usPrimaryLanguage);
+	{
+		info("Failed to set locale to \"%d\"", usPrimaryLanguage);
+	}
 	else
+	{
 		debug(LOG_WZ, "Requested locale \"%d\"", usPrimaryLanguage);
+	}
 
 	setlocale(LC_NUMERIC, "C"); // set radix character to the period (".")
 
@@ -240,9 +253,13 @@ static BOOL setLocaleUnix(const char* locale)
 	const char *actualLocale = setlocale(LC_ALL, locale);
 
 	if (actualLocale == NULL)
-		debug(LOG_ERROR, "Failed to set locale to \"%s\"", locale);
+	{
+		info("Failed to set locale to \"%s\"", locale);
+	}
 	else
+	{
 		debug(LOG_WZ, "Requested locale \"%s\", got \"%s\" instead", locale, actualLocale);
+	}
 
 	setlocale(LC_NUMERIC, "C"); // set radix character to the period (".")
 
