@@ -3,15 +3,11 @@
 // ANSI C + stdint.h
 //
 // See header file for documentation.
-
-// #include <assert.h>
-// #include <stdlib.h>
-// #include <string.h>
+#include "tagfile.h"
 
 #include "frame.h"
 
-#include "tagfile.h"
-
+#include "string_ext.h"
 #include "physfs_ext.h"
 
 // Tags of values 0x0 and 0xFF are reserved as instance separators and group end
@@ -33,7 +29,7 @@ enum internal_types
 typedef struct _define
 {
 	unsigned int vm; //! value multiple
-	uint8_t element; //! tag number
+	element_t element; //! tag number
 	char vr[2]; //! value representation (type)
 	struct _define *parent;	//! parent group
 	struct _define *group;	//! child group
@@ -264,6 +260,7 @@ static bool init(const char *definition, const char *datafile, bool write)
 	tag_error = false;
 	line = 1;
 	fp = PHYSFS_openRead(definition);
+	debug(LOG_WZ, "Reading...[directory: %s] %s", PHYSFS_getRealDir(definition), definition);
 	sstrcpy(saveDefine, definition);
 	sstrcpy(saveTarget, datafile);
 	if (!fp)
@@ -317,6 +314,7 @@ static bool init(const char *definition, const char *datafile, bool write)
 	{
 		fp = PHYSFS_openRead(datafile);
 	}
+	debug(LOG_WZ, "Reading...[directory: %s] %s", PHYSFS_getRealDir(datafile), datafile);
 	handle = fp;
 	if (!fp)
 	{
