@@ -1,7 +1,7 @@
 /*
 	This file is part of Warzone 2100.
 	Copyright (C) 1999-2004  Eidos Interactive
-	Copyright (C) 2005-2007  Warzone Resurrection Project
+	Copyright (C) 2005-2009  Warzone Resurrection Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -49,7 +49,7 @@
 static GLubyte *aColour = NULL;
 static GLfloat *aTexCoord = NULL;
 static GLfloat *aVertex = NULL;
-static GLuint allocX = 0, allocY = 0;
+static GLint allocX = 0, allocY = 0;
 static size_t sizeColour = 0, sizeTexCoord = 0, sizeVertex = 0;
 
 extern BOOL drawing_interface;
@@ -218,7 +218,7 @@ static void pie_Draw3DShape2(iIMDShape *shape, int frame, PIELIGHT colour, WZ_DE
 	{
 		Vector2f	texCoords[pie_MAX_VERTICES_PER_POLYGON];
 		Vector3f	vertexCoords[pie_MAX_VERTICES_PER_POLYGON];
-		int		n;
+		unsigned int n;
 		VERTEXID	*index;
 
 		for (n = 0, index = pPolys->pindex;
@@ -388,7 +388,7 @@ static void pie_DrawShadow(iIMDShape *shape, int flag, int flag_data, Vector3f*
 			for(j = 0; j < 3; j++)
 			{
 				current = pPolys->pindex[j];
-				Vector3f_Set(&p[j], pVertices[current].x, scale_y(pVertices[current].y, flag, flag_data), pVertices[current].z);
+				p[j] = Vector3f_Init(pVertices[current].x, scale_y(pVertices[current].y, flag, flag_data), pVertices[current].z);
 			}
 
 			v[0] = Vector3f_Sub(p[2], p[0]);
@@ -741,7 +741,7 @@ void pie_RemainingPasses(void)
  * replaces all ivis blit functions
  *
  ***************************************************************************/
-void pie_DrawImage(PIEIMAGE *image, PIERECT *dest)
+void pie_DrawImage(const PIEIMAGE *image, const PIERECT *dest)
 {
 	PIELIGHT colour = WZCOL_WHITE;
 
