@@ -429,7 +429,10 @@ typedef enum
 
 	/* Misc */
 
+	ID_SOUND_HELP,
 	ID_SOUND_BARB_SCREAM,
+	ID_SOUND_BARB_SCREAM2,
+	ID_SOUND_BARB_SCREAM3,
 	ID_SOUND_OF_SILENCE,
 
 	/* Extra */
