@@ -1,7 +1,7 @@
 /*
 	This file is part of Warzone 2100.
 	Copyright (C) 1999-2004  Eidos Interactive
-	Copyright (C) 2005-2007  Warzone Resurrection Project
+	Copyright (C) 2005-2009  Warzone Resurrection Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -31,9 +31,9 @@ extern "C"
 void PlayList_Init(void);
 void PlayList_Quit(void);
 bool PlayList_Read(const char* path);
-void PlayList_SetTrack(unsigned int t);
 const char* PlayList_CurrentSong(void);
 const char* PlayList_NextSong(void);
+void playListTest(void);
 
 #if defined(__cplusplus)
 }
