@@ -1,7 +1,7 @@
 /*
 	This file is part of Warzone 2100.
 	Copyright (C) 1999-2004  Eidos Interactive
-	Copyright (C) 2005-2007  Warzone Resurrection Project
+	Copyright (C) 2005-2009  Warzone Resurrection Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -22,6 +22,7 @@
  */
 
 #include "lib/framework/frame.h"
+#include "lib/framework/string_ext.h"
 #include "lib/framework/frameint.h"
 #include "lib/ivis_common/textdraw.h"
 
@@ -164,7 +165,7 @@ void widgReleaseWidgetList(WIDGET *psWidgets)
 			sliderFree((W_SLIDER *)psCurr);
 			break;
 		default:
-			ASSERT( false,"widgReleaseWidgetList: Unknown widget type" );
+			ASSERT(!"Unknown widget type", "Unknown widget type");
 			break;
 		}
 	}
@@ -206,7 +207,7 @@ static void widgRelease(WIDGET *psWidget)
 		sliderFree((W_SLIDER *)psWidget);
 		break;
 	default:
-		ASSERT( false,"widgRelease: Unknown widget type" );
+		ASSERT(!"Unknown widget type", "Unknown widget type");
 		break;
 	}
 }
@@ -718,7 +719,7 @@ static void widgStartForm(W_FORM *psForm)
 			sliderInitialise((W_SLIDER *)psCurr);
 			break;
 		default:
-			ASSERT( false,"widgStartScreen: Unknown widget type" );
+			ASSERT(!"Unknown widget type", "Unknown widget type");
 			break;
 		}
 
@@ -836,7 +837,7 @@ void widgGetPos(W_SCREEN *psScreen, UDWORD id, SWORD *pX, SWORD *pY)
 	}
 	else
 	{
-		ASSERT( false, "widgGetPos: Couldn't find widget from ID" );
+		ASSERT(!"Couldn't find widget by ID", "Couldn't find widget by ID");
 		*pX = 0;
 		*pY = 0;
 	}
@@ -981,7 +982,7 @@ void widgSetTipText(WIDGET* psWidget, const char* pTip)
 			break;
 
 		default:
-			ASSERT(!"unknown widget type", "widgSetTip: Unknown widget type");
+			ASSERT(!"Unknown widget type", "Unknown widget type");
 			break;
 	}
 }
@@ -1007,7 +1008,7 @@ UDWORD widgGetButtonState(W_SCREEN *psScreen, UDWORD id)
 	psWidget = widgGetFromID(psScreen, id);
 	if (psWidget == NULL)
 	{
-		ASSERT( false, "widgGetButtonState: Couldn't find button/click form from ID" );
+		ASSERT(!"Couldn't find widget by ID", "Couldn't find button or clickable widget by ID");
 	}
 	else if (psWidget->type == WIDG_BUTTON)
 	{
@@ -1019,7 +1020,7 @@ UDWORD widgGetButtonState(W_SCREEN *psScreen, UDWORD id)
 	}
 	else
 	{
-		ASSERT( false, "widgGetButtonState: Couldn't find button/click form from ID" );
+		ASSERT(!"Couldn't find widget by ID", "Couldn't find button or clickable widget by ID");
 	}
 	return 0;
 }
@@ -1033,7 +1034,7 @@ void widgSetButtonFlash(W_SCREEN *psScreen, UDWORD id)
 	psWidget = widgGetFromID(psScreen, id);
 	if (psWidget == NULL)
 	{
-		ASSERT( false, "widgSetButtonFlash: Couldn't find button/click form from ID" );
+		ASSERT(!"Couldn't find widget by ID", "Couldn't find button or clickable widget by ID");
 	}
 	else if (psWidget->type == WIDG_BUTTON)
 	{
@@ -1049,7 +1050,7 @@ void widgSetButtonFlash(W_SCREEN *psScreen, UDWORD id)
 	}
 	else
 	{
-		ASSERT( false, "widgSetButtonFlash: Couldn't find button/click form from ID" );
+		ASSERT(!"Couldn't find widget by ID", "Couldn't find button or clickable widget by ID");
 	}
 }
 
@@ -1062,7 +1063,7 @@ void widgClearButtonFlash(W_SCREEN *psScreen, UDWORD id)
 	psWidget = widgGetFromID(psScreen, id);
 	if (psWidget == NULL)
 	{
-		ASSERT( false, "widgSetButtonFlash: Couldn't find button/click form from ID" );
+		ASSERT(!"Couldn't find widget by ID", "Couldn't find button or clickable widget by ID");
 	}
 	else if (psWidget->type == WIDG_BUTTON)
 	{
@@ -1077,7 +1078,7 @@ void widgClearButtonFlash(W_SCREEN *psScreen, UDWORD id)
 	}
 	else
 	{
-		ASSERT( false, "widgClearButtonFlash: Couldn't find button/click form from ID" );
+		ASSERT(!"Couldn't find widget by ID", "Couldn't find button or clickable widget by ID");
 	}
 }
 
@@ -1091,7 +1092,7 @@ void widgSetButtonState(W_SCREEN *psScreen, UDWORD id, UDWORD state)
 	psWidget = widgGetFromID(psScreen, id);
 	if (psWidget == NULL)
 	{
-		ASSERT( false, "widgSetButtonState: Couldn't find button/click form from ID" );
+		ASSERT(!"Couldn't find widget by ID", "Couldn't find button or clickable widget by ID");
 	}
 	else if (psWidget->type == WIDG_BUTTON)
 	{
@@ -1107,7 +1108,7 @@ void widgSetButtonState(W_SCREEN *psScreen, UDWORD id, UDWORD state)
 	}
 	else
 	{
-		ASSERT( false, "widgSetButtonState: Couldn't find button/click form from ID" );
+		ASSERT(!"Couldn't find widget by ID", "Couldn't find button or clickable widget by ID");
 	}
 }
 
@@ -1155,14 +1156,14 @@ const char *widgGetString(W_SCREEN *psScreen, UDWORD id)
 				aStringRetBuffer[0] = '\0';
 				break;
 			default:
-				ASSERT( false,"widgGetString: Unknown widget type" );
+				ASSERT(!"Unknown widget type", "Unknown widget type");
 				aStringRetBuffer[0] = '\0';
 				break;
 		}
 	}
 	else
 	{
-		ASSERT( false, "widgGetString: couldn't get widget from id" );
+		ASSERT(!"Couldn't find widget by ID", "widgGetString: couldn't find widget by ID");
 		aStringRetBuffer[0] = '\0';
 	}
 
@@ -1217,7 +1218,7 @@ void widgSetString(W_SCREEN *psScreen, UDWORD id, const char *pText)
 			break;
 
 		default:
-			ASSERT( !"unknown widget type", "widgSetString: Unknown widget type");
+			ASSERT(!"Unknown widget type", "Unknown widget type");
 			break;
 	}
 }
@@ -1576,7 +1577,7 @@ static void widgFocusLost(W_SCREEN* psScreen, WIDGET *psWidget)
 	case WIDG_SLIDER:
 		break;
 	default:
-		ASSERT( false,"widgFocusLost: Unknown widget type" );
+		ASSERT(!"Unknown widget type", "Unknown widget type");
 		break;
 	}
 }
@@ -1627,7 +1628,7 @@ void widgHiLite(WIDGET *psWidget, W_CONTEXT *psContext)
 		sliderHiLite((W_SLIDER *)psWidget);
 		break;
 	default:
-		ASSERT( false,"widgHiLite: Unknown widget type" );
+		ASSERT(!"Unknown widget type", "Unknown widget type");
 		break;
 	}
 }
@@ -1658,7 +1659,7 @@ void widgHiLiteLost(WIDGET *psWidget, W_CONTEXT *psContext)
 		sliderHiLiteLost((W_SLIDER *)psWidget);
 		break;
 	default:
-		ASSERT( false,"widgHiLiteLost: Unknown widget type" );
+		ASSERT(!"Unknown widget type", "Unknown widget type");
 		break;
 	}
 }
@@ -1685,7 +1686,7 @@ static void widgClicked(WIDGET *psWidget, UDWORD key, W_CONTEXT *psContext)
 		sliderClicked((W_SLIDER *)psWidget, psContext);
 		break;
 	default:
-		ASSERT( false,"widgClicked: Unknown widget type" );
+		ASSERT(!"Unknown widget type", "Unknown widget type");
 		break;
 	}
 }
@@ -1713,7 +1714,7 @@ static void widgReleased(WIDGET *psWidget, UDWORD key, W_CONTEXT *psContext)
 		sliderReleased((W_SLIDER *)psWidget);
 		break;
 	default:
-		ASSERT( false,"widgReleased: Unknown widget type" );
+		ASSERT(!"Unknown widget type", "Unknown widget type");
 		break;
 	}
 }
@@ -1741,7 +1742,7 @@ static void widgRun(WIDGET *psWidget, W_CONTEXT *psContext)
 		sliderRun((W_SLIDER *)psWidget, psContext);
 		break;
 	default:
-		ASSERT( false,"widgRun: Unknown widget type" );
+		ASSERT(!"Unknown widget type", "Unknown widget type");
 		break;
 	}
 }
