@@ -1,7 +1,7 @@
 /*
 	This file is part of Warzone 2100.
 	Copyright (C) 1999-2004  Eidos Interactive
-	Copyright (C) 2005-2007  Warzone Resurrection Project
+	Copyright (C) 2005-2009  Warzone Resurrection Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -43,6 +43,73 @@ static SDWORD targetAttackWeight(BASE_OBJECT *psTarget, BASE_OBJECT *psAttacker,
 // alliances
 UBYTE	alliances[MAX_PLAYERS][MAX_PLAYERS];
 
+
+// see if a structure has the range to fire on a target
+static BOOL aiStructHasRange(STRUCTURE *psStruct, BASE_OBJECT *psTarget, int weapon_slot)
+{
+	WEAPON_STATS		*psWStats;
+	SDWORD				xdiff,ydiff, longRange;
+
+	if (psStruct->numWeaps == 0 || psStruct->asWeaps[0].nStat == 0)
+	{
+		// Can't attack without a weapon
+		return false;
+	}
+
+	psWStats = psStruct->asWeaps[weapon_slot].nStat + asWeaponStats;
+
+	xdiff = (SDWORD)psStruct->pos.x - (SDWORD)psTarget->pos.x;
+	ydiff = (SDWORD)psStruct->pos.y - (SDWORD)psTarget->pos.y;
+	longRange = proj_GetLongRange(psWStats);
+	if (xdiff*xdiff + ydiff*ydiff < longRange*longRange)
+	{
+		// in range
+		return true;
+
+	}
+
+	return false;
+}
+
+static BOOL aiDroidHasRange(DROID *psDroid, BASE_OBJECT *psTarget, int weapon_slot)
+{
+	WEAPON_STATS		*psWStats;
+	SDWORD			xdiff, ydiff, longRange;
+
+	if (psDroid->numWeaps == 0 || psDroid->asWeaps[0].nStat == 0)
+	{
+		// Can't attack without a weapon
+		return false;
+	}
+
+	psWStats = psDroid->asWeaps[weapon_slot].nStat + asWeaponStats;
+
+	xdiff = (SDWORD)psDroid->pos.x - (SDWORD)psTarget->pos.x;
+	ydiff = (SDWORD)psDroid->pos.y - (SDWORD)psTarget->pos.y;
+	longRange = proj_GetLongRange(psWStats);
+	if (xdiff*xdiff + ydiff*ydiff < longRange*longRange)
+	{
+		// in range
+		return true;
+
+	}
+
+	return false;
+}
+
+static BOOL aiObjHasRange(BASE_OBJECT *psObj, BASE_OBJECT *psTarget, int weapon_slot)
+{
+	if (psObj->type == OBJ_DROID)
+	{
+		return aiDroidHasRange((DROID *)psObj, psTarget, weapon_slot);
+	}
+	else if (psObj->type == OBJ_STRUCTURE)
+	{
+		return aiStructHasRange((STRUCTURE *)psObj, psTarget, weapon_slot);
+	}
+	return false;
+}
+
 /* alliance code for ai. return true if an alliance has formed. */
 BOOL aiCheckAlliances(UDWORD s1,UDWORD s2)
 {
@@ -82,16 +149,76 @@ BOOL aiShutdown(void)
 	return true;
 }
 
+/** Search the global list of sensors for a possible target for psObj. */
+BASE_OBJECT *aiSearchSensorTargets(BASE_OBJECT *psObj, int weapon_slot, WEAPON_STATS *psWStats)
+{
+	int		longRange = proj_GetLongRange(psWStats);
+	int		tarDist = longRange * longRange;
+	int		minDist = psWStats->minRange * psWStats->minRange;
+	BASE_OBJECT	*psSensor, *psTarget = NULL;
+
+	for (psSensor = apsSensorList[0]; psSensor; psSensor = psSensor->psNextFunc)
+	{
+		BASE_OBJECT	*psTemp = NULL;
+		bool		isCB = false;
+
+		if (!aiCheckAlliances(psSensor->player, psObj->player))
+		{
+			continue;
+		}
+		else if (psSensor->type == OBJ_DROID)
+		{
+			DROID		*psDroid = (DROID *)psSensor;
+
+			ASSERT_OR_RETURN(false, psDroid->droidType == DROID_SENSOR, "A non-sensor droid in a sensor list is non-sense");
+			psTemp = psDroid->psTarget;
+			isCB = cbSensorDroid(psDroid);
+		}
+		else if (psSensor->type == OBJ_STRUCTURE)
+		{
+			STRUCTURE	*psCStruct = (STRUCTURE *)psSensor;
+
+			// skip incomplete structures
+			if (psCStruct->status != SS_BUILT)
+			{
+				continue;
+			}
+			psTemp = psCStruct->psTarget[0];
+			isCB = structCBSensor(psCStruct);
+		}
+		if (!psTemp || psTemp->died || !validTarget(psObj, psTemp, 0) || aiCheckAlliances(psTemp->player, psObj->player))
+		{
+			continue;
+		}
+		if (aiObjHasRange(psObj, psTemp, weapon_slot))
+		{
+			int distSq = objPosDiffSq(psTemp->pos, psObj->pos);
+
+			// Need to be in range, prefer closer targets or CB targets
+			if ((isCB || distSq < tarDist) && distSq > minDist)
+			{
+				tarDist = distSq;
+				psTarget = psTemp;
+				if (isCB)
+				{
+					break;	// got CB target, drop everything and shoot!
+				}
+			}
+		}
+	}
+	return psTarget;
+}
+
 // Find the best nearest target for a droid
 // Returns integer representing target priority, -1 if failed
 SDWORD aiBestNearestTarget(DROID *psDroid, BASE_OBJECT **ppsObj, int weapon_slot)
 {
 	UDWORD				i;
-	SDWORD				bestMod,newMod,failure=-1;
-	BASE_OBJECT			*psTarget,*friendlyObj,*bestTarget,*targetInQuestion,*tempTarget;
+	SDWORD				bestMod = 0,newMod, failure = -1;
+	BASE_OBJECT			*psTarget = NULL, *friendlyObj, *bestTarget = NULL, *targetInQuestion, *tempTarget;
 	BOOL				electronic = false;
 	STRUCTURE			*targetStructure;
-	WEAPON_EFFECT		weaponEffect;
+	WEAPON_EFFECT			weaponEffect;
 
 	//don't bother looking if empty vtol droid
 	if (vtolEmpty(psDroid))
@@ -100,23 +227,24 @@ SDWORD aiBestNearestTarget(DROID *psDroid, BASE_OBJECT **ppsObj, int weapon_slot
 	}
 
 	/* Return if have no weapons */
-	// Watermelon:added a protection against no weapon droid 'numWeaps'
 	// The ai orders a non-combat droid to patrol = crash without it...
 	if(psDroid->asWeaps[0].nStat == 0 || psDroid->numWeaps == 0)
 	{
 		return failure;
 	}
+	// Check if we have a CB target to begin with
+	if (!proj_Direct(asWeaponStats + psDroid->asWeaps[weapon_slot].nStat))
+	{
+		WEAPON_STATS	*psWStats = psWStats = psDroid->asWeaps[weapon_slot].nStat + asWeaponStats;
 
+		bestTarget = aiSearchSensorTargets((BASE_OBJECT *)psDroid, weapon_slot, psWStats);
+		bestMod = targetAttackWeight(bestTarget, (BASE_OBJECT *)psDroid, weapon_slot);
+	}
 	droidGetNaybors(psDroid);
 
-	//weaponMod = asWeaponModifier[weaponEffect][(asPropulsionStats + ((DROID*)psObj)->asBits[COMP_PROPULSION].nStat)->propulsionType];
 	weaponEffect = ((WEAPON_STATS *)(asWeaponStats + psDroid->asWeaps[weapon_slot].nStat))->weaponEffect;
 
-	//electronic warfare can only be used against structures at present - not any more! AB 6/11/98
 	electronic = electronicDroid(psDroid);
-	psTarget = NULL;
-	bestTarget = NULL;
-	bestMod = 0;
 	for (i=0; i< numNaybors; i++)
 	{
 		friendlyObj = NULL;
@@ -147,8 +275,7 @@ SDWORD aiBestNearestTarget(DROID *psDroid, BASE_OBJECT **ppsObj, int weapon_slot
 							if(friendlyDroid->order != DORDER_ATTACK)
 							{
 								// make sure target is near enough
-								if (dirtySqrt(psDroid->pos.x, psDroid->pos.y, tempTarget->pos.x, tempTarget->pos.y)
-								    < droidSensorRange(psDroid))
+								if (aiDroidHasRange(psDroid, tempTarget, weapon_slot))
 								{
 									targetInQuestion = tempTarget;		//consider this target
 								}
@@ -159,7 +286,7 @@ SDWORD aiBestNearestTarget(DROID *psDroid, BASE_OBJECT **ppsObj, int weapon_slot
 				else if(friendlyObj->type == OBJ_STRUCTURE)
 				{
 					tempTarget = ((STRUCTURE*)friendlyObj)->psTarget[0];
-					if (tempTarget && !tempTarget->died)
+					if (tempTarget && !tempTarget->died && aiDroidHasRange(psDroid, tempTarget, weapon_slot))
 					{
 						targetInQuestion = tempTarget;
 					}
@@ -167,20 +294,16 @@ SDWORD aiBestNearestTarget(DROID *psDroid, BASE_OBJECT **ppsObj, int weapon_slot
 			}
 		}
 
-		if (targetInQuestion != NULL &&
-			targetInQuestion != (BASE_OBJECT *)psDroid &&		//in case friendly unit had me as target
-			(targetInQuestion->type == OBJ_DROID ||
-			 targetInQuestion->type == OBJ_STRUCTURE) &&
-			 targetInQuestion->visible[psDroid->player] &&
-			 targetInQuestion->player != psDroid->player &&
-			 !aiCheckAlliances(targetInQuestion->player,psDroid->player))
+		if (targetInQuestion != NULL
+		    && targetInQuestion != (BASE_OBJECT *)psDroid		// in case friendly unit had me as target
+		    && (targetInQuestion->type == OBJ_DROID ||  targetInQuestion->type == OBJ_STRUCTURE)
+		    && targetInQuestion->visible[psDroid->player]
+		    && targetInQuestion->player != psDroid->player
+		    && !aiCheckAlliances(targetInQuestion->player,psDroid->player)
+		    && validTarget((BASE_OBJECT *)psDroid, targetInQuestion, weapon_slot)
+		    && aiDroidHasRange(psDroid, targetInQuestion, weapon_slot))
 		{
-
-			if ( !validTarget((BASE_OBJECT *)psDroid, targetInQuestion, weapon_slot) )
-			{
-				continue;
-			}
-			else if (targetInQuestion->type == OBJ_DROID)
+			if (targetInQuestion->type == OBJ_DROID)
 			{
 				// in multiPlayer - don't attack Transporters with EW
 				if (bMultiPlayer)
@@ -201,6 +324,8 @@ SDWORD aiBestNearestTarget(DROID *psDroid, BASE_OBJECT **ppsObj, int weapon_slot
 			}
 			else if (targetInQuestion->type == OBJ_STRUCTURE)
 			{
+				STRUCTURE *psStruct = (STRUCTURE *)targetInQuestion;
+
 				if (electronic)
 				{
 					/* don't want to target structures with resistance of zero if using electronic warfare */
@@ -209,17 +334,13 @@ SDWORD aiBestNearestTarget(DROID *psDroid, BASE_OBJECT **ppsObj, int weapon_slot
 						psTarget = targetInQuestion;
 					}
 				}
-				else if (((STRUCTURE *)targetInQuestion)->asWeaps[weapon_slot].nStat > 0)
+				else if (psStruct->asWeaps[weapon_slot].nStat > 0)
 				{
 					// structure with weapons - go for this
 					psTarget = targetInQuestion;
 				}
-				else if ( (  ((STRUCTURE *)targetInQuestion)->pStructureType->type != REF_WALL
-						   &&((STRUCTURE *)targetInQuestion)->pStructureType->type != REF_WALLCORNER
-						  )
-						 || driveModeActive()
-						 || (bMultiPlayer && game.type == SKIRMISH && !isHumanPlayer(psDroid->player))
-						)
+				else if ((psStruct->pStructureType->type != REF_WALL && psStruct->pStructureType->type != REF_WALLCORNER)
+				         || driveModeActive() || (bMultiPlayer && !isHumanPlayer(psDroid->player)))
 				{
 					psTarget = targetInQuestion;
 				}
@@ -245,9 +366,11 @@ SDWORD aiBestNearestTarget(DROID *psDroid, BASE_OBJECT **ppsObj, int weapon_slot
 	if (bestTarget)
 	{
 		ASSERT(!bestTarget->died, "aiBestNearestTarget: AI gave us a target that is already dead.");
+		targetStructure = visGetBlockingWall((BASE_OBJECT *)psDroid, bestTarget);
+
 		/* See if target is blocked by a wall; only affects direct weapons */
 		if (proj_Direct(asWeaponStats + psDroid->asWeaps[weapon_slot].nStat)
-		 && (targetStructure = visGetBlockingWall((BASE_OBJECT *)psDroid, bestTarget)))
+		 && targetStructure)
 		{
 			//are we any good against walls?
 			if(asStructStrengthModifier[weaponEffect][targetStructure->pStructureType->strength] >= 100)		//can attack atleast with default strength
@@ -403,7 +526,7 @@ static SDWORD targetAttackWeight(BASE_OBJECT *psTarget, BASE_OBJECT *psAttacker,
 
 		/* Now calculate the overall weight */
 		attackWeight = asWeaponModifier[weaponEffect][(asPropulsionStats + targetDroid->asBits[COMP_PROPULSION].nStat)->propulsionType] // Our weapon's effect against target
-				- WEIGHT_DIST_TILE_DROID * map_coord(dirtySqrt(psAttacker->pos.x, psAttacker->pos.y, targetDroid->pos.x, targetDroid->pos.y)) // farer droids are less attractive
+				- WEIGHT_DIST_TILE_DROID * map_coord(dirtyHypot(psAttacker->pos.x - targetDroid->pos.x, psAttacker->pos.y - targetDroid->pos.y)) // farer droids are less attractive
 				+ WEIGHT_HEALTH_DROID * damageRatio // we prefer damaged droids
 				+ targetTypeBonus; // some droid types have higher priority
 
@@ -444,7 +567,7 @@ static SDWORD targetAttackWeight(BASE_OBJECT *psTarget, BASE_OBJECT *psAttacker,
 
 		/* Now calculate the overall weight */
 		attackWeight = asStructStrengthModifier[weaponEffect][targetStructure->pStructureType->strength] // Our weapon's effect against target
-				- WEIGHT_DIST_TILE_STRUCT * map_coord(dirtySqrt(psAttacker->pos.x, psAttacker->pos.y, targetStructure->pos.x, targetStructure->pos.y)) // farer structs are less attractive
+				- WEIGHT_DIST_TILE_STRUCT * map_coord(dirtyHypot(psAttacker->pos.x - targetStructure->pos.x, psAttacker->pos.y - targetStructure->pos.y)) // farer structs are less attractive
 				+ WEIGHT_HEALTH_STRUCT * damageRatio // we prefer damaged structures
 				+ targetTypeBonus; // some structure types have higher priority
 
@@ -467,7 +590,7 @@ static SDWORD targetAttackWeight(BASE_OBJECT *psTarget, BASE_OBJECT *psAttacker,
 	}
 
 	/* We prefer objects we can see and can attack immediately */
-	if(!visibleObjWallBlock((BASE_OBJECT *)psAttacker, psTarget))
+	if(!visibleObject((BASE_OBJECT *)psAttacker, psTarget, true))
 	{
 		attackWeight /= WEIGHT_NOT_VISIBLE_F;
 	}
@@ -504,34 +627,6 @@ static SDWORD targetAttackWeight(BASE_OBJECT *psTarget, BASE_OBJECT *psAttacker,
 }
 
 
-// see if a structure has the range to fire on a target
-static BOOL aiStructHasRange(STRUCTURE *psStruct, BASE_OBJECT *psTarget, int weapon_slot)
-{
-	WEAPON_STATS		*psWStats;
-	SDWORD				xdiff,ydiff, longRange;
-
-	if (psStruct->numWeaps == 0 || psStruct->asWeaps[0].nStat == 0)
-	{
-		// Can't attack without a weapon
-		return false;
-	}
-
-	psWStats = psStruct->asWeaps[weapon_slot].nStat + asWeaponStats;
-
-	xdiff = (SDWORD)psStruct->pos.x - (SDWORD)psTarget->pos.x;
-	ydiff = (SDWORD)psStruct->pos.y - (SDWORD)psTarget->pos.y;
-	longRange = proj_GetLongRange(psWStats);
-	if (xdiff*xdiff + ydiff*ydiff < longRange*longRange)
-	{
-		// in range
-		return true;
-
-	}
-
-	return false;
-}
-
-
 // see if an object is a wall
 static BOOL aiObjIsWall(BASE_OBJECT *psObj)
 {
@@ -553,16 +648,9 @@ static BOOL aiObjIsWall(BASE_OBJECT *psObj)
 /* See if there is a target in range */
 BOOL aiChooseTarget(BASE_OBJECT *psObj, BASE_OBJECT **ppsTarget, int weapon_slot, BOOL bUpdateTarget)
 {
-	UDWORD	radSquared;
 	BASE_OBJECT		*psTarget = NULL;
-	SDWORD			xdiff,ydiff, distSq, tarDist, minDist;//, longRange;
-	BOOL			bCBTower;
-	STRUCTURE		*psCStruct;
 	DROID			*psCommander;
-	BOOL			bCommanderBlock;
-	UDWORD			sensorRange = objSensorRange(psObj);
-	SECONDARY_STATE	state;
-	SDWORD			curTargetWeight=-1,newTargetWeight;
+	SDWORD			curTargetWeight=-1;
 
 	/* Get the sensor range */
 	switch (psObj->type)
@@ -579,7 +667,6 @@ BOOL aiChooseTarget(BASE_OBJECT *psObj, BASE_OBJECT **ppsTarget, int weapon_slot
 			// Can't attack without a weapon
 			return false;
 		}
-		radSquared = sensorRange * sensorRange;
 		break;
 	case OBJ_STRUCTURE:
 		if (((STRUCTURE *)psObj)->numWeaps == 0 || ((STRUCTURE *)psObj)->asWeaps[0].nStat == 0)
@@ -587,19 +674,8 @@ BOOL aiChooseTarget(BASE_OBJECT *psObj, BASE_OBJECT **ppsTarget, int weapon_slot
 			// Can't attack without a weapon
 			return false;
 		}
-
-		// increase the sensor range for AA sites
-		// AA sites are defensive structures that can only shoot in the air
-		if ( (((STRUCTURE *)psObj)->pStructureType->type == REF_DEFENSE) &&
-			 (asWeaponStats[((STRUCTURE *)psObj)->asWeaps[weapon_slot].nStat].surfaceToAir == SHOOT_IN_AIR) )
-		{
-			sensorRange = 3 * sensorRange / 2;
-		}
-
-		radSquared = sensorRange*sensorRange;
 		break;
 	default:
-		radSquared = 0;
 		break;
 	}
 
@@ -609,7 +685,7 @@ BOOL aiChooseTarget(BASE_OBJECT *psObj, BASE_OBJECT **ppsTarget, int weapon_slot
 		BASE_OBJECT *psCurrTarget = ((DROID *)psObj)->psActionTarget[0];
 
 		/* find a new target */
-		newTargetWeight = aiBestNearestTarget((DROID *)psObj, &psTarget, weapon_slot);
+		int newTargetWeight = aiBestNearestTarget((DROID *)psObj, &psTarget, weapon_slot);
 
 		/* Calculate weight of the current target if updating; but take care not to target
 		 * ourselves... */
@@ -618,42 +694,32 @@ BOOL aiChooseTarget(BASE_OBJECT *psObj, BASE_OBJECT **ppsTarget, int weapon_slot
 			curTargetWeight = targetAttackWeight(psCurrTarget, psObj, weapon_slot);
 		}
 
-		if (newTargetWeight >= 0 &&		//found a new target
-			(!bUpdateTarget ||			//choosing a new target, don't care if current one is better
-			(curTargetWeight <= 0) ||	//attacker had no valid target, use new one
-			(newTargetWeight > (curTargetWeight + OLD_TARGET_THRESHOLD))	//updating and new target is better
-			))
+		if (newTargetWeight >= 0		// found a new target
+		    && (!bUpdateTarget			// choosing a new target, don't care if current one is better
+		        || curTargetWeight <= 0		// attacker had no valid target, use new one
+			|| newTargetWeight > curTargetWeight + OLD_TARGET_THRESHOLD)	// updating and new target is better
+		    && validTarget(psObj, psTarget, weapon_slot)
+		    && (aiDroidHasRange((DROID *)psObj, psTarget, weapon_slot) || (secondaryGetState((DROID *)psObj, DSO_HALTTYPE) != DSS_HALT_HOLD)))
 		{
-			/*check its a valid target*/
-			if (validTarget(psObj, psTarget, weapon_slot))
-			{
-    			/* See if in sensor range */
-			    xdiff = psTarget->pos.x - psObj->pos.x;
-			    ydiff = psTarget->pos.y - psObj->pos.y;
-			    if ((xdiff*xdiff + ydiff*ydiff < (SDWORD)radSquared) ||			//target is within our sensor range
-					(secondaryGetState((DROID *)psObj, DSO_HALTTYPE, &state) &&	//in case we got this target from a friendly unit see if can pursue it
-					(state != DSS_HALT_HOLD)))									//make sure it's guard or pursue
-				{
-					ASSERT(!isDead(psTarget), "aiChooseTarget: Droid found a dead target!");
-				    *ppsTarget = psTarget;
-				    return true;
-				}
-			}
+			ASSERT(!isDead(psTarget), "aiChooseTarget: Droid found a dead target!");
+			*ppsTarget = psTarget;
+			return true;
 		}
 	}
 	else if (psObj->type == OBJ_STRUCTURE)
 	{
-		WEAPON_STATS	*psWStats;
+		WEAPON_STATS	*psWStats = NULL;
+		int	tarDist, longRange = 0;
+		BOOL	bCommanderBlock = false;
 
-		ASSERT( ((STRUCTURE *)psObj)->asWeaps[weapon_slot].nStat > 0,
-			"aiChooseTarget: no weapons on structure" );
+		ASSERT(((STRUCTURE *)psObj)->asWeaps[weapon_slot].nStat > 0, "no weapons on structure");
 
 		psWStats = ((STRUCTURE *)psObj)->asWeaps[weapon_slot].nStat + asWeaponStats;
+		longRange = proj_GetLongRange(psWStats);
 
 		// see if there is a target from the command droids
 		psTarget = NULL;
 		psCommander = cmdDroidGetDesignator(psObj->player);
-		bCommanderBlock = false;
 		if (!proj_Direct(psWStats) && (psCommander != NULL) &&
 			aiStructHasRange((STRUCTURE *)psObj, (BASE_OBJECT *)psCommander, weapon_slot))
 		{
@@ -680,67 +746,13 @@ BOOL aiChooseTarget(BASE_OBJECT *psObj, BASE_OBJECT **ppsTarget, int weapon_slot
 		}
 
 		// indirect fire structures use sensor towers first
-		tarDist = SDWORD_MAX;
-		minDist = psWStats->minRange * psWStats->minRange;
-		bCBTower = false;
-		if (psTarget == NULL &&
-			!bCommanderBlock &&
-			!proj_Direct(psWStats))
+		tarDist = longRange * longRange;
+		if (psTarget == NULL && !bCommanderBlock && !proj_Direct(psWStats))
 		{
-			for(psCStruct=apsStructLists[psObj->player]; psCStruct; psCStruct=psCStruct->psNext)
-			{
-				// skip incomplete structures
-				if (psCStruct->status != SS_BUILT)
-				{
-					continue;
-				}
-
-				if (!bCBTower
-				    && structStandardSensor(psCStruct)
-				    && psCStruct->psTarget[0] != NULL
-				    && !psCStruct->psTarget[0]->died)
-				{
-					/*check its a valid target*/
-					//Watermelon:Greater than 1 for now
-					if ( validTarget(psObj, psCStruct->psTarget[0], 0) &&
-						aiStructHasRange((STRUCTURE *)psObj, psCStruct->psTarget[0], weapon_slot))
-					{
-					    xdiff = (SDWORD)psCStruct->psTarget[0]->pos.x - (SDWORD)psObj->pos.x;
-					    ydiff = (SDWORD)psCStruct->psTarget[0]->pos.y - (SDWORD)psObj->pos.y;
-					    distSq = xdiff*xdiff + ydiff*ydiff;
-					    if ((distSq < tarDist) &&
-						    (distSq > minDist))
-					    {
-						    tarDist = distSq;
-						    psTarget = psCStruct->psTarget[0];
-					    }
-					}
-				}
-				else if (structCBSensor(psCStruct)
-				         && psCStruct->psTarget[0] != NULL
-				         && !psCStruct->psTarget[0]->died)
-				{
-					/*check its a valid target*/
-					if ( validTarget(psObj, psCStruct->psTarget[0], 0) &&
-						aiStructHasRange((STRUCTURE *)psObj, psCStruct->psTarget[0], weapon_slot))
-					{
-						xdiff = (SDWORD)psCStruct->psTarget[0]->pos.x - (SDWORD)psObj->pos.x;
-						ydiff = (SDWORD)psCStruct->psTarget[0]->pos.y - (SDWORD)psObj->pos.y;
-						distSq = xdiff*xdiff + ydiff*ydiff;
-					    if ((!bCBTower || (distSq < tarDist)) &&
-						    (distSq > minDist))
-					    {
-						    tarDist = distSq;
-						    psTarget = psCStruct->psTarget[0];
-						    bCBTower = true;
-					    }
-					}
-				}
-			}
+			psTarget = aiSearchSensorTargets(psObj, weapon_slot, psWStats);
 		}
 
-		if ((psTarget == NULL) &&
-			!bCommanderBlock)
+		if (psTarget == NULL && !bCommanderBlock)
 		{
 			BASE_OBJECT *psCurr;
 
@@ -748,29 +760,20 @@ BOOL aiChooseTarget(BASE_OBJECT *psObj, BASE_OBJECT **ppsTarget, int weapon_slot
 			psCurr = gridIterate();
 			while (psCurr != NULL)
 			{
-				//don't target features
-				if (psCurr->type != OBJ_FEATURE && !psCurr->died)
+				/* Check that it is a valid target */
+				if (psCurr->type != OBJ_FEATURE && !psCurr->died && aiStructHasRange((STRUCTURE *)psObj, psCurr, weapon_slot)
+				    && psObj->player != psCurr->player && !aiCheckAlliances(psCurr->player, psObj->player)
+				    && validTarget(psObj, psCurr, weapon_slot) && psCurr->visible[psObj->player])
 				{
-					if (psObj->player != psCurr->player &&
-						!aiCheckAlliances(psCurr->player,psObj->player))
+					// See if in sensor range and visible
+					int distSq = objPosDiffSq(psCurr->pos, psObj->pos);
+
+					if (distSq < tarDist
+					    || (psTarget && psTarget->type == OBJ_STRUCTURE && ((STRUCTURE *)psTarget)->status != SS_BUILT)
+					    || (psTarget && aiObjIsWall(psTarget) && !aiObjIsWall(psCurr)))
 					{
-						/*check its a valid target*/
-						//Watermelon:Greater than 1 for now
-						if ( validTarget(psObj, psCurr, weapon_slot) &&
-							!aiObjIsWall(psCurr))
-						{
-						    // See if in sensor range and visible
-						    xdiff = psCurr->pos.x - psObj->pos.x;
-						    ydiff = psCurr->pos.y - psObj->pos.y;
-						    distSq = xdiff*xdiff + ydiff*ydiff;
-						    if (distSq < (SDWORD)radSquared &&
-							    psCurr->visible[psObj->player] &&
-							    distSq < tarDist)
-						    {
-							    psTarget = psCurr;
-							    tarDist = distSq;
-						    }
-						}
+						psTarget = psCurr;
+						tarDist = distSq;
 					}
 				}
 				psCurr = gridIterate();
@@ -792,81 +795,105 @@ BOOL aiChooseTarget(BASE_OBJECT *psObj, BASE_OBJECT **ppsTarget, int weapon_slot
 /* See if there is a target in range for Sensor objects*/
 BOOL aiChooseSensorTarget(BASE_OBJECT *psObj, BASE_OBJECT **ppsTarget)
 {
-	SDWORD	sensorRange = objSensorRange(psObj);
-	UDWORD	radSquared;
-	BASE_OBJECT		*psCurr,*psTemp = NULL;
-	BASE_OBJECT		*psTarget = NULL;
-	SDWORD	xdiff,ydiff, distSq, tarDist;
+	int		sensorRange = objSensorRange(psObj);
+	unsigned int	radSquared = sensorRange * sensorRange;
+	bool		radarDetector = objRadarDetector(psObj);
 
-	/* Get the sensor range */
-	switch (psObj->type)
+	if (!objActiveRadar(psObj) && !radarDetector)
 	{
-	case OBJ_DROID:
-		if (asSensorStats[((DROID *)psObj)->asBits[COMP_SENSOR].nStat].
-			location != LOC_TURRET)
-		{
-			// to be used for Turret Sensors only
-			return false;
-		}
-		radSquared = sensorRange * sensorRange;
-		break;
-	case OBJ_STRUCTURE:
-		if (!(structStandardSensor((STRUCTURE *)psObj) ||
-			structVTOLSensor((STRUCTURE *)psObj)))
-		{
-			// to be used for Standard and VTOL intercept Turret Sensors only
-			return false;
-		}
-		radSquared = sensorRange * sensorRange;
-		break;
-	default:
-		radSquared = 0;
-		break;
+		ASSERT(false, "Only to be used for sensor turrets!");
+		return false;
 	}
 
-	/* See if there is a something in range */
-	if (psObj->type == OBJ_DROID && CAN_UPDATE_NAYBORS( (DROID *)psObj ))
+	/* See if there is something in range */
+	if (radarDetector)
 	{
+		BASE_OBJECT	*psCurr, *psTemp = NULL;
+		int		tarDist = SDWORD_MAX;
+
+		gridStartIterate((SDWORD)psObj->pos.x, (SDWORD)psObj->pos.y);
+		psCurr = gridIterate();
+		while (psCurr != NULL)
+		{
+			if (psCurr->type == OBJ_STRUCTURE || psCurr->type == OBJ_DROID)
+			{
+				if (psObj->player != psCurr->player
+				    && !aiCheckAlliances(psCurr->player,psObj->player)
+				    && objActiveRadar(psCurr))
+				{
+					// See if in twice *their* sensor range
+					const int xdiff = psCurr->pos.x - psObj->pos.x;
+					const int ydiff = psCurr->pos.y - psObj->pos.y;
+					const unsigned int distSq = xdiff * xdiff + ydiff * ydiff;
+					const int targetSensor = objSensorRange(psCurr) * 2;
+					const unsigned int sensorSquared = targetSensor * targetSensor;
+
+					if (distSq < sensorSquared && distSq < tarDist)
+					{
+						psTemp = psCurr;
+						tarDist = distSq;
+					}
+				}
+			}
+			psCurr = gridIterate();
+		}
+
+		if (psTemp)
+		{
+			objTrace(psTemp->id, "Targetted by radar detector %d", (int)psObj->id);
+			objTrace(psObj->id, "Targetting radar %d", (int)psTemp->id);
+			ASSERT(!psTemp->died, "aiChooseSensorTarget gave us a dead target");
+			*ppsTarget = psTemp;
+			return true;
+		}
+	}
+	else if (psObj->type == OBJ_DROID && CAN_UPDATE_NAYBORS((DROID *)psObj))
+	{
+		BASE_OBJECT	*psTarget = NULL;
+
 		if (aiBestNearestTarget((DROID *)psObj, &psTarget, 0) >= 0)
 		{
 			/* See if in sensor range */
-			xdiff = psTarget->pos.x - psObj->pos.x;
-			ydiff = psTarget->pos.y - psObj->pos.y;
-			if (xdiff*xdiff + ydiff*ydiff < (SDWORD)radSquared)
+			const int xdiff = psTarget->pos.x - psObj->pos.x;
+			const int ydiff = psTarget->pos.y - psObj->pos.y;
+			const unsigned int distSq = xdiff * xdiff + ydiff * ydiff;
+
+			if (distSq < radSquared)
 			{
 				*ppsTarget = psTarget;
 				return true;
 			}
 		}
 	}
-	else
+	else	// structure
 	{
-		tarDist = SDWORD_MAX;
+		BASE_OBJECT	*psCurr, *psTemp = NULL;
+		int		tarDist = SDWORD_MAX;
+
 		gridStartIterate((SDWORD)psObj->pos.x, (SDWORD)psObj->pos.y);
 		psCurr = gridIterate();
 		while (psCurr != NULL)
 		{
-			    //don't target features
-			    if (psCurr->type != OBJ_FEATURE && !psCurr->died)
-			    {
-				    if (psObj->player != psCurr->player &&
-					    !aiCheckAlliances(psCurr->player,psObj->player) &&
-					    !aiObjIsWall(psCurr))
-				    {
-					    // See if in sensor range and visible
-					    xdiff = psCurr->pos.x - psObj->pos.x;
-					    ydiff = psCurr->pos.y - psObj->pos.y;
-					    distSq = xdiff*xdiff + ydiff*ydiff;
-					    if (distSq < (SDWORD)radSquared &&
-						    psCurr->visible[psObj->player] &&
-						    distSq < tarDist)
-					    {
-						    psTemp = psCurr;
-						    tarDist = distSq;
-					    }
-				    }
-			    }
-			    psCurr = gridIterate();
+			// Don't target features or dead objects
+			if (psCurr->type != OBJ_FEATURE && !psCurr->died)
+			{
+				if (psObj->player != psCurr->player &&
+				    !aiCheckAlliances(psCurr->player,psObj->player) &&
+				    !aiObjIsWall(psCurr))
+				{
+					// See if in sensor range and visible
+					const int xdiff = psCurr->pos.x - psObj->pos.x;
+					const int ydiff = psCurr->pos.y - psObj->pos.y;
+					const unsigned int distSq = xdiff * xdiff + ydiff * ydiff;
+
+					if (distSq < radSquared && psCurr->visible[psObj->player] && distSq < tarDist)
+					{
+						psTemp = psCurr;
+						tarDist = distSq;
+					}
+				}
+			}
+			psCurr = gridIterate();
 		}
 
 		if (psTemp)
@@ -884,7 +911,6 @@ BOOL aiChooseSensorTarget(BASE_OBJECT *psObj, BASE_OBJECT **ppsTarget)
 void aiUpdateDroid(DROID *psDroid)
 {
 	BASE_OBJECT	*psTarget;
-	SECONDARY_STATE	state;
 	BOOL		lookForTarget,updateTarget;
 
 	ASSERT(psDroid != NULL, "Invalid droid pointer");
@@ -896,7 +922,7 @@ void aiUpdateDroid(DROID *psDroid)
 	// HACK: we always want to update orders when NOT running a MP game,
 	// and we don't want to update when the droid belongs to another human player
 	if (!myResponsibility(psDroid->player) && bMultiPlayer
-		&& isHumanPlayer(psDroid->player))
+		  && isHumanPlayer(psDroid->player))
 	{
 		return;		// we should not order this droid around
 	}
@@ -956,8 +982,7 @@ void aiUpdateDroid(DROID *psDroid)
 	if ((psDroid->order == DORDER_NONE) &&
 		(psDroid->player == selectedPlayer) &&
 		!isVtolDroid(psDroid) &&
-		secondaryGetState(psDroid, DSO_HALTTYPE, &state) &&
-		(state == DSS_HALT_GUARD))
+		secondaryGetState(psDroid, DSO_HALTTYPE) == DSS_HALT_GUARD)
 	{
 		lookForTarget = false;
 		updateTarget = false;
@@ -991,12 +1016,9 @@ void aiUpdateDroid(DROID *psDroid)
 	}
 
 	// do not attack if the attack level is wrong
-	if (secondaryGetState(psDroid, DSO_ATTACK_LEVEL, &state))
+	if (secondaryGetState(psDroid, DSO_ATTACK_LEVEL) != DSS_ALEV_ALWAYS)
 	{
-		if (state != DSS_ALEV_ALWAYS)
-		{
-			lookForTarget = false;
-		}
+		lookForTarget = false;
 	}
 
 	/* Don't rebuild 'Naybor' list too often */
@@ -1014,7 +1036,7 @@ void aiUpdateDroid(DROID *psDroid)
 			if((psDroid->id % TARGET_UPD_SKIP_FRAMES) ==
 				(frameGetFrameNumber() % TARGET_UPD_SKIP_FRAMES))
 			{
-				int i;
+				unsigned int i;
 
 				(void)updateAttackTarget((BASE_OBJECT*)psDroid, 0); // this function always has to be called on weapon-slot 0 (even if ->numWeaps == 0)
 
@@ -1033,8 +1055,7 @@ void aiUpdateDroid(DROID *psDroid)
 	{
 		if (psDroid->droidType == DROID_SENSOR)
 		{
-			//Watermelon:only 1 target for sensor droid
-			if ( aiChooseTarget((BASE_OBJECT *)psDroid, &psTarget, 0, true) )
+			if (aiChooseSensorTarget((BASE_OBJECT *)psDroid, &psTarget))
 			{
 				orderDroidObj(psDroid, DORDER_OBSERVE, psTarget);
 			}
@@ -1049,9 +1070,7 @@ void aiUpdateDroid(DROID *psDroid)
 	}
 }
 
-/*set of rules which determine whether the weapon associated with the object
-can fire on the propulsion type of the target*/
-//Watermelon:added weapon_slot
+/* Set of rules which determine whether the weapon associated with the object can fire on the propulsion type of the target. */
 BOOL validTarget(BASE_OBJECT *psObject, BASE_OBJECT *psTarget, int weapon_slot)
 {
 	BOOL	bTargetInAir, bValidTarget = false;
@@ -1062,45 +1081,45 @@ BOOL validTarget(BASE_OBJECT *psObject, BASE_OBJECT *psTarget, int weapon_slot)
 		return false;
 	}
 
-	//need to check propulsion type of target
+	// Need to check propulsion type of target
 	switch (psTarget->type)
 	{
 	case OBJ_DROID:
-        if (asPropulsionTypes[asPropulsionStats[((DROID *)psTarget)->asBits[
-            COMP_PROPULSION].nStat].propulsionType].travel == AIR)
-        {
+		if (asPropulsionTypes[asPropulsionStats[((DROID *)psTarget)->asBits[
+		                                            COMP_PROPULSION].nStat].propulsionType].travel == AIR)
+		{
 			if (((DROID *)psTarget)->sMove.Status != MOVEINACTIVE)
 			{
-	            bTargetInAir = true;
+				bTargetInAir = true;
 			}
 			else
 			{
-	            bTargetInAir = false;
+				bTargetInAir = false;
 			}
-        }
-        else
-        {
-            bTargetInAir = false;
-        }
-        break;
-    case OBJ_STRUCTURE:
+		}
+		else
+		{
+			bTargetInAir = false;
+		}
+		break;
+	case OBJ_STRUCTURE:
 	default:
-        //lets hope so!
-        bTargetInAir = false;
-        break;
-    }
+		//lets hope so!
+		bTargetInAir = false;
+		break;
+	}
 
 	//need what can shoot at
 	switch (psObject->type)
 	{
 	case OBJ_DROID:
-    	// Can't attack without a weapon
+		// Can't attack without a weapon
 		//Watermelon:re-enabled if (((DROID *)psObject)->numWeaps != 0) to prevent crash
-		if ( ((DROID *)psObject)->numWeaps != 0 &&
-			((DROID *)psObject)->asWeaps[weapon_slot].nStat != 0 )
+		if (((DROID *)psObject)->numWeaps != 0 &&
+		        ((DROID *)psObject)->asWeaps[weapon_slot].nStat != 0)
 		{
 			surfaceToAir = asWeaponStats[((DROID *)psObject)->asWeaps[weapon_slot].nStat].surfaceToAir;
-			if ( ((surfaceToAir & SHOOT_IN_AIR) && bTargetInAir) || ((surfaceToAir & SHOOT_ON_GROUND) && !bTargetInAir) )
+			if (((surfaceToAir & SHOOT_IN_AIR) && bTargetInAir) || ((surfaceToAir & SHOOT_ON_GROUND) && !bTargetInAir))
 			{
 				return true;
 			}
@@ -1110,10 +1129,10 @@ BOOL validTarget(BASE_OBJECT *psObject, BASE_OBJECT *psTarget, int weapon_slot)
 			return false;
 		}
 		/*
-        if (((DROID *)psObject)->asWeaps[0].nStat != 0 && ((DROID *)psObject)->numWeaps > 0)
+		if (((DROID *)psObject)->asWeaps[0].nStat != 0 && ((DROID *)psObject)->numWeaps > 0)
 		{
-            surfaceToAir = asWeaponStats[((DROID *)psObject)->asWeaps[0].nStat].surfaceToAir;
-        }
+		    surfaceToAir = asWeaponStats[((DROID *)psObject)->asWeaps[0].nStat].surfaceToAir;
+		}
 		else
 		{
 			 surfaceToAir = 0;
@@ -1123,8 +1142,8 @@ BOOL validTarget(BASE_OBJECT *psObject, BASE_OBJECT *psTarget, int weapon_slot)
 	case OBJ_STRUCTURE:
 		// Can't attack without a weapon
 		//Watermelon:re-enabled if (((DROID *)psObject)->numWeaps != 0) to prevent crash
-		if ( ((STRUCTURE *)psObject)->numWeaps != 0 &&
-			((STRUCTURE *)psObject)->asWeaps[weapon_slot].nStat != 0 )
+		if (((STRUCTURE *)psObject)->numWeaps != 0 &&
+		        ((STRUCTURE *)psObject)->asWeaps[weapon_slot].nStat != 0)
 		{
 			surfaceToAir = asWeaponStats[((STRUCTURE *)psObject)->asWeaps[weapon_slot].nStat].surfaceToAir;
 		}
@@ -1133,29 +1152,29 @@ BOOL validTarget(BASE_OBJECT *psObject, BASE_OBJECT *psTarget, int weapon_slot)
 			surfaceToAir = 0;
 		}
 
-		if ( ((surfaceToAir & SHOOT_IN_AIR) && bTargetInAir) || ((surfaceToAir & SHOOT_ON_GROUND) && !bTargetInAir) )
+		if (((surfaceToAir & SHOOT_IN_AIR) && bTargetInAir) || ((surfaceToAir & SHOOT_ON_GROUND) && !bTargetInAir))
 		{
 			return true;
 		}
-			break;
-		default:
-			surfaceToAir = 0;
-			break;
-		}
+		break;
+	default:
+		surfaceToAir = 0;
+		break;
+	}
 
-    //if target is in the air and you can shoot in the air - OK
-    if (bTargetInAir && (surfaceToAir & SHOOT_IN_AIR))
-    {
-        bValidTarget = true;
-    }
+	//if target is in the air and you can shoot in the air - OK
+	if (bTargetInAir && (surfaceToAir & SHOOT_IN_AIR))
+	{
+		bValidTarget = true;
+	}
 
-    //if target is on the ground and can shoot at it - OK
-    if (!bTargetInAir && (surfaceToAir & SHOOT_ON_GROUND))
-    {
-        bValidTarget = true;
-    }
+	//if target is on the ground and can shoot at it - OK
+	if (!bTargetInAir && (surfaceToAir & SHOOT_ON_GROUND))
+	{
+		bValidTarget = true;
+	}
 
-    return bValidTarget;
+	return bValidTarget;
 }
 
 /* Make droid/structure look for a better target */
