@@ -1,7 +1,7 @@
 /*
 	This file is part of Warzone 2100.
 	Copyright (C) 1999-2004  Eidos Interactive
-	Copyright (C) 2005-2007  Warzone Resurrection Project
+	Copyright (C) 2005-2009  Warzone Resurrection Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -38,4 +38,10 @@ extern void setDefaultFrameRateLimit(void);
 /* Default frame limit for single player: skirmish ans campaign */
 #define	SP_FRAME_LIMIT	60
 
+/// Default map for Mayhem
+#define DEFAULTCAMPAIGNMAP	"Rush"
+/// Default map for Skirmish
+#define DEFAULTSKIRMISHMAP	"Sk-Rush"
+
+
 #endif // __INCLUDED_SRC_CONFIGURATION_H__
