@@ -1,7 +1,7 @@
 /*
 	This file is part of Warzone 2100.
 	Copyright (C) 1999-2004  Eidos Interactive
-	Copyright (C) 2005-2007  Warzone Resurrection Project
+	Copyright (C) 2005-2009  Warzone Resurrection Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -419,6 +419,10 @@ void setConsoleTextColor(SDWORD player)
 	{
 		iV_SetTextColour(WZCOL_CONS_TEXT_SYSTEM);
 	}
+	else if (player == NOTIFY_MESSAGE)
+	{
+		iV_SetTextColour(WZCOL_YELLOW);
+	}
 	else
 	{
 		// Don't use friend-foe colors in the lobby
@@ -446,13 +450,12 @@ void setConsoleTextColor(SDWORD player)
 void	displayConsoleMessages( void )
 {
 	CONSOLE_MESSAGE *psMessage;
-	int numProcessed;
 	int linePitch;
 	int boxDepth;
 	int drop;
 	int MesY;
 	int clipDepth;
-	int exceed;
+	unsigned int exceed, numProcessed;
 
 	/* Are there any to display? */
 	if(consoleMessages == NULL && !bConsoleDropped)
@@ -467,9 +470,6 @@ void	displayConsoleMessages( void )
 		return;
 	}
 
-	/* Haven't done any yet */
-	numProcessed = 0;
-
 	/* Get the travel to the next line */
 	linePitch = iV_GetTextLineSize();
 
@@ -489,13 +489,13 @@ void	displayConsoleMessages( void )
 	/* Do we want a box under it? */
 	if(bTextBoxActive)
 	{
-		for(psMessage = consoleMessages,exceed = 0;
-			psMessage && (numProcessed<consoleVisibleLines) && (exceed < 4); // ho ho ho!!!
-			psMessage = psMessage->psNext)
+		for (psMessage = consoleMessages, exceed = 0;
+		     psMessage && consoleVisibleLines > 0 && exceed < 4; // ho ho ho!!!
+		     psMessage = psMessage->psNext)
 		{
-			if((UDWORD)iV_GetTextWidth(psMessage->text) > mainConsole.width)
+			if (iV_GetTextWidth(psMessage->text) > mainConsole.width)
 			{
-				exceed++;
+				++exceed;
 			}
 		}
 
@@ -503,7 +503,7 @@ void	displayConsoleMessages( void )
 		boxDepth = (numActiveMessages> consoleVisibleLines ? consoleVisibleLines-1 : numActiveMessages-1);
 
 		/* Add on the extra - hope it doesn't exceed two lines! */
-		boxDepth+=exceed;
+		boxDepth += exceed;
 
 		/* GET RID OF THE MAGIC NUMBERS BELOW */
 		clipDepth = (mainConsole.topY+(boxDepth*linePitch)+CON_BORDER_HEIGHT+drop);
@@ -519,9 +519,9 @@ void	displayConsoleMessages( void )
 	/* Stop when we've drawn enough or we're at the end */
 	MesY = mainConsole.topY + drop;
 
-	for(psMessage = consoleMessages,numProcessed = 0;
-		psMessage && numProcessed<consoleVisibleLines && MesY < (pie_GetVideoBufferHeight()-linePitch);
-		psMessage = psMessage->psNext)
+	for (psMessage = consoleMessages, numProcessed = 0;
+	     psMessage && numProcessed < consoleVisibleLines && MesY < (pie_GetVideoBufferHeight() - linePitch);
+	     psMessage = psMessage->psNext)
 	{
 
 		/* Set text color depending on message type */
@@ -532,7 +532,7 @@ void	displayConsoleMessages( void )
 									mainConsole.width, psMessage->JustifyType);
 
 		/* Move on */
-		numProcessed++;
+		++numProcessed;
 	}
 }
 
@@ -540,18 +540,16 @@ void	displayConsoleMessages( void )
 	\return The number of messages actually shown */
 int displayOldMessages()
 {
-	int thisIndex;
 	int i;
-	int count;
 	BOOL bGotIt;
 	BOOL bQuit;
 	int marker = 0;
 	int linePitch;
 	int MesY;
+	unsigned int count = 0;
 
 	/* Check there actually are any messages */
-	thisIndex = messageId;
-	count = 0;
+	int thisIndex = messageId;
 
 	if(thisIndex)
 	{
@@ -560,7 +558,7 @@ int displayOldMessages()
 		{
 			for(i=0,bGotIt = false; i<MAX_CONSOLE_MESSAGES && !bGotIt; i++)
 			{
-				if(consoleStorage[i].id == thisIndex-1)
+				if (consoleStorage[i].id == thisIndex-1)
 				{
 					bGotIt = true;
 					marker = i;
