@@ -1,7 +1,7 @@
 /*
 	This file is part of Warzone 2100.
 	Copyright (C) 1999-2004  Eidos Interactive
-	Copyright (C) 2005-2007  Warzone Resurrection Project
+	Copyright (C) 2005-2009  Warzone Resurrection Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -90,7 +90,6 @@ void dataClearSaveFlag(void)
 	saveFlag = false;
 }
 
-
 /* Load the body stats */
 static BOOL bufferSBODYLoad(const char *pBuffer, UDWORD size, void **ppData)
 {
@@ -575,6 +574,19 @@ static BOOL bufferSMSGLoad(const char *pBuffer, UDWORD size, void **ppData)
 	return true;
 }
 
+/* Load research message viewdata */
+static BOOL dataResearchMsgLoad(const char* fileName, void** ppData)
+{
+	VIEWDATA* pViewData = loadResearchViewData(fileName);
+	if (!pViewData)
+	{
+		return false;
+	}
+
+	// set the pointer so the release function gets called with it
+	*ppData = pViewData;
+	return true;
+}
 
 // release the message viewdata
 static void dataSMSGRelease(void *pData)
@@ -591,7 +603,6 @@ static BOOL dataIMDBufferLoad(const char *pBuffer, UDWORD size, void **ppData)
 	psIMD = iV_ProcessIMD( &pBufferPosition, pBufferPosition + size );
 	if (psIMD == NULL) {
 		debug( LOG_ERROR, "IMD load failed - %s", GetLastResourceFilename() );
-		abort();
 		return false;
 	}
 
@@ -626,7 +637,10 @@ static BOOL dataImageLoad(const char *fileName, void **ppData)
 // Tertiles (terrain tiles) loader.
 static BOOL dataTERTILESLoad(const char *fileName, void **ppData)
 {
-	texLoad(fileName);
+	bool status;
+
+	status = texLoad(fileName);
+	ASSERT_OR_RETURN(false, status, "Error loading tertiles!");
 	debug(LOG_TEXTURE, "HW Tiles loaded");
 
 	*ppData = NULL;	// don't bother calling cleanup
@@ -676,13 +690,13 @@ static BOOL dataTexPageLoad(const char *fileName, void **ppData)
 	{
 		// replace the old texture page with the new one
 		debug(LOG_TEXTURE, "replacing %s with new texture %s", texpage, fileName);
-		(void) pie_ReplaceTexPage(*ppData, texpage);
+		(void) pie_ReplaceTexPage(*ppData, texpage, getTextureSize());
 	}
 	else
 	{
 		debug(LOG_TEXTURE, "adding page %s with texture %s", texpage, fileName);
 		SetLastResourceFilename(texpage);
-		(void) pie_AddTexPage(*ppData, texpage, 0);
+		(void) pie_AddTexPage(*ppData, texpage, 0, getTextureSize());
 	}
 
 	return true;
@@ -730,7 +744,7 @@ static BOOL dataAudioCfgLoad(const char* fileName, void **ppData)
 	{
 		return true;
 	}
-
+	debug(LOG_WZ, "Reading...[directory: %s] %s", PHYSFS_getRealDir(fileName), fileName);
 	fileHandle = PHYSFS_openRead(fileName);
 
 	if (fileHandle == NULL)
@@ -749,6 +763,7 @@ static BOOL dataAudioCfgLoad(const char* fileName, void **ppData)
 static BOOL dataAnimLoad(const char *fileName, void **ppData)
 {
 	PHYSFS_file* fileHandle = PHYSFS_openRead(fileName);
+	debug(LOG_WZ, "Reading...[directory: %s] %s", PHYSFS_getRealDir(fileName), fileName);
 	if (fileHandle == NULL)
 	{
 		*ppData = NULL;
@@ -769,6 +784,7 @@ static BOOL dataAnimCfgLoad(const char *fileName, void **ppData)
 	PHYSFS_file* fileHandle = PHYSFS_openRead(fileName);
 	*ppData = NULL;
 
+	debug(LOG_WZ, "Reading...[directory: %s] %s", PHYSFS_getRealDir(fileName), fileName);
 	if (fileHandle == NULL)
 	{
 		return false;
@@ -830,7 +846,7 @@ static BOOL dataScriptLoad(const char* fileName, void **ppData)
 	scr_lineno = 1;
 
 	fileHandle = PHYSFS_openRead(fileName);
-
+	debug(LOG_WZ, "Reading...[directory: %s] %s", PHYSFS_getRealDir(fileName), fileName);
 	if (fileHandle == NULL)
 	{
 		return false;
@@ -854,6 +870,14 @@ static BOOL dataScriptLoad(const char* fileName, void **ppData)
 	return true;
 }
 
+
+static void dataScriptRelease(void *pData)
+{
+	SCRIPT_CODE *psCode = pData;
+	scriptFreeCode(psCode);
+}
+
+
 // Load a script variable values file
 static BOOL dataScriptLoadVals(const char* fileName, void **ppData)
 {
@@ -871,7 +895,7 @@ static BOOL dataScriptLoadVals(const char* fileName, void **ppData)
 	debug(LOG_WZ, "Loading script data %s", GetLastResourceFilename());
 
 	fileHandle = PHYSFS_openRead(fileName);
-
+	debug(LOG_WZ, "Reading...[directory: %s] %s", PHYSFS_getRealDir(fileName), fileName);
 	if (fileHandle == NULL)
 	{
 		return false;
@@ -893,9 +917,9 @@ static BOOL dataScriptLoadVals(const char* fileName, void **ppData)
 // This basically matches the argument list of resAddBufferLoad in frameresource.c
 typedef struct
 {
-	const char *aType;                      // points to the string defining the type (e.g. SCRIPT) - NULL indicates end of list
-	RES_BUFFERLOAD buffLoad;                // routine to process the data for this type
-	RES_FREE release;                       // routine to release the data (NULL indicates none)
+	const char *aType;                      ///< points to the string defining the type (e.g. SCRIPT) - NULL indicates end of list
+	RES_BUFFERLOAD buffLoad;                ///< routine to process the data for this type
+	RES_FREE release;                       ///< routine to release the data (NULL indicates none)
 } RES_TYPE_MIN_BUF;
 
 static const RES_TYPE_MIN_BUF BufferResourceTypes[] =
@@ -937,9 +961,9 @@ static const RES_TYPE_MIN_BUF BufferResourceTypes[] =
 
 typedef struct
 {
-	const char *aType;                      // points to the string defining the type (e.g. SCRIPT) - NULL indicates end of list
-	RES_FILELOAD fileLoad;                  // routine to process the data for this type
-	RES_FREE release;                       // routine to release the data (NULL indicates none)
+	const char *aType;                      ///< points to the string defining the type (e.g. SCRIPT) - NULL indicates end of list
+	RES_FILELOAD fileLoad;                  ///< routine to process the data for this type
+	RES_FREE release;                       ///< routine to release the data (NULL indicates none)
 } RES_TYPE_MIN_FILE;
 
 static const RES_TYPE_MIN_FILE FileResourceTypes[] =
@@ -952,9 +976,10 @@ static const RES_TYPE_MIN_FILE FileResourceTypes[] =
 	{"TERTILES", dataTERTILESLoad, dataTERTILESRelease},
 	{"IMG", dataIMGLoad, dataIMGRelease},
 	{"TEXPAGE", dataTexPageLoad, dataImageRelease},
-	{"SCRIPT", dataScriptLoad, (RES_FREE)scriptFreeCode},
+	{"SCRIPT", dataScriptLoad, dataScriptRelease},
 	{"SCRIPTVAL", dataScriptLoadVals, NULL},
 	{"STR_RES", dataStrResLoad, dataStrResRelease},
+	{ "RESEARCHMSG", dataResearchMsgLoad, dataSMSGRelease },
 };
 
 /* Pass all the data loading functions to the framework library */
@@ -988,7 +1013,7 @@ BOOL dataInitLoadFuncs(void)
 		{
 			if(!resAddFileLoad(CurrentType->aType, CurrentType->fileLoad, CurrentType->release))
 			{
-				return false; // error whilst adding a buffer load
+				return false; // error whilst adding a file load
 			}
 		}
 	}
