@@ -1,7 +1,7 @@
 /*
 	This file is part of Warzone 2100.
 	Copyright (C) 1999-2004  Eidos Interactive
-	Copyright (C) 2005-2007  Warzone Resurrection Project
+	Copyright (C) 2005-2009  Warzone Resurrection Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -36,4 +36,8 @@
 #define	RADBRX		(RADTLX + RADWIDTH) -1
 #define	RADBRY		(RADTLY + RADHEIGHT) -1
 
+#define SKY_MULT	1
+#define SKY_SHIMMY_BASE	((DEG(1)*SKY_MULT)/2)
+#define SKY_SHIMMY (SKY_SHIMMY_BASE - (rand()%(2*SKY_SHIMMY_BASE)))
+
 #endif // __INCLUDED_SRC_DISPLAY3DDEF_H__
