@@ -1,7 +1,7 @@
 /*
 	This file is part of Warzone 2100.
 	Copyright (C) 1999-2004  Eidos Interactive
-	Copyright (C) 2005-2007  Warzone Resurrection Project
+	Copyright (C) 2005-2009  Warzone Resurrection Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -61,7 +61,6 @@
 #include "target.h"
 
 // all the bollox needed for script callbacks
-#include "lib/script/interp.h"				// needed to define types in scripttabs.h
 #include "lib/script/parse.h"				// needed to define types in scripttabs.h (Arse!)
 #include "scripttabs.h"			// needed to define the callback
 #include "scriptextern.h"		// needed to include the GLOBAL for checking bInTutorial
@@ -556,8 +555,8 @@ void driveUpdate(void)
 					if(	(psDroid->selected) &&
 						(psDroid != psDrivenDroid) &&
 						(psDroid->droidType != DROID_TRANSPORTER) &&
-						//((psPropStats->propulsionType != LIFT) || (psDroid->droidType == DROID_CYBORG)) ) {
-                        ((psPropStats->propulsionType != LIFT) || cyborgDroid(psDroid)) )
+						//((psPropStats->propulsionType != PROPULSION_TYPE_LIFT) || (psDroid->droidType == DROID_CYBORG)) ) {
+                        ((psPropStats->propulsionType != PROPULSION_TYPE_LIFT) || cyborgDroid(psDroid)) )
                     {
 						// Send new orders to it's followers.
 						driveMoveFollower(psDroid);
@@ -734,11 +733,11 @@ void driveTacticalSelectionChanged(void)
 //
 void driveProcessRadarInput(int x,int y)
 {
-	SDWORD PosX,PosY;
+	int PosX, PosY;
 
 	// when drive mode is active, clicking on the radar orders all selected droids
 	// to move to this position.
-	CalcRadarPosition(x,y,(UDWORD *)&PosX,(UDWORD *)&PosY);
+	CalcRadarPosition(x, y, &PosX, &PosY);
 	orderSelectedLoc(selectedPlayer, PosX*TILE_UNITS,PosY*TILE_UNITS);
 }
 /*
