@@ -1,7 +1,7 @@
 /*
 	This file is part of Warzone 2100.
 	Copyright (C) 1999-2004  Eidos Interactive
-	Copyright (C) 2005-2007  Warzone Resurrection Project
+	Copyright (C) 2005-2009  Warzone Resurrection Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -52,7 +52,6 @@
 
 #include "mapgrid.h"
 #include "display3d.h"
-#include "gateway.h"
 
 
 /* The statistics for the features */
@@ -78,9 +77,9 @@ struct featureTypeMap
 	FEATURE_TYPE type;
 };
 
-const static struct featureTypeMap map[] =
+static const struct featureTypeMap map[] =
 {
-	{ "HOVER WRECK", FEAT_HOVER },
+	{ "PROPULSION_TYPE_HOVER WRECK", FEAT_HOVER },
 	{ "TANK WRECK", FEAT_TANK },
 	{ "GENERIC ARTEFACT", FEAT_GEN_ARTE },
 	{ "OIL RESOURCE", FEAT_OIL_RESOURCE },
@@ -104,9 +103,9 @@ void featureInitVars(void)
 
 static void featureType(FEATURE_STATS* psFeature, const char *pType)
 {
-	int i;
+	unsigned int i;
 	
-	for (i = 0; i < sizeof(map) / sizeof(map[0]); i++)
+	for (i = 0; i < ARRAY_SIZE(map); i++)
 	{
 		if (strcmp(pType, map[i].typeStr) == 0)
 		{
@@ -141,6 +140,7 @@ BOOL loadFeatureStats(const char *pFeatureData, UDWORD bufferSize)
 	for (i = 0; i < numFeatureStats; i++)
 	{
 		UDWORD Width, Breadth;
+		int damageable = 0, tileDraw = 0, allowLOS = 0, visibleAtStart = 0;
 
 		memset(psFeature, 0, sizeof(FEATURE_STATS));
 
@@ -148,18 +148,25 @@ BOOL loadFeatureStats(const char *pFeatureData, UDWORD bufferSize)
 		GfxFile[0] = '\0';
 		type[0] = '\0';
 
+
 		//read the data into the storage - the data is delimeted using comma's
-		sscanf(pFeatureData,"%[^','],%d,%d,%d,%d,%d,%[^','],%[^','],%d,%d,%d",
+		sscanf(pFeatureData, "%[^','],%d,%d,%d,%d,%d,%[^','],%[^','],%d,%d,%d",
 			featureName, &Width, &Breadth,
-			&psFeature->damageable, &psFeature->armourValue, &psFeature->body,
-			GfxFile, type, &psFeature->tileDraw, &psFeature->allowLOS,
-			&psFeature->visibleAtStart);
+			&damageable, &psFeature->armourValue, &psFeature->body,
+			GfxFile, type, &tileDraw, &allowLOS,
+			&visibleAtStart);
+
+		psFeature->damageable = damageable;
+		psFeature->tileDraw = tileDraw;
+		psFeature->allowLOS = allowLOS;
+		psFeature->visibleAtStart = visibleAtStart;
 
 		// These are now only 16 bits wide - so we need to copy them
-		psFeature->baseWidth = (UWORD)Width;
-		psFeature->baseBreadth = (UWORD)Breadth;
+		psFeature->baseWidth = Width;
+		psFeature->baseBreadth = Breadth;
 
-		if (!allocateName(&psFeature->pName, featureName))
+		psFeature->pName = allocateName(featureName);
+		if (!psFeature->pName)
 		{
 			return false;
 		}
@@ -178,7 +185,6 @@ BOOL loadFeatureStats(const char *pFeatureData, UDWORD bufferSize)
 		if (psFeature->psImd == NULL)
 		{
 			debug( LOG_ERROR, "Cannot find the feature PIE for record %s",  getName( psFeature->pName ) );
-			abort();
 			return false;
 		}
 
@@ -214,7 +220,7 @@ float featureDamage(FEATURE *psFeature, UDWORD damage, UDWORD weaponClass, UDWOR
 {
 	float		relativeDamage;
 
-	ASSERT(psFeature != NULL, "featureDamage: Invalid feature pointer");
+	ASSERT_OR_RETURN(0.0, psFeature != NULL, "Invalid feature pointer");
 
 	debug(LOG_ATTACK, "featureDamage(%d): body %d armour %d damage: %d",
 	      psFeature->id, psFeature->body, psFeature->armour[impactSide][weaponClass], damage);
@@ -278,13 +284,6 @@ FEATURE * buildFeature(FEATURE_STATS *psStats, UDWORD x, UDWORD y,BOOL FromSave)
 	//return the average of max/min height
 	height = (foundationMin + foundationMax) / 2;
 
-	// check you can reach an oil resource
-	if ((psStats->subType == FEAT_OIL_RESOURCE) &&
-		!gwZoneReachable(gwGetZone(startX,startY)))
-	{
-		debug( LOG_NEVER, "Oil resource at (%d,%d) is unreachable", startX, startY );
-	}
-
 	if(FromSave == true) {
 		psFeature->pos.x = (UWORD)x;
 		psFeature->pos.y = (UWORD)y;
@@ -342,7 +341,7 @@ FEATURE * buildFeature(FEATURE_STATS *psStats, UDWORD x, UDWORD y,BOOL FromSave)
 	{
 		int j;
 
-		for (j = 0; j < NUM_WEAPON_CLASS; j++)
+		for (j = 0; j < WC_NUM_WEAPON_CLASSES; j++)
 		{
 			psFeature->armour[i][j] = psFeature->psStats->armourValue;
 		}
@@ -375,7 +374,7 @@ FEATURE * buildFeature(FEATURE_STATS *psStats, UDWORD x, UDWORD y,BOOL FromSave)
   	}
 
 
-	assert(psFeature->sDisplay.imd);		// make sure we have an imd.
+	ASSERT_OR_RETURN(NULL, psFeature->sDisplay.imd, "No IMD for feature");		// make sure we have an imd.
 
 	for (width = 0; width <= psStats->baseWidth; width++)
 	{
@@ -406,6 +405,12 @@ FEATURE * buildFeature(FEATURE_STATS *psStats, UDWORD x, UDWORD y,BOOL FromSave)
 
 				psTile->psObject = (BASE_OBJECT*)psFeature;
 
+				// if it's a tall feature then flag it in the map.
+				if (psFeature->sDisplay.imd->max.y > TALLOBJECT_YMAX)
+				{
+					SET_TILE_TALLSTRUCTURE(psTile);
+				}
+
 				if (psStats->subType == FEAT_GEN_ARTE || psStats->subType == FEAT_OIL_DRUM || psStats->subType == FEAT_BUILD_WRECK)// they're there - just can see me
 				{
 					SET_TILE_NOTBLOCKING(psTile);
@@ -473,30 +478,20 @@ void featureUpdate(FEATURE *psFeat)
 
 
 // free up a feature with no visual effects
-void removeFeature(FEATURE *psDel)
+bool removeFeature(FEATURE *psDel)
 {
 	UDWORD		mapX, mapY, width,breadth, player;
 	MESSAGE		*psMessage;
 	Vector3i pos;
 
-	ASSERT( psDel != NULL,
-		"removeFeature: invalid feature pointer\n" );
-
-	if (psDel->died)
-	{
-		// feature has already been killed, quit
-		ASSERT( false,
-			"removeFeature: feature already dead" );
-		return;
-	}
-
+	ASSERT_OR_RETURN(false, psDel != NULL, "Invalid feature pointer");
+	ASSERT_OR_RETURN(false, !psDel->died, "Feature already dead");
 
 	if(bMultiPlayer && !ingame.localJoiningInProgress)
 	{
 		SendDestroyFeature(psDel);	// inform other players of destruction
 	}
 
-
 	//remove from the map data
 	mapX = map_coord(psDel->pos.x - psDel->psStats->baseWidth * TILE_UNITS / 2);
 	mapY = map_coord(psDel->pos.y - psDel->psStats->baseBreadth * TILE_UNITS / 2);
@@ -508,6 +503,7 @@ void removeFeature(FEATURE *psDel)
 
 			psTile->psObject = NULL;
 
+			CLEAR_TILE_TALLSTRUCTURE(psTile);
 			CLEAR_TILE_NOTBLOCKING(psTile);
 		}
 	}
@@ -542,10 +538,12 @@ void removeFeature(FEATURE *psDel)
 	gridRemoveObject((BASE_OBJECT *)psDel);
 
 	killFeature(psDel);
+
+	return true;
 }
 
 /* Remove a Feature and free it's memory */
-void destroyFeature(FEATURE *psDel)
+bool destroyFeature(FEATURE *psDel)
 {
 	UDWORD			widthScatter,breadthScatter,heightScatter, i;
 	EFFECT_TYPE		explosionSize;
@@ -554,8 +552,7 @@ void destroyFeature(FEATURE *psDel)
 	UDWORD			mapX,mapY;
 	UDWORD			texture;
 
-	ASSERT( psDel != NULL,
-		"destroyFeature: invalid feature pointer\n" );
+	ASSERT_OR_RETURN(false, psDel != NULL, "Invalid feature pointer");
 
  	/* Only add if visible and damageable*/
 	if(psDel->visible[selectedPlayer] && psDel->psStats->damageable)
@@ -654,6 +651,7 @@ void destroyFeature(FEATURE *psDel)
 //---------------------------------------------------------------------------------------
 
 	removeFeature(psDel);
+	return true;
 }
 
 
