@@ -1,7 +1,7 @@
 /*
 	This file is part of Warzone 2100.
 	Copyright (C) 1999-2004  Eidos Interactive
-	Copyright (C) 2005-2007  Warzone Resurrection Project
+	Copyright (C) 2005-2009  Warzone Resurrection Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@
  */
 #include "lib/framework/frame.h"
 #include "lib/framework/strres.h"
-#include "lib/framework/math-help.h"
+#include "lib/framework/math_ext.h"
 
 /* Includes direct access to render library */
 #include "lib/ivis_common/ivisdef.h"
@@ -135,6 +135,7 @@ STRUCTURE *droidGetCommandFactory(DROID *psDroid);
 static SDWORD ButtonDrawXOffset;
 static SDWORD ButtonDrawYOffset;
 
+static void DeleteButtonData(void);
 
 // Set audio IDs for form opening/closing anims.
 // Use -1 to dissable audio.
@@ -270,7 +271,7 @@ void intUpdateProgressBar(WIDGET *psWidget, W_CONTEXT *psContext)
 					BuildPoints = Range;
 				}
 				// prevent a division by 0 error
-				if (Range == 0 )
+				if (Range == 0)
 				{
 					Range = 1;
 				}
@@ -327,7 +328,7 @@ void intUpdateProgressBar(WIDGET *psWidget, W_CONTEXT *psContext)
 					BuildPoints = Range;
 				}
 				// prevent a division by 0 error
-				if (Range == 0 )
+				if (Range == 0)
 				{
 					Range = 1;
 				}
@@ -514,7 +515,7 @@ void intUpdateCommandSize(WIDGET *psWidget, W_CONTEXT *psContext)
 		ASSERT( psDroid->droidType == DROID_COMMAND,
 			"intUpdateCommandSize: droid is not a command droid" );
 
-		snprintf(Label->aText, sizeof(Label->aText), "%d/%d", psDroid->psGroup ? grpNumMembers(psDroid->psGroup) : 0, cmdDroidMaxGroup(psDroid));
+		ssprintf(Label->aText, "%u/%u", psDroid->psGroup ? grpNumMembers(psDroid->psGroup) : 0, cmdDroidMaxGroup(psDroid));
 		Label->style &= ~WIDG_HIDDEN;
 	}
 	else
@@ -1916,7 +1917,7 @@ void intInitialiseGraphics(void)
 
 // Free up all surfaces,graphics etc. used by the interface.
 //
-void intDeleteGraphics(void)
+void interfaceDeleteGraphics(void)
 {
 	DeleteButtonData();
 }
@@ -2132,10 +2133,10 @@ SDWORD GetSystem0Buffer(void)
 	return -1;
 }
 
-
-// Free up data for interface buttons.
-//
-void DeleteButtonData(void)
+/**
+ * Free up surfaces and data from interface buttons.
+ */
+void DeleteButtonData()
 {
 	unsigned int i;
 
@@ -2476,7 +2477,7 @@ void CreateIMDButton(IMAGEFILE *ImageFile, UWORD ImageID, void *Object, UDWORD P
 		} else if(IMDType == IMDTYPE_STRUCTURE) {
 			displayStructureButton((STRUCTURE*)Object,&Rotation,&Position,true, scale);
 		} else if(IMDType == IMDTYPE_STRUCTURESTAT) {
-			displayStructureStatButton((STRUCTURE_STATS*)Object,Player,&Rotation,&Position,true, scale);
+			displayStructureStatButton((STRUCTURE_STATS*)Object, &Rotation, &Position, true, scale);
 		} else {
 			displayIMDButton((iIMDShape*)Object,&Rotation,&Position,true, scale);
 		}
@@ -2833,9 +2834,9 @@ BOOL StatGetComponentIMD(BASE_STATS *Stat, SDWORD compID,iIMDShape **CompIMD,iIM
 	switch (compID)
 	{
 	case COMP_BODY:
-		*CompIMD = ((COMP_BASE_STATS *)Stat)->pIMD;
+		*CompIMD = ((COMPONENT_STATS *)Stat)->pIMD;
 		return true;
-//		return ((COMP_BASE_STATS *)Stat)->pIMD;
+//		return ((COMPONENT_STATS *)Stat)->pIMD;
 
 	case COMP_BRAIN:
 //		ASSERT( ((UBYTE*)Stat >= (UBYTE*)asCommandDroids) &&
@@ -2850,31 +2851,31 @@ BOOL StatGetComponentIMD(BASE_STATS *Stat, SDWORD compID,iIMDShape **CompIMD,iIM
 
 	case COMP_WEAPON:
 		*MountIMD = ((WEAPON_STATS*)Stat)->pMountGraphic;
-		*CompIMD = ((COMP_BASE_STATS *)Stat)->pIMD;
+		*CompIMD = ((COMPONENT_STATS *)Stat)->pIMD;
 		return true;
 
 	case COMP_SENSOR:
 		*MountIMD = ((SENSOR_STATS*)Stat)->pMountGraphic;
-		*CompIMD = ((COMP_BASE_STATS *)Stat)->pIMD;
+		*CompIMD = ((COMPONENT_STATS *)Stat)->pIMD;
 		return true;
 
 	case COMP_ECM:
 		*MountIMD = ((ECM_STATS*)Stat)->pMountGraphic;
-		*CompIMD = ((COMP_BASE_STATS *)Stat)->pIMD;
+		*CompIMD = ((COMPONENT_STATS *)Stat)->pIMD;
 		return true;
 
 	case COMP_CONSTRUCT:
 		*MountIMD = ((CONSTRUCT_STATS*)Stat)->pMountGraphic;
-		*CompIMD = ((COMP_BASE_STATS *)Stat)->pIMD;
+		*CompIMD = ((COMPONENT_STATS *)Stat)->pIMD;
 		return true;
 
 	case COMP_PROPULSION:
-		*CompIMD = ((COMP_BASE_STATS *)Stat)->pIMD;
+		*CompIMD = ((COMPONENT_STATS *)Stat)->pIMD;
 		return true;
 
 	case COMP_REPAIRUNIT:
 		*MountIMD = ((REPAIR_STATS*)Stat)->pMountGraphic;
-		*CompIMD = ((COMP_BASE_STATS *)Stat)->pIMD;
+		*CompIMD = ((COMPONENT_STATS *)Stat)->pIMD;
 		return true;
 
 	default:
@@ -3120,14 +3121,18 @@ void intDisplayTransportButton(WIDGET *psWidget, UDWORD xOffset,
     }
 }
 
-
 /* Draws blips on radar to represent Proximity Display and damaged structures */
-void drawRadarBlips(float pixSizeH, float pixSizeV, int RadarOffsetX, int RadarOffsetY)
+void drawRadarBlips(int radarX, int radarY, float pixSizeH, float pixSizeV)
 {
 	PROXIMITY_DISPLAY	*psProxDisp;
 	UWORD			imageID;
 	UDWORD			delay = 150;
 	UDWORD			i;
+	SDWORD width, height;
+
+	// store the width & height of the radar/mini-map
+	width = scrollMaxX - scrollMinX;
+	height = scrollMaxY - scrollMinY;
 
 	// Check if it's time to remove beacons
 	for (i = 0; i < MAX_PLAYERS; i++)
@@ -3144,11 +3149,8 @@ void drawRadarBlips(float pixSizeH, float pixSizeV, int RadarOffsetX, int RadarO
 
 				if (pViewData->type == VIEW_BEACON)
 				{
-					if (pViewData->pData == NULL)
-					{
-						debug(LOG_ERROR, "Found a help message without data!");
-					}
-					else if ((((VIEW_PROXIMITY *)pViewData->pData)->timeAdded + 60000) <= gameTime)
+					ASSERT(pViewData->pData != NULL, "Help message without data!");
+					if (pViewData->pData != NULL && (((VIEW_PROXIMITY *)pViewData->pData)->timeAdded + 60000) <= gameTime)
 					{
 						debug(LOG_MSG, "blip timeout for %d, from %d", i, (((VIEW_PROXIMITY *)pViewData->pData)->sender));
 						removeMessage(psCurrMsg, i);	//remove beacon
@@ -3163,6 +3165,7 @@ void drawRadarBlips(float pixSizeH, float pixSizeV, int RadarOffsetX, int RadarO
 	for (psProxDisp = apsProxDisp[selectedPlayer]; psProxDisp != NULL; psProxDisp = psProxDisp->psNext)
 	{
 		PROX_TYPE	proxType;
+		int		x = 0, y = 0;
 
 		if (psProxDisp->type == POS_PROXDATA)
 		{
@@ -3172,7 +3175,7 @@ void drawRadarBlips(float pixSizeH, float pixSizeV, int RadarOffsetX, int RadarO
 		{
 			FEATURE *psFeature = (FEATURE *)psProxDisp->psMessage->pViewData;
 
-			ASSERT(psFeature && psFeature->psStats, "Bad feature message")
+			ASSERT(psFeature && psFeature->psStats, "Bad feature message");
 			if (psFeature && psFeature->psStats && psFeature->psStats->subType == FEAT_OIL_RESOURCE)
 			{
 				proxType = PROX_RESOURCE;
@@ -3202,8 +3205,34 @@ void drawRadarBlips(float pixSizeH, float pixSizeV, int RadarOffsetX, int RadarO
 			}
 			imageID = (UWORD)(IMAGE_RAD_ENM1 + psProxDisp->strobe + (proxType * (NUM_PULSES + 1)));
 		}
-		// Draw the 'blip'
-		iV_DrawImage(IntImages, imageID, psProxDisp->radarX + RADTLX, psProxDisp->radarY + RADTLY);
+
+		if (psProxDisp->type == POS_PROXDATA)
+		{
+			VIEW_PROXIMITY *psViewProx = (VIEW_PROXIMITY *)((VIEWDATA *)psProxDisp->psMessage->pViewData)->pData;
+
+			x = (psViewProx->x / TILE_UNITS - scrollMinX) * pixSizeH;
+			y = (psViewProx->y / TILE_UNITS - scrollMinY) * pixSizeV;
+		}
+		else if (psProxDisp->type == POS_PROXOBJ)
+		{
+			x = (((BASE_OBJECT *)psProxDisp->psMessage->pViewData)->pos.x / TILE_UNITS - scrollMinX) * pixSizeH;
+			y = (((BASE_OBJECT *)psProxDisp->psMessage->pViewData)->pos.y / TILE_UNITS - scrollMinY) * pixSizeV;
+		}
+		else
+		{
+			ASSERT(false, "Bad message type");
+			continue;
+		}
+
+		// NOTE:  On certain missions (limbo & expand), there is still valid data that is stored outside the
+		// normal radar/mini-map view.  We must now calculate the radar/mini-map's bounding box, and clip
+		// everything outside the box.
+		if ( (x+radarX) < width && (x+radarX) > -width 
+			&& (y+radarY) < height && (y+radarY) > -height)
+		{
+			// Draw the 'blip'
+			iV_DrawImage(IntImages, imageID, x + radarX, y + radarY);
+		}
 	}
 }
 
