@@ -1,7 +1,7 @@
 /*
 	This file is part of Warzone 2100.
 	Copyright (C) 1999-2004  Eidos Interactive
-	Copyright (C) 2005-2007  Warzone Resurrection Project
+	Copyright (C) 2005-2009  Warzone Resurrection Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -156,7 +156,7 @@ enum
 	STR_DORD_FIRE2,
 	STR_DORD_FIRE3,
 	STR_DORD_PATROL,
-	STR_DORD_PERSUE,
+	STR_DORD_PURSUE,
 	STR_DORD_GUARD,
 	STR_DORD_HOLDPOS,
 	STR_DORD_RETREPAIR,
@@ -186,7 +186,7 @@ static const char *getDORDDescription(int id)
 			case STR_DORD_FIRE2          : return _("Return Fire");
 			case STR_DORD_FIRE3          : return _("Hold Fire");
 			case STR_DORD_PATROL         : return _("Patrol");
-			case STR_DORD_PERSUE         : return _("Pursue");
+			case STR_DORD_PURSUE         : return _("Pursue");
 			case STR_DORD_GUARD          : return _("Guard Position");
 			case STR_DORD_HOLDPOS        : return _("Hold Position");
 			case STR_DORD_RETREPAIR      : return _("Return For Repair");
@@ -287,8 +287,8 @@ ORDERBUTTONS OrderButtons[NUM_ORDERS]=
 		ORD_JUSTIFY_COMBINE,
 		IDORDER_CIRCLE,
 		1,0,
-		{IMAGE_ORD_PATROLUP,	0,	0},
-		{IMAGE_ORD_PATROLUP,	0,	0},
+		{IMAGE_ORD_CIRCLEUP,	0,	0},
+		{IMAGE_ORD_CIRCLEUP,	0,	0},
 		{IMAGE_DES_HILIGHT,	0,	0},
 		{STR_DORD_CIRCLE,	0,	0},
 		{DSS_CIRCLE_SET,	0,	0}
@@ -302,11 +302,11 @@ ORDERBUTTONS OrderButtons[NUM_ORDERS]=
 		ORD_JUSTIFY_CENTER | ORD_JUSTIFY_NEWLINE,
 		IDORDER_HALT_TYPE,
 		3,0,
-		{IMAGE_ORD_PERSUEUP,	IMAGE_ORD_GUARDUP,	IMAGE_ORD_HALTUP},
-		{IMAGE_ORD_PERSUEUP,	IMAGE_ORD_GUARDUP,	IMAGE_ORD_HALTUP},
+		{IMAGE_ORD_PURSUEUP,	IMAGE_ORD_GUARDUP,	IMAGE_ORD_HALTUP},
+		{IMAGE_ORD_PURSUEUP,	IMAGE_ORD_GUARDUP,	IMAGE_ORD_HALTUP},
 		{IMAGE_DES_HILIGHT,		IMAGE_DES_HILIGHT,	IMAGE_DES_HILIGHT},
-		{STR_DORD_PERSUE,	STR_DORD_GUARD,	STR_DORD_HOLDPOS},
-		{DSS_HALT_PERSUE,	DSS_HALT_GUARD,	DSS_HALT_HOLD}
+		{STR_DORD_PURSUE,	STR_DORD_GUARD,	STR_DORD_HOLDPOS},
+		{DSS_HALT_PURSUE,	DSS_HALT_GUARD,	DSS_HALT_HOLD}
 	},
 	{
 		ORDBUTCLASS_NORMAL,
@@ -683,7 +683,6 @@ BOOL intAddOrder(BASE_OBJECT *psObj)
    		OrdIndex = AvailableOrders[j].OrderIndex;
 
 		// Get current order state.
-   		//secondaryGetState(SelectedDroids[0], OrderButtons[OrdIndex].Order, &State);
 		State = GetSecondaryStates(OrderButtons[OrdIndex].Order);
 
 		// Get number of buttons.
@@ -1428,7 +1427,6 @@ static BOOL intRefreshOrderButtons(void)
    		OrdIndex = AvailableOrders[j].OrderIndex;
 
 		// Get current order state.
-   		//secondaryGetState(SelectedDroids[0], OrderButtons[OrdIndex].Order, &State);
 		State = GetSecondaryStates(OrderButtons[OrdIndex].Order);
 
 		// Get number of buttons.
@@ -1512,20 +1510,18 @@ static SDWORD GetSecondaryStates(SECONDARY_ORDER sec)
     }
     else //droids
     {
-	    for(i=0; i<NumSelectedDroids; i++)
+	    for (i=0; i<NumSelectedDroids; i++)
 	    {
-		    if (secondaryGetState(SelectedDroids[i], sec, &currState))
-		    {
-			    if (bFirst)
-			    {
-				    state = currState;
-				    bFirst = false;
-			    }
-			    else if (state != currState)
-			    {
-				    state = 0;
-			    }
-		    }
+		    currState = secondaryGetState(SelectedDroids[i], sec);
+			if (bFirst)
+			{
+				state = currState;
+				bFirst = false;
+			}
+			else if (state != currState)
+			{
+				state = 0;
+			}
 	    }
     }
 
