@@ -1,7 +1,7 @@
 /*
 	This file is part of Warzone 2100.
 	Copyright (C) 1999-2004  Eidos Interactive
-	Copyright (C) 2005-2007  Warzone Resurrection Project
+	Copyright (C) 2005-2009  Warzone Resurrection Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -27,10 +27,10 @@
 #include <ctype.h>
 #include <string.h>
 
-// levLoadData printf's
 #include "lib/framework/frame.h"
 #include "lib/framework/frameresource.h"
 #include "lib/framework/listmacs.h"
+#include "lib/exceptionhandler/dumpinfo.h"
 #include "init.h"
 #include "objects.h"
 #include "hci.h"
@@ -46,6 +46,7 @@
 #include "scripttabs.h"
 #include "research.h"
 #include "lib/framework/lexer_input.h"
+#include "effects.h"
 
 extern int lev_get_lineno(void);
 extern char* lev_get_text(void);
@@ -59,11 +60,11 @@ extern int lev_lex_destroy(void);
 static	char	currentLevelName[32];
 
 // the current level descriptions
-LEVEL_DATASET	*psLevels;
+LEVEL_DATASET	*psLevels = NULL;
 
 // the currently loaded data set
-static LEVEL_DATASET	*psBaseData;
-static LEVEL_DATASET	*psCurrLevel;
+static LEVEL_DATASET	*psBaseData = NULL;
+static LEVEL_DATASET	*psCurrLevel = NULL;
 
 // dummy level data for single WRF loads
 static LEVEL_DATASET	sSingleWRF = { 0, 0, 0, 0, 0, { 0 }, 0, 0, 0 };
@@ -123,6 +124,7 @@ void levShutDown(void)
 		free(toDelete->pName);
 		free(toDelete);
 	}
+	psLevels = NULL;
 }
 
 
@@ -196,7 +198,7 @@ BOOL levParse(const char* buffer, size_t size, searchPathMode datadir)
 				psDataSet->players = 1;
 				psDataSet->game = -1;
 				psDataSet->dataDir = datadir;
-				LIST_ADDEND(psLevels, psDataSet, LEVEL_DATASET);
+				LIST_APPEND(psLevels, psDataSet, LEVEL_DATASET);
 				currData = 0;
 
 				// set the dataset type
@@ -487,11 +489,14 @@ BOOL levReleaseAll(void)
 {
 	SDWORD i;
 
+	// clear out old effect data first
+	initEffectsSystem();
 	// release old data if any was loaded
 	if (psCurrLevel != NULL)
 	{
 		if (!levReleaseMissionData())
 		{
+			debug(LOG_ERROR, "Failed to unload mission data");
 			return false;
 		}
 
@@ -500,6 +505,7 @@ BOOL levReleaseAll(void)
 		{
 			if (!stageTwoShutDown())
 			{
+				debug(LOG_ERROR, "Failed stage two shutdown");
 				return false;
 			}
 		}
@@ -518,6 +524,7 @@ BOOL levReleaseAll(void)
 
 		if (!stageOneShutDown())
 		{
+			debug(LOG_ERROR, "Failed stage one shutdown");
 			return false;
 		}
 	}
@@ -531,7 +538,10 @@ BOOL levReleaseAll(void)
 static BOOL levLoadSingleWRF(const char* name)
 {
 	// free the old data
-	levReleaseAll();
+	if (!levReleaseAll())
+	{
+		return false;
+	}
 
 	// create the dummy level data
 	if (sSingleWRF.pName)
@@ -549,7 +559,7 @@ static BOOL levLoadSingleWRF(const char* name)
 	}
 
 	// load the data
-	debug(LOG_WZ, "levLoadSingleWRF: Loading %s ...", name);
+	debug(LOG_WZ, "Loading %s ...", name);
 	if (!resLoad(name, 0))
 	{
 		return false;
@@ -573,13 +583,13 @@ char *getLevelName( void )
 
 
 // load up the data for a level
-BOOL levLoadData(const char* name, char *pSaveName, SDWORD saveType)
+BOOL levLoadData(const char* name, char *pSaveName, GAME_TYPE saveType)
 {
 	LEVEL_DATASET	*psNewLevel, *psBaseData, *psChangeLevel;
 	SDWORD			i;
 	BOOL            bCamChangeSaveGame;
 
-	debug(LOG_WZ, "Loading level %s (%s)", name, pSaveName);
+	debug(LOG_WZ, "Loading level %s (%s, type %d)", name, pSaveName, (int)saveType);
 	if (saveType == GTYPE_SAVE_START || saveType == GTYPE_SAVE_MIDMISSION)
 	{
 		if (!levReleaseAll())
@@ -595,9 +605,10 @@ BOOL levLoadData(const char* name, char *pSaveName, SDWORD saveType)
 	psNewLevel = levFindDataSet(name);
 	if (psNewLevel == NULL)
 	{
-		debug(LOG_NEVER, "levLoadData: dataset %s not found - trying to load as WRF", name);
+		debug(LOG_WZ, "Dataset %s not found - trying to load as WRF", name);
 		return levLoadSingleWRF(name);
 	}
+	debug(LOG_WZ, "** Data set found is %s type %d", psNewLevel->pName, (int)psNewLevel->type);
 
 	/* Keep a copy of the present level name */
 	sstrcpy(currentLevelName, name);
@@ -608,6 +619,7 @@ BOOL levLoadData(const char* name, char *pSaveName, SDWORD saveType)
 		if (psNewLevel->psChange != NULL)
 		{
 			bCamChangeSaveGame = true;
+			debug(LOG_WZ, "** CAMCHANGE FOUND");
 		}
 	}
 
@@ -616,7 +628,7 @@ BOOL levLoadData(const char* name, char *pSaveName, SDWORD saveType)
 	if (((psNewLevel->psChange != NULL) && (psCurrLevel != NULL)) || bCamChangeSaveGame)
 	{
 		//store the level name
-		debug( LOG_WZ, "levLoadData: Found CAMCHANGE dataset\n" );
+		debug(LOG_WZ, "Found CAMCHANGE dataset");
 		psChangeLevel = psNewLevel;
 		psNewLevel = psNewLevel->psChange;
 	}
@@ -624,7 +636,7 @@ BOOL levLoadData(const char* name, char *pSaveName, SDWORD saveType)
 	// ensure the correct dataset is loaded
 	if (psNewLevel->type == LDS_CAMPAIGN)
 	{
-		debug( LOG_ERROR, "levLoadData: Cannot load a campaign dataset (%s)", psNewLevel->pName );
+		debug(LOG_ERROR, "Cannot load a campaign dataset (%s)", psNewLevel->pName);
 		return false;
 	}
 	else
@@ -636,12 +648,16 @@ BOOL levLoadData(const char* name, char *pSaveName, SDWORD saveType)
 				(psCurrLevel->type >= LDS_NONE && psNewLevel->type  < LDS_NONE))
 			{
 				// there is a dataset loaded but it isn't the correct one
-				debug(LOG_WZ, "levLoadData: Incorrect base dataset loaded - levReleaseAll()");
-				levReleaseAll();	// this sets psCurrLevel to NULL
+				debug(LOG_WZ, "Incorrect base dataset loaded (%p != %p, %d - %d)",
+				      psCurrLevel->psBaseData, psNewLevel->psBaseData, (int)psCurrLevel->type, (int)psNewLevel->type);
+				if (!levReleaseAll())	// this sets psCurrLevel to NULL
+				{
+					return false;
+				}
 			}
 			else
 			{
-				debug(LOG_WZ, "levLoadData: Correct base dataset already loaded.");
+				debug(LOG_WZ, "Correct base dataset already loaded.");
 			}
 		}
 
@@ -650,13 +666,13 @@ BOOL levLoadData(const char* name, char *pSaveName, SDWORD saveType)
 		{
 			if (psNewLevel->psBaseData != NULL)
 			{
-				debug(LOG_WZ, "levLoadData: Setting base dataset to load: %s", psNewLevel->psBaseData->pName);
+				debug(LOG_WZ, "Setting base dataset to load: %s", psNewLevel->psBaseData->pName);
 			}
 			psBaseData = psNewLevel->psBaseData;
 		}
 		else
 		{
-			debug(LOG_WZ, "levLoadData: No base dataset to load");
+			debug(LOG_WZ, "No base dataset to load");
 			psBaseData = NULL;
 		}
 	}
@@ -666,7 +682,7 @@ BOOL levLoadData(const char* name, char *pSaveName, SDWORD saveType)
 	// reset the old mission data if necessary
 	if (psCurrLevel != NULL)
 	{
-		debug(LOG_WZ, "levLoadData: reseting old mission data");
+		debug(LOG_WZ, "Reseting old mission data");
 		if (!levReleaseMissionData())
 		{
 			return false;
@@ -687,7 +703,7 @@ BOOL levLoadData(const char* name, char *pSaveName, SDWORD saveType)
 	if (psNewLevel->type == LDS_COMPLETE || //psNewLevel->type >= LDS_MULTI_TYPE_START ||
 		psBaseData != NULL)
 	{
-		debug(LOG_WZ, "levLoadData: Calling stageOneInitialise!");
+		debug(LOG_WZ, "Calling stageOneInitialise!");
 		if (!stageOneInitialise())
 		{
 			return false;
@@ -697,13 +713,13 @@ BOOL levLoadData(const char* name, char *pSaveName, SDWORD saveType)
 	// load up a base dataset if necessary
 	if (psBaseData != NULL)
 	{
-		debug( LOG_NEVER, "levLoadData: loading base dataset %s\n", psBaseData->pName );
+		debug(LOG_WZ, "Loading base dataset %s", psBaseData->pName);
 		for(i=0; i<LEVEL_MAXFILES; i++)
 		{
 			if (psBaseData->apDataFiles[i])
 			{
 				// load the data
-				debug(LOG_WZ, "levLoadData: Loading %s ...", psBaseData->apDataFiles[i]);
+				debug(LOG_WZ, "Loading [directory: %s] %s ...", PHYSFS_getRealDir(psBaseData->apDataFiles[i]), psBaseData->apDataFiles[i]);
 				if (!resLoad(psBaseData->apDataFiles[i], i))
 				{
 					return false;
@@ -722,7 +738,7 @@ BOOL levLoadData(const char* name, char *pSaveName, SDWORD saveType)
 	{
 		ASSERT( psNewLevel->type == LDS_BETWEEN,
 			"levLoadData: only BETWEEN missions do not need a .gam file" );
-		debug( LOG_NEVER, "levLoadData: no .gam file for level: BETWEEN mission\n" );
+		debug(LOG_WZ, "No .gam file for level: BETWEEN mission");
 		if (pSaveName != NULL)
 		{
 			if (psBaseData != NULL)
@@ -736,17 +752,17 @@ BOOL levLoadData(const char* name, char *pSaveName, SDWORD saveType)
 			//set the mission type before the saveGame data is loaded
 			if (saveType == GTYPE_SAVE_MIDMISSION)
 			{
-				debug( LOG_NEVER, "levLoadData: init mission stuff\n" );
+				debug(LOG_WZ, "Init mission stuff");
 				if (!startMissionSave(psNewLevel->type))
 				{
 					return false;
 				}
 
-				debug( LOG_NEVER, "levLoadData: dataSetSaveFlag\n" );
+				debug(LOG_NEVER, "dataSetSaveFlag");
 				dataSetSaveFlag();
 			}
 
-			debug( LOG_NEVER, "levLoadData: loading savegame: %s\n", pSaveName );
+			debug(LOG_NEVER, "Loading savegame: %s", pSaveName);
 			if (!loadGame(pSaveName, false, true,true))
 			{
 				return false;
@@ -756,7 +772,7 @@ BOOL levLoadData(const char* name, char *pSaveName, SDWORD saveType)
 		if ((pSaveName == NULL) ||
 			(saveType == GTYPE_SAVE_START))
 		{
-			debug( LOG_NEVER, "levLoadData: start mission - no .gam\n" );
+			debug(LOG_NEVER, "Start mission - no .gam");
 			if (!startMission(psNewLevel->type, NULL))
 			{
 				return false;
@@ -767,7 +783,6 @@ BOOL levLoadData(const char* name, char *pSaveName, SDWORD saveType)
 	//we need to load up the save game data here for a camchange
 	if (bCamChangeSaveGame)
 	{
-		debug( LOG_NEVER, "levLoadData: no .gam file for level: BETWEEN mission\n" );
 		if (pSaveName != NULL)
 		{
 			if (psBaseData != NULL)
@@ -778,7 +793,7 @@ BOOL levLoadData(const char* name, char *pSaveName, SDWORD saveType)
 				}
 			}
 
-			debug( LOG_NEVER, "levLoadData: loading savegame: %s\n", pSaveName );
+			debug(LOG_NEVER, "loading savegame: %s", pSaveName);
 			if (!loadGame(pSaveName, false, true,true))
 			{
 				return false;
@@ -793,7 +808,7 @@ BOOL levLoadData(const char* name, char *pSaveName, SDWORD saveType)
 
 
 	// load the new data
-	debug( LOG_NEVER, "levLoadData: loading mission dataset: %s\n", psNewLevel->pName );
+	debug(LOG_NEVER, "Loading mission dataset: %s", psNewLevel->pName);
 	for(i=0; i < LEVEL_MAXFILES; i++)
 	{
 		if (psNewLevel->game == i)
@@ -813,17 +828,17 @@ BOOL levLoadData(const char* name, char *pSaveName, SDWORD saveType)
 				//set the mission type before the saveGame data is loaded
 				if (saveType == GTYPE_SAVE_MIDMISSION)
 				{
-					debug( LOG_NEVER, "levLoadData: init mission stuff\n" );
+					debug(LOG_WZ, "Init mission stuff");
 					if (!startMissionSave(psNewLevel->type))
 					{
 						return false;
 					}
 
-					debug( LOG_NEVER, "levLoadData: dataSetSaveFlag\n" );
+					debug(LOG_NEVER, "dataSetSaveFlag");
 					dataSetSaveFlag();
 				}
 
-				debug( LOG_NEVER, "levLoadData: loading save game %s\n", pSaveName );
+				debug(LOG_NEVER, "Loading save game %s", pSaveName);
 				if (!loadGame(pSaveName, false, true,true))
 				{
 					return false;
@@ -839,14 +854,14 @@ BOOL levLoadData(const char* name, char *pSaveName, SDWORD saveType)
 				{
 				case LDS_COMPLETE:
 				case LDS_CAMSTART:
-					debug(LOG_WZ, "levLoadData: LDS_COMPLETE / LDS_CAMSTART");
+					debug(LOG_WZ, "LDS_COMPLETE / LDS_CAMSTART");
 					if (!startMission(LDS_CAMSTART, psNewLevel->apDataFiles[i]))
 					{
 						return false;
 					}
 					break;
 				case LDS_BETWEEN:
-					debug(LOG_WZ, "levLoadData: LDS_BETWEEN");
+					debug(LOG_WZ, "LDS_BETWEEN");
 					if (!startMission(LDS_BETWEEN, psNewLevel->apDataFiles[i]))
 					{
 						return false;
@@ -854,14 +869,14 @@ BOOL levLoadData(const char* name, char *pSaveName, SDWORD saveType)
 					break;
 
 				case LDS_MKEEP:
-					debug(LOG_WZ, "levLoadData: LDS_MKEEP");
+					debug(LOG_WZ, "LDS_MKEEP");
 					if (!startMission(LDS_MKEEP, psNewLevel->apDataFiles[i]))
 					{
 						return false;
 					}
 					break;
 				case LDS_CAMCHANGE:
-					debug(LOG_WZ, "levLoadData: LDS_CAMCHANGE");
+					debug(LOG_WZ, "LDS_CAMCHANGE");
 					if (!startMission(LDS_CAMCHANGE, psNewLevel->apDataFiles[i]))
 					{
 						return false;
@@ -869,14 +884,14 @@ BOOL levLoadData(const char* name, char *pSaveName, SDWORD saveType)
 					break;
 
 				case LDS_EXPAND:
-					debug(LOG_WZ, "levLoadData: LDS_EXPAND");
+					debug(LOG_WZ, "LDS_EXPAND");
 					if (!startMission(LDS_EXPAND, psNewLevel->apDataFiles[i]))
 					{
 						return false;
 					}
 					break;
 				case LDS_EXPAND_LIMBO:
-					debug(LOG_WZ, "levLoadData: LDS_LIMBO");
+					debug(LOG_WZ, "LDS_LIMBO");
 					if (!startMission(LDS_EXPAND_LIMBO, psNewLevel->apDataFiles[i]))
 					{
 						return false;
@@ -884,15 +899,14 @@ BOOL levLoadData(const char* name, char *pSaveName, SDWORD saveType)
 					break;
 
 				case LDS_MCLEAR:
-					debug(LOG_WZ, "levLoadData: LDS_MCLEAR");
+					debug(LOG_WZ, "LDS_MCLEAR");
 					if (!startMission(LDS_MCLEAR, psNewLevel->apDataFiles[i]))
 					{
 						return false;
 					}
 					break;
 				case LDS_MKEEP_LIMBO:
-					debug(LOG_WZ, "levLoadData: LDS_MKEEP_LIMBO");
-					debug( LOG_NEVER, "MKEEP_LIMBO\n" );
+					debug(LOG_WZ, "LDS_MKEEP_LIMBO");
 					if (!startMission(LDS_MKEEP_LIMBO, psNewLevel->apDataFiles[i]))
 					{
 						return false;
@@ -901,7 +915,7 @@ BOOL levLoadData(const char* name, char *pSaveName, SDWORD saveType)
 				default:
 					ASSERT( psNewLevel->type >= LDS_MULTI_TYPE_START,
 						"levLoadData: Unexpected mission type" );
-					debug(LOG_WZ, "levLoadData: default (MULTIPLAYER)");
+					debug(LOG_WZ, "default (MULTIPLAYER)");
 					if (!startMission(LDS_CAMSTART, psNewLevel->apDataFiles[i]))
 					{
 						return false;
@@ -913,7 +927,7 @@ BOOL levLoadData(const char* name, char *pSaveName, SDWORD saveType)
 		else if (psNewLevel->apDataFiles[i])
 		{
 			// load the data
-			debug(LOG_WZ, "levLoadData: Loading %s", psNewLevel->apDataFiles[i]);
+			debug(LOG_WZ, "Loading %s", psNewLevel->apDataFiles[i]);
 			if (!resLoad(psNewLevel->apDataFiles[i], i + CURRENT_DATAID))
 			{
 				return false;
@@ -936,7 +950,7 @@ BOOL levLoadData(const char* name, char *pSaveName, SDWORD saveType)
 	{
 		//load script stuff
 		// load the event system state here for a save game
-		debug( LOG_NEVER, "levLoadData: loading script system state\n" );
+		debug(LOG_NEVER, "Loading script system state");
 		if (!loadScriptState(pSaveName))
 		{
 			return false;
@@ -965,6 +979,14 @@ BOOL levLoadData(const char* name, char *pSaveName, SDWORD saveType)
 		psCurrLevel = psChangeLevel;
 	}
 
+	{
+		// Copy this info to be used by the crash handler for the dump file
+		char buf[256];
+
+		ssprintf(buf, "Current Level/map is %s", psCurrLevel->pName);
+		addDumpInfo(buf);
+	}
+
 
 	return true;
 }
@@ -988,7 +1010,8 @@ static void levTestLoad(const char* level)
 	strcpy(savegameName, "selftest/");	// we need to recreate string, because saveGame clobbered it
 	strcat(savegameName, level);
 	strcat(savegameName, ".gam");
-	levReleaseAll();
+	retval = levReleaseAll();
+	assert(retval == true);
 	fprintf(stdout, "\t\tSaved: %s\n", savegameName);
 }
 
