@@ -309,7 +309,21 @@ GAMECODE gameLoop(void)
 						case DROID_TRANSPORTER:
 							if( (psCurr->psGroup != NULL) )
 							{
+								DROID *psDroid = NULL;
+
 								numTransporterDroids[i] += psCurr->psGroup->refCount-1;
+								// and count the units inside it...for MP games only(?)
+								if (bMultiPlayer)
+								{
+									for (psDroid = psCurr->psGroup->psList; psDroid != NULL && psDroid != psCurr; psDroid = psDroid->psGrpNext)
+									{
+										// since in MP we can only have cyborgs in transport, don't need to count DROID_CONSTRUCT as well.
+										if (psDroid->droidType == DROID_CYBORG_CONSTRUCT)
+										{
+											numConstructorDroids[i] += 1;
+										}
+									}
+								}
 							}
 							break;
 						default:
@@ -692,9 +706,6 @@ GAMECODE gameLoop(void)
 /* The video playback loop */
 void videoLoop(void)
 {
-	bool bVolKilled = false;
-	float originalVolume = 0.0;
-
 	// There is something really odd here. - Per
 	static BOOL bActiveBackDrop = false;
 
@@ -709,14 +720,6 @@ void videoLoop(void)
 		bQuitVideo = !seq_UpdateFullScreenVideo(NULL);
 	}
 
-	if ( (keyPressed(KEY_ESC) || mouseReleased(MOUSE_LMB) || bQuitVideo) && !seq_AnySeqLeft() )
-	{
-		/* zero volume before video quit - restore later */
-		originalVolume = sound_GetUIVolume();
-		sound_SetUIVolume(0.0);
-		bVolKilled = true;
-	}
-
 	//toggling display mode disabled in video mode
 	// Check for quit
 	if (keyPressed(KEY_ESC) || mouseReleased(MOUSE_LMB))
@@ -812,11 +815,6 @@ void videoLoop(void)
 
 	pie_ScreenFlip(CLEAR_BLACK);// videoloopflip
 
-	/* restore volume after video quit */
-	if (bVolKilled)
-	{
-		sound_SetUIVolume(originalVolume);
-	}
 }
 
 
