@@ -1,7 +1,7 @@
 /*
 	This file is part of Warzone 2100.
 	Copyright (C) 1999-2004  Eidos Interactive
-	Copyright (C) 2005-2007  Warzone Resurrection Project
+	Copyright (C) 2005-2009  Warzone Resurrection Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -26,9 +26,10 @@
 #include <time.h>
 
 #include "lib/framework/frame.h"
-#include "lib/framework/tagfile.h"
+#include "lib/framework/endian_hack.h"
 #include "lib/framework/file.h"
 #include "lib/framework/physfs_ext.h"
+#include "lib/framework/tagfile.h"
 #include "lib/ivis_common/tex.h"
 
 #include "map.h"
@@ -62,19 +63,10 @@ typedef struct _map_save_header
 	UDWORD		height;
 } MAP_SAVEHEADER;
 
-
-#define SAVE_MAP_V2 \
-	UWORD		texture; \
-	UBYTE		height
-
 typedef struct _map_save_tilev2
 {
-	SAVE_MAP_V2;
-} MAP_SAVETILEV2;
-
-typedef struct _map_save_tile
-{
-	SAVE_MAP_V2;
+	UWORD		texture;
+	UBYTE		height;
 } MAP_SAVETILE;
 
 typedef struct _gateway_save_header
@@ -88,11 +80,6 @@ typedef struct _gateway_save
 	UBYTE	x0,y0,x1,y1;
 } GATEWAY_SAVE;
 
-typedef struct _zonemap_save_header_v1 {
-	UWORD version;
-	UWORD numZones;
-} ZONEMAP_SAVEHEADER_V1;
-
 typedef struct _zonemap_save_header {
 	UWORD version;
 	UWORD numZones;
@@ -112,10 +99,6 @@ typedef struct _zonemap_save_header {
 /* Number of entries in the sqrt(1/(1+x*x)) table for aaLine */
 #define	ROOT_TABLE_SIZE		1024
 
-/* aaLine direction bits and tables */
-#define DIR_STEEP			1  /* set when abs(dy) > abs(dx) */
-#define DIR_NEGY			2  /* set whey dy < 0 */
-
 /* The size and contents of the map */
 UDWORD	mapWidth = 0, mapHeight = 0;
 MAPTILE	*psMapTiles = NULL;
@@ -147,13 +130,18 @@ BOOL mapNew(UDWORD width, UDWORD height)
 
 	if (width*height > MAP_MAXAREA)
 	{
-		debug( LOG_ERROR, "mapNew: map too large : %d %d\n", width, height );
-		abort();
+		debug(LOG_ERROR, "map too large : %u %u", width, height);
+
 		return false;
 	}
 
 	psMapTiles = calloc(width * height, sizeof(MAPTILE));
-	ASSERT(psMapTiles != NULL, "mapNew: Out of memory")
+	if (psMapTiles == NULL)
+	{
+		debug(LOG_ERROR, "Out of memory");
+		abort();
+		return false;
+	}
 
 	psTile = psMapTiles;
 	for (i = 0; i < width * height; i++)
@@ -187,19 +175,15 @@ BOOL mapNew(UDWORD width, UDWORD height)
 static BOOL mapLoadV3(char *pFileData, UDWORD fileSize)
 {
 	UDWORD				i,j;
-	MAP_SAVETILEV2		*psTileData;
+	MAP_SAVETILE		*psTileData;
 	GATEWAY_SAVEHEADER	*psGateHeader;
 	GATEWAY_SAVE		*psGate;
-	ZONEMAP_SAVEHEADER	*psZoneHeader;
-	UWORD ZoneSize;
-	UBYTE *pZone;
-	UBYTE *pDestZone;
 
 	/* Load in the map data */
-	psTileData = (MAP_SAVETILEV2 *)(pFileData + SAVE_HEADER_SIZE);
+	psTileData = (MAP_SAVETILE *)(pFileData + SAVE_HEADER_SIZE);
 	for(i=0; i< mapWidth * mapHeight; i++)
 	{
-		/* MAP_SAVETILEV2 */
+		/* MAP_SAVETILE */
 		endian_uword(&psTileData->texture);
 
 		psMapTiles[i].texture = psTileData->texture;
@@ -208,7 +192,7 @@ static BOOL mapLoadV3(char *pFileData, UDWORD fileSize)
 		{
 			psMapTiles[i].tileVisBits =(UBYTE)(( (psMapTiles[i].tileVisBits) &~ (UBYTE)(1<<j) ));
 		}
-		psTileData = (MAP_SAVETILEV2 *)(((UBYTE *)psTileData) + SAVE_TILE_SIZE);
+		psTileData = (MAP_SAVETILE *)(((UBYTE *)psTileData) + SAVE_TILE_SIZE);
 	}
 
 	psGateHeader = (GATEWAY_SAVEHEADER*)psTileData;
@@ -223,133 +207,12 @@ static BOOL mapLoadV3(char *pFileData, UDWORD fileSize)
 	for(i=0; i<psGateHeader->numGateways; i++) {
 		if (!gwNewGateway(psGate->x0,psGate->y0, psGate->x1,psGate->y1)) {
 			debug( LOG_ERROR, "mapLoadV3: Unable to add gateway" );
-			abort();
+
 			return false;
 		}
 		psGate++;
 	}
 
-	psZoneHeader = (ZONEMAP_SAVEHEADER*)psGate;
-
-	/* ZONEMAP_SAVEHEADER */
-	endian_uword(&psZoneHeader->version);
-	endian_uword(&psZoneHeader->numZones);
-	endian_uword(&psZoneHeader->numEquivZones);
-	endian_uword(&psZoneHeader->pad);
-
-	ASSERT(psZoneHeader->version == 1
-	    || psZoneHeader->version == 2
-	    || psZoneHeader->version == 3, "Invalid zone map version: %u", psZoneHeader->version);
-
-	/* Special case due to r4637: a version 2 with numZones and
-	 * numEquivZones set to zero is actually version 3.
-	 */
-	if (psZoneHeader->version == 2
-	 && psZoneHeader->numZones == 0
-	 && psZoneHeader->numEquivZones == 0)
-	{
-		psZoneHeader->version = 3;
-	}
-
-	if(!gwNewZoneMap()) {
-		return false;
-	}
-
-	// This is a bit nasty but should work fine.
-	if      (psZoneHeader->version == 1)
-	{
-		// version 1 so add the size of a version 1 header.
-		pZone = ((UBYTE*)psZoneHeader) + sizeof(ZONEMAP_SAVEHEADER_V1);
-	}
-	else if (psZoneHeader->version == 2)
-	{
-		// version 2 so add the size of a version 2 header.
-		pZone = ((UBYTE*)psZoneHeader) + sizeof(ZONEMAP_SAVEHEADER);
-	}
-	/* Loading of savegames with ZONEMAP_SAVEHEADER structure versions of 3
-	 * or larger is not supported (and for version 3 cannot be supported as
-	 * long as we depend on zones).
-	 */
-	else if (psZoneHeader->version == 3)
-	{
-		debug(LOG_ERROR, "Unsupported savegame (zone map version 3): this savegame cannot be loaded with this version of warzone because \"zone map\" version 3 fails to provide required data. NOTE: Warzone 2100 2.2 will most likely support this savegame anyway.");
-		return false;
-	}
-	else
-	{
-		return false;
-	}
-
-	for(i=0; i<psZoneHeader->numZones; i++) {
-		ZoneSize = *((UWORD*)(pZone));
-		endian_uword(&ZoneSize);
-
-		pDestZone = gwNewZoneLine(i,ZoneSize);
-
-		if(pDestZone == NULL) {
-			return false;
-		}
-
-		for(j=0; j<ZoneSize; j++) {
-			pDestZone[j] = pZone[2+j];
-		}
-
-		pZone += ZoneSize+2;
-	}
-
-	// Version 2 has the zone equivelancy lists tacked on the end.
-	if(psZoneHeader->version == 2) {
-
-		if(psZoneHeader->numEquivZones > 0) {
-			// Load in the zone equivelance lists.
-			if(!gwNewEquivTable(psZoneHeader->numEquivZones)) {
-				debug( LOG_ERROR, "gwNewEquivTable failed" );
-				abort();
-				return false;
-			}
-
-			for(i=0; i<psZoneHeader->numEquivZones; i++) {
-				if(*pZone != 0) {
-					if(!gwSetZoneEquiv(i, (SDWORD)*pZone, pZone+1)) {
-						debug( LOG_ERROR, "gwSetZoneEquiv failed" );
-						abort();
-						return false;
-					}
-				}
-				pZone += ((UDWORD)*pZone)+1;
-			}
-		}
-	}
-
-	if ((char *)pZone - pFileData > fileSize)
-	{
-		debug( LOG_ERROR, "mapLoadV3: unexpected end of file" );
-		abort();
-		return false;
-	}
-
-	LOADBARCALLBACK();	//	loadingScreenCallback();
-
-	if ((apEquivZones != NULL) &&
-		!gwGenerateLinkGates())
-	{
-		return false;
-	}
-
-	LOADBARCALLBACK();	//	loadingScreenCallback();
-
-	//add new map initialise
-	if (!gwLinkGateways())
-	{
-		return false;
-	}
-
-	LOADBARCALLBACK();	//	loadingScreenCallback();
-
-#if defined(DEBUG)
-	gwCheckZoneSizes();
-#endif
-
 	return true;
 }
 
@@ -365,8 +228,7 @@ BOOL mapLoad(char *pFileData, UDWORD fileSize)
 	if (psHeader->aFileType[0] != 'm' || psHeader->aFileType[1] != 'a' ||
 		psHeader->aFileType[2] != 'p' || psHeader->aFileType[3] != ' ')
 	{
-		debug( LOG_ERROR, "mapLoad: Incorrect file type" );
-		abort();
+		ASSERT(false, "Incorrect map type");
 		free(pFileData);
 		return false;
 	}
@@ -379,13 +241,13 @@ BOOL mapLoad(char *pFileData, UDWORD fileSize)
 	/* Check the file version */
 	if (psHeader->version <= VERSION_9)
 	{
-		ASSERT(false, "MapLoad: unsupported save format version %d", psHeader->version);
+		ASSERT(false, "Unsupported save format version %d", psHeader->version);
 		free(pFileData);
 		return false;
 	}
 	else if (psHeader->version > CURRENT_VERSION_NUM)
 	{
-		ASSERT(false, "MapLoad: undefined save format version %d", psHeader->version);
+		ASSERT(false, "Undefined save format version %d", psHeader->version);
 		free(pFileData);
 		return false;
 	}
@@ -396,8 +258,8 @@ BOOL mapLoad(char *pFileData, UDWORD fileSize)
 
 	if (width*height > MAP_MAXAREA)
 	{
-		debug( LOG_ERROR, "mapLoad: map too large : %d %d\n", width, height );
-		abort();
+		debug( LOG_ERROR, "Map too large : %d %d\n", width, height );
+		free(pFileData);
 		return false;
 	}
 
@@ -406,14 +268,13 @@ BOOL mapLoad(char *pFileData, UDWORD fileSize)
 
 	/* Allocate the memory for the map */
 	psMapTiles = calloc(width * height, sizeof(MAPTILE));
-	ASSERT(psMapTiles != NULL, "mapLoad: Out of memory" );
+	ASSERT(psMapTiles != NULL, "Out of memory" );
 
 	mapWidth = width;
 	mapHeight = height;
 
 	//load in the map data itself
-	if (!mapLoadV3(pFileData, fileSize))
-		return false;
+	mapLoadV3(pFileData, fileSize);
 
 	environReset();
 
@@ -475,7 +336,7 @@ static void objectStatTagged(BASE_OBJECT *psObj, int body, int resistance)
 
 	tagWriteEnter(0x03, 1);
 	tagWrite(0x01, body);
-	tagWrite(0x02, NUM_WEAPON_CLASS);
+	tagWrite(0x02, WC_NUM_WEAPON_CLASSES);
 	tagWriteEnter(0x03, NUM_HIT_SIDES);
 	for (i = 0; i < NUM_HIT_SIDES; i++)
 	{
@@ -488,7 +349,7 @@ static void objectStatTagged(BASE_OBJECT *psObj, int body, int resistance)
 	tagWriteLeave(0x03);
 }
 
-static void objectWeaponTagged(int num, UWORD *rotation, UWORD *pitch, WEAPON *asWeaps, BASE_OBJECT **psTargets)
+static void objectWeaponTagged(int num, WEAPON *asWeaps, BASE_OBJECT **psTargets)
 {
 	int i;
 
@@ -496,9 +357,8 @@ static void objectWeaponTagged(int num, UWORD *rotation, UWORD *pitch, WEAPON *a
 	for (i = 0; i < num; i++)
 	{
 		tagWrite(0x01, asWeaps[i].nStat);
-		tagWrite(0x02, rotation[i]);
-		tagWrite(0x03, pitch[i]);
-		tagWrite(0x04, asWeaps[i].hitPoints);
+		tagWrite(0x02, asWeaps[i].rotation);
+		tagWrite(0x03, asWeaps[i].pitch);
 		tagWrite(0x05, asWeaps[i].ammo);
 		tagWrite(0x06, asWeaps[i].lastFired);
 		tagWrite(0x07, asWeaps[i].recoilValue);
@@ -514,7 +374,7 @@ static void objectWeaponTagged(int num, UWORD *rotation, UWORD *pitch, WEAPON *a
 static void droidSaveTagged(DROID *psDroid)
 {
 	int plr = psDroid->player;
-	uint16_t v[DROID_MAXCOMP], i, order[4], ammo[VTOL_MAXWEAPS];
+	uint16_t v[ARRAY_SIZE(psDroid->asBits)], i, order[4], ammo[VTOL_MAXWEAPS];
 	int32_t sv[2];
 	float fv[3];
 
@@ -523,17 +383,17 @@ static void droidSaveTagged(DROID *psDroid)
 	objectSaveTagged((BASE_OBJECT *)psDroid); /* 0x01 */
 	objectSensorTagged(psDroid->sensorRange, psDroid->sensorPower, 0, psDroid->ECMMod); /* 0x02 */
 	objectStatTagged((BASE_OBJECT *)psDroid, psDroid->originalBody, psDroid->resistance); /* 0x03 */
-	objectWeaponTagged(psDroid->numWeaps, psDroid->turretRotation, psDroid->turretPitch, psDroid->asWeaps, psDroid->psActionTarget);
+	objectWeaponTagged(psDroid->numWeaps, psDroid->asWeaps, psDroid->psActionTarget);
 
 	/* DROID GROUP */
 
 	tagWriteEnter(0x0a, 1);
 	tagWrite(0x01, psDroid->droidType);
-	for (i = 0; i < DROID_MAXCOMP; i++)
+	for (i = 0; i < ARRAY_SIZE(v); ++i)
 	{
 		v[i] = psDroid->asBits[i].nStat;
 	}
-	tagWrite16v(0x02, DROID_MAXCOMP, v);
+	tagWrite16v(0x02, ARRAY_SIZE(v), v);
 	// transporter droid in the mission list
 	if (psDroid->droidType == DROID_TRANSPORTER && apsDroidLists[plr] == mission.apsDroidLists[plr])
 	{
@@ -645,7 +505,7 @@ static void structureSaveTagged(STRUCTURE *psStruct)
 	objectSaveTagged((BASE_OBJECT *)psStruct); /* 0x01 */
 	objectSensorTagged(psStruct->sensorRange, psStruct->sensorPower, 0, psStruct->ECMMod); /* 0x02 */
 	objectStatTagged((BASE_OBJECT *)psStruct, psStruct->pStructureType->bodyPoints, psStruct->resistance); /* 0x03 */
-	objectWeaponTagged(psStruct->numWeaps, psStruct->turretRotation, psStruct->turretPitch, psStruct->asWeaps, psStruct->psTarget);
+	objectWeaponTagged(psStruct->numWeaps, psStruct->asWeaps, psStruct->psTarget);
 
 	/* STRUCTURE GROUP */
 
@@ -865,10 +725,11 @@ BOOL mapSaveTagged(char *pFileName)
 		tagWrite(0x03, TRI_FLIPPED(psTile));
 		tagWrite(0x04, psTile->texture & TILE_XFLIP);
 		tagWrite(0x05, psTile->texture & TILE_YFLIP);
-		tagWrite(0x06, TILE_IS_NOTBLOCKING(psTile));
+		tagWrite(0x06, TileIsNotBlocking(psTile)); // Redundant, since already included in tileInfoBits
 		tagWrite(0x08, psTile->height);
 		tagWrite(0x09, psTile->tileVisBits);
 		tagWrite(0x0a, psTile->tileInfoBits);
+		tagWrite(0x0b, (psTile->texture & TILE_ROTMASK) >> TILE_ROTSHIFT);
 
 		psTile++;
 		x++;
@@ -1099,29 +960,6 @@ BOOL mapSaveTagged(char *pFileName)
 	}
 	tagWriteLeave(0x0f);
 
-	tagWriteEnter(0x10, gwNumZoneLines()); // gateway zone lines
-	for (i = 0; i < gwNumZoneLines(); i++)
-	{
-		// binary blob for now... ugh, this is ugly, but no worse than it was
-		tagWrite(0x01, gwZoneLineSize(i));
-		tagWrite8v(0x02, gwZoneLineSize(i), apRLEZones[i]);
-		tagWriteNext();
-	}
-	tagWriteLeave(0x10);
-
-	tagWriteEnter(0x11, gwNumZones); // gateway equivalency zones
-	for (i = 0; i < gwNumZones; i++)
-	{
-		tagWrite(0x01, aNumEquiv[i]);
-		if (aNumEquiv[i])
-		{
-			// another ugly blob job
-			tagWrite8v(0x02, aNumEquiv[i], apEquivZones[i]);
-		}
-		tagWriteNext();
-	}
-	tagWriteLeave(0x11);
-
 	tagClose();
 	return true;
 }
@@ -1200,10 +1038,8 @@ BOOL mapSave(char **ppFileData, UDWORD *pFileSize)
 	GATEWAY *psCurrGate = NULL;
 	GATEWAY_SAVEHEADER *psGateHeader = NULL;
 	GATEWAY_SAVE *psGate = NULL;
-	ZONEMAP_SAVEHEADER *psZoneHeader = NULL;
-	UBYTE *psZone = NULL;
-	UBYTE *psLastZone = NULL;
 	SDWORD	numGateways = 0;
+	ZONEMAP_SAVEHEADER *psZoneHeader = NULL;
 
 	// find the number of non water gateways
 	for(psCurrGate = gwGetGateways(); psCurrGate; psCurrGate = psCurrGate->psNext)
@@ -1218,16 +1054,8 @@ BOOL mapSave(char **ppFileData, UDWORD *pFileSize)
 	*pFileSize = SAVE_HEADER_SIZE + mapWidth*mapHeight * SAVE_TILE_SIZE;
 	// Add on the size of the gateway data.
 	*pFileSize += sizeof(GATEWAY_SAVEHEADER) + sizeof(GATEWAY_SAVE)*numGateways;
-	// Add on the size of the zone data header.
+	// Add on the size of the zone data header. For backwards compatibility.
 	*pFileSize += sizeof(ZONEMAP_SAVEHEADER);
-	// Add on the size of the zone data.
-	for(i=0; i<gwNumZoneLines(); i++) {
-		*pFileSize += 2+gwZoneLineSize(i);
-	}
-	// Add on the size of the equivalency lists.
-	for(i=0; i<(UDWORD)gwNumZones; i++) {
-		*pFileSize += 1+aNumEquiv[i];
-	}
 
 	*ppFileData = (char*)malloc(*pFileSize);
 	if (*ppFileData == NULL)
@@ -1257,13 +1085,10 @@ BOOL mapSave(char **ppFileData, UDWORD *pFileSize)
 	psTile = psMapTiles;
 	for(i=0; i<mapWidth*mapHeight; i++)
 	{
-
-		// don't save the noblock flag as it gets set again when the objects are loaded
-		psTileData->texture = (UWORD)(psTile->texture & (UWORD)~TILE_NOTBLOCKING);
-
+		psTileData->texture = psTile->texture;
 		psTileData->height = psTile->height;
 
-		/* MAP_SAVETILEV2 */
+		/* MAP_SAVETILE */
 		endian_uword(&psTileData->texture);
 
 		psTileData = (MAP_SAVETILE *)((UBYTE *)psTileData + SAVE_TILE_SIZE);
@@ -1298,9 +1123,9 @@ BOOL mapSave(char **ppFileData, UDWORD *pFileSize)
 
 	// Put the zone header.
 	psZoneHeader = (ZONEMAP_SAVEHEADER*)psGate;
-	psZoneHeader->version = 2;
-	psZoneHeader->numZones =(UWORD)gwNumZoneLines();
-	psZoneHeader->numEquivZones =(UWORD)gwNumZones;
+	psZoneHeader->version = 3;
+	psZoneHeader->numZones = 0;
+	psZoneHeader->numEquivZones = 0;
 
 	/* ZONEMAP_SAVEHEADER */
 	endian_uword(&psZoneHeader->version);
@@ -1308,33 +1133,6 @@ BOOL mapSave(char **ppFileData, UDWORD *pFileSize)
 	endian_uword(&psZoneHeader->numEquivZones);
 	endian_uword(&psZoneHeader->pad);
 
-	// Put the zone data.
-	psZone = (UBYTE*)(psZoneHeader+1);
-	for(i=0; i<gwNumZoneLines(); i++) {
-		psLastZone = psZone;
-		*((UWORD*)psZone) = (UWORD)gwZoneLineSize(i);
-		endian_uword(((UWORD *) psZone));
-
-		psZone += sizeof(UWORD);
-		memcpy(psZone,apRLEZones[i],gwZoneLineSize(i));
-		psZone += gwZoneLineSize(i);
-	}
-
-	// Put the equivalency lists.
-	if(gwNumZones > 0) {
-		for(i=0; i<(UDWORD)gwNumZones; i++) {
-			psLastZone = psZone;
-			*psZone = aNumEquiv[i];
-			psZone ++;
-			if(aNumEquiv[i]) {
-				memcpy(psZone,apEquivZones[i],aNumEquiv[i]);
-				psZone += aNumEquiv[i];
-			}
-		}
-	}
-
-	ASSERT( (intptr_t)psLastZone - (intptr_t)*ppFileData < (intptr_t)*pFileSize, "Buffer overflow saving map" );
-
 	return true;
 }
 
@@ -1361,8 +1159,8 @@ extern SWORD map_Height(int x, int y)
 	ASSERT(x >= 0, "map_Height: Negative x value");
 	ASSERT(y >= 0, "map_Height: Negative y value");
 
-	x = x >= world_coord(mapWidth) ? world_coord(mapWidth - 1) : x;
-	y = y >= world_coord(mapHeight) ? world_coord(mapHeight - 1) : y;
+	x = (x >= world_coord(mapWidth) ? world_coord(mapWidth - 1) : x);
+	y = (y >= world_coord(mapHeight) ? world_coord(mapHeight - 1) : y);
 
 	/* Turn into tile coordinates */
 	tileX = map_coord(x);
@@ -1663,7 +1461,7 @@ bool readVisibilityData(const char* fileName)
 	{
 		PHYSFS_close(fileHandle);
 		ASSERT(!"readVisibilityData: unexpected filesize", "readVisibilityData: unexpected filesize; should be %u, but is %u", expectedFileSize, fileSize);
-		abort();
+
 		return false;
 	}
 
@@ -1689,38 +1487,44 @@ bool readVisibilityData(const char* fileName)
 static void astarTest(const char *name, int x1, int y1, int x2, int y2)
 {
 	int		asret, i;
-	ASTAR_ROUTE	route;
+	MOVE_CONTROL	route;
 	int		x = world_coord(x1);
 	int		y = world_coord(y1);
 	int		endx = world_coord(x2);
 	int		endy = world_coord(y2);
 	clock_t		stop;
 	clock_t		start = clock();
-	int		iterations;
 	bool		retval;
 
 	retval = levLoadData(name, NULL, 0);
 	ASSERT(retval, "Could not load %s", name);
-	fpathInitialise();
+	route.asPath = NULL;
 	for (i = 0; i < 100; i++)
 	{
-		iterations = 1;
+		PATHJOB job;
+
 		route.numPoints = 0;
 		astarResetCounters();
-		ASSERT(astarInner == 0, "astarInner not reset");
-		asret = fpathAStarRoute(ASR_NEWROUTE, &route, x, y, endx, endy);
-		while (asret == ASR_PARTIAL)
-		{
-			astarResetCounters();
-			ASSERT(astarInner == 0, "astarInner not reset");
-			asret = fpathAStarRoute(ASR_CONTINUE, &route, x, y, endx, endy);
-			iterations++;
-		}
+		job.origX = x;
+		job.origY = y;
+		job.destX = endx;
+		job.destY = endy;
+		job.propulsion = PROPULSION_TYPE_WHEELED;
+		job.droidID = 1;
+		asret = fpathAStarRoute(&route, &job);
+		free(route.asPath);
+		route.asPath = NULL;
 	}
 	stop = clock();
-	fprintf(stdout, "\t\tPath-finding timing %s: %.02f (%d nodes, %d iterations)\n", name,
-	        (double)(stop - start) / (double)CLOCKS_PER_SEC, route.numPoints, iterations);
-	levReleaseAll();
+	fprintf(stdout, "\t\tA* timing %s: %.02f (%d nodes)\n", name,
+	        (double)(stop - start) / (double)CLOCKS_PER_SEC, route.numPoints);
+	start = clock();
+	fpathTest(x, y, endx, endy);
+	stop = clock();
+	fprintf(stdout, "\t\tfPath timing %s: %.02f (%d nodes)\n", name,
+	        (double)(stop - start) / (double)CLOCKS_PER_SEC, route.numPoints);
+	retval = levReleaseAll();
+	assert(retval);
 }
 
 void mapTest()
@@ -1732,3 +1536,15 @@ void mapTest()
 
 	fprintf(stdout, "\tMap self-test: PASSED\n");
 }
+
+
+/** Check if tile contained within the given world coordinates is burning. */
+bool fireOnLocation(unsigned int x, unsigned int y)
+{
+	const int posX = map_coord(x);
+	const int posY = map_coord(y);
+	const MAPTILE *psTile = mapTile(posX, posY);
+
+	ASSERT(psTile, "Checking fire on tile outside the map (%d, %d)", posX, posY);
+	return psTile != NULL && TileIsBurning(psTile);
+}
