@@ -1,7 +1,7 @@
 /*
 	This file is part of Warzone 2100.
 	Copyright (C) 1999-2004  Eidos Interactive
-	Copyright (C) 2005-2007  Warzone Resurrection Project
+	Copyright (C) 2005-2009  Warzone Resurrection Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -56,7 +56,7 @@
 // function definitions
 
 static BOOL sendStructureCheck	(void);							//Structure
-static void packageCheck		(DROID *pD);
+static void packageCheck		(const DROID* pD);
 static BOOL sendDroidCheck		(void);							//droids
 
 static void highLevelDroidUpdate(DROID *psDroid,
@@ -87,8 +87,8 @@ static UDWORD averagePing(void);
 // Defined numeric values
 #define AV_PING_FREQUENCY	45000					// how often to update average pingtimes. in approx millisecs.
 #define PING_FREQUENCY		12000					// how often to update pingtimes. in approx millisecs.
-#define STRUCT_FREQUENCY	700						// how often (ms) to send a structure check.
-#define DROID_FREQUENCY		1000					// how ofter (ms) to send droid checks
+#define STRUCT_FREQUENCY	450						// how often (ms) to send a structure check.
+#define DROID_FREQUENCY		300						// how ofter (ms) to send droid checks
 #define POWER_FREQUENCY		10000					// how often to send power levels
 #define SCORE_FREQUENCY		25000					// how often to update global score.
 
@@ -137,18 +137,35 @@ BOOL sendCheck(void)
 	{
 		sendStructureCheck();
 	}
+	else
+	{
+		debug(LOG_SYNC, "Couldn't sendStructureCheck()");
+	}
 	if(okToSend())
 	{
 		sendPowerCheck();
 	}
+	else
+	{
+		debug(LOG_SYNC, "Couldn't sendPowerCheck()");
+	}
 	if(okToSend())
 	{
 		sendScoreCheck();
 	}
+	else
+	{
+		debug(LOG_SYNC, "Couldn't sendScoreCheck()");
+	}
 	if(okToSend())
 	{
 		sendDroidCheck();
 	}
+	else
+	{
+		debug(LOG_SYNC, "Couldn't sendDroidCheck()");
+	}
+
 	return true;
 }
 
@@ -205,15 +222,19 @@ static DROID* pickADroid(void)
 
 	return pD;
 }
-// ///////////////////////////////////////////////////////////////////////////
-// force a droid sync
-// We need to update the droid in question ASAP, to prevent 'lost' droids until
-// the next 'real' update is done.
-BOOL ForceDroidSync( DROID* droidToSend )
+
+/** Force a droid to be synced
+ *
+ *  Call this when you need to update the given droid right now.
+ */
+BOOL ForceDroidSync(const DROID* droidToSend)
 {
 	uint8_t count = 1;		// *always* one
-	if(!droidToSend) return false;
-	debug(LOG_SYNC,"Force sync of droid %d from player %d",droidToSend->id,droidToSend->player);
+
+	ASSERT(droidToSend != NULL, "NULL pointer passed");
+
+	debug(LOG_SYNC, "Force sync of droid %u from player %u", droidToSend->id, droidToSend->player);
+
 	NETbeginEncode(NET_CHECK_DROID, NET_ALL_PLAYERS);
 		NETuint8_t(&count);
 		packageCheck(droidToSend);
@@ -258,6 +279,11 @@ static BOOL sendDroidCheck(void)
 			{
 				ppD[count++] = pD;
 			}
+			// All droids are synced! (We're done.)
+			else
+			{
+				break;
+			}
 		}
 
 		// Send the number of droids to expect
@@ -274,25 +300,34 @@ static BOOL sendDroidCheck(void)
 
 // ////////////////////////////////////////////////////////////////////////////
 // Send a Single Droid Check message
-static void packageCheck(DROID *pD)
+static void packageCheck(const DROID* pD)
 {
+	// Copy these variables so that we don't have to violate pD's constness
+	uint8_t player = pD->player;
+	uint32_t droidID = pD->id;
+	int32_t order = pD->order;
+	uint32_t secondaryOrder = pD->secondaryOrder;
+	uint32_t body = pD->body;
+	float direction = pD->direction;
+	float experience = pD->experience;
+
 	// Send the player to which the droid belongs
-	NETuint8_t(&pD->player);
+	NETuint8_t(&player);
 
 	// Now the droid's ID
-	NETuint32_t(&pD->id);
+	NETuint32_t(&droidID);
 
 	// The droid's order
-	NETint32_t(&pD->order);
+	NETint32_t(&order);
 
 	// The droids secondary order
-	NETuint32_t(&pD->secondaryOrder);
+	NETuint32_t(&secondaryOrder);
 
 	// Droid's current HP
-	NETuint32_t(&pD->body);
+	NETuint32_t(&body);
 
 	// Direction it is going in
-	NETfloat(&pD->direction);
+	NETfloat(&direction);
 
 	// Fractional move
 	if (pD->order == DORDER_ATTACK
@@ -300,29 +335,40 @@ static void packageCheck(DROID *pD)
 	 || pD->order == DORDER_RTB
 	 || pD->order == DORDER_RTR)
 	{
-		NETfloat(&pD->sMove.fx);
-		NETfloat(&pD->sMove.fy);
+		float sMoveX = pD->sMove.fx;
+		float sMoveY = pD->sMove.fy;
+
+		NETfloat(&sMoveX);
+		NETfloat(&sMoveY);
 	}
 	// Non-fractional move, send regular coords
 	else
 	{
-		NETuint16_t(&pD->pos.x);
-		NETuint16_t(&pD->pos.y);
+		uint16_t posX = pD->pos.x;
+		uint16_t posY = pD->pos.y;
+
+		NETuint16_t(&posX);
+		NETuint16_t(&posY);
 	}
 
 
 	if (pD->order == DORDER_ATTACK)
 	{
-		NETuint32_t(&pD->psTarget->id);
+		uint32_t targetID = pD->psTarget->id;
+
+		NETuint32_t(&targetID);
 	}
 	else if (pD->order == DORDER_MOVE)
 	{
-		NETuint16_t(&pD->orderX);
-		NETuint16_t(&pD->orderY);
+		uint16_t orderX = pD->orderX;
+		uint16_t orderY = pD->orderY;
+
+		NETuint16_t(&orderX);
+		NETuint16_t(&orderY);
 	}
 
 	// Last send the droid's experience
-	NETfloat(&pD->experience);
+	NETfloat(&experience);
 }
 
 
@@ -424,9 +470,7 @@ BOOL recvDroidCheck()
 			 * and the player who owns the droid has a low ping then do an
 			 * onscreen update, otherwise do an offscreen one.
 			 */
-			if (DrawnInLastFrame(pD->sDisplay.frameNumber)
-			 && pD->sDisplay.screenX < pie_GetVideoBufferWidth()
-			 && pD->sDisplay.screenY < pie_GetVideoBufferHeight()
+			if (droidOnScreen(pD, 0)
 			 && ingame.PingTimes[player] < PING_LIMIT)
 			{
 				onscreen = true;
@@ -624,7 +668,7 @@ static void offscreenUpdate(DROID *psDroid,
 	// snap droid(if on ground)  to terrain level at x,y.
 	psPropStats = asPropulsionStats + psDroid->asBits[COMP_PROPULSION].nStat;
 	ASSERT( psPropStats != NULL, "offscreenUpdate: invalid propulsion stats pointer" );
-	if(	psPropStats->propulsionType != LIFT )		// if not airborne.
+	if(	psPropStats->propulsionType != PROPULSION_TYPE_LIFT )		// if not airborne.
 	{
 		psDroid->pos.z = map_Height(psDroid->pos.x, psDroid->pos.y);
 	}
@@ -969,7 +1013,7 @@ BOOL recvPowerCheck()
 	power2 = getPower(player);
 	if (power != power2)
 	{
-		debug(LOG_NET, "NET_CHECK_POWER: Adjusting power for player %d (%s) from %u to %u",
+		debug(LOG_SYNC, "NET_CHECK_POWER: Adjusting power for player %d (%s) from %u to %u",
 		      (int)player, isHumanPlayer(player) ? "Human" : "AI", power2, power);
 		setPower(player, power);
 	}
@@ -1109,7 +1153,7 @@ static UDWORD averagePing(void)
 			count ++;
 		}
 	}
-	return total/count;
+	return total / MAX(count, 1);
 }
 
 BOOL sendPing(void)
