@@ -1,7 +1,7 @@
 /*
 	This file is part of Warzone 2100.
 	Copyright (C) 1999-2004  Eidos Interactive
-	Copyright (C) 2005-2007  Warzone Resurrection Project
+	Copyright (C) 2005-2009  Warzone Resurrection Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -28,7 +28,7 @@
 
 // --------------------------------------------------------------------
 #include "lib/framework/frame.h"
-#include "lib/framework/math-help.h"
+#include "lib/framework/math_ext.h"
 #include "lib/framework/strres.h"
 #include "lib/framework/tagfile.h"
 #include "lib/gamelib/gtime.h"
@@ -68,7 +68,7 @@
 #define	RANK_BAR_WIDTH	100
 #define STAT_BAR_WIDTH	100
 
-enum
+typedef enum
 {
 	STR_MR_UNITS_LOST,
 	STR_MR_UNITS_KILLED,
@@ -88,13 +88,13 @@ enum
 	STR_MR_LEVEL_ELITE,
 	STR_MR_LEVEL_SPECIAL,
 	STR_MR_LEVEL_ACE
-};
+} MR_STRING;
 
 
 // return translated string
-static const char *getDescription(int id)
+static const char *getDescription(MR_STRING id)
 {
-	switch ( id )
+	switch (id)
 	{
 		case STR_MR_UNITS_LOST   : return _("Own Units: %u");
 		case STR_MR_UNITS_KILLED : return _("Enemy Units: %u");
@@ -114,9 +114,10 @@ static const char *getDescription(int id)
 		case STR_MR_LEVEL_ELITE  : return _("Elite: %u");
 		case STR_MR_LEVEL_SPECIAL: return _("Special: %u");
 		case STR_MR_LEVEL_ACE    : return _("Hero: %u");
-
-		default: return "";  // make compiler shut up
 	}
+
+	// make compiler shut up
+	return "";
 }
 
 static PIELIGHT getColour(int id)
@@ -188,8 +189,7 @@ static	UDWORD	dispST;
 static	BOOL	bDispStarted = false;
 static	char	text[255];
 static	char	text2[255];
-
-
+extern bool Cheated;
 // --------------------------------------------------------------------
 /* Initialise the mission data info - done before each mission */
 BOOL	scoreInitSystem( void )
@@ -207,6 +207,7 @@ BOOL	scoreInitSystem( void )
 	missionData.shotsOnTarget	= 0;
 	missionData.shotsOffTarget	= 0;
 	missionData.babasMowedDown	= 0;
+	Cheated = false;
 	bDispStarted = false;
 	return(true);
 }
@@ -344,8 +345,8 @@ UDWORD	width,height;
 
 	fillUpStats();
 
-	pie_UniTransBoxFill(16 + D_W, MT_Y_POS - 16, pie_GetVideoBufferWidth() - D_W - 16, MT_Y_POS + 256, WZCOL_SCORE_BOX);
-	iV_Box(16 + D_W, MT_Y_POS - 16, pie_GetVideoBufferWidth() - D_W - 16, MT_Y_POS + 256, WZCOL_SCORE_BOX_BORDER);
+	pie_UniTransBoxFill(16 + D_W, MT_Y_POS - 16, pie_GetVideoBufferWidth() - D_W - 16, MT_Y_POS + 256+16, WZCOL_SCORE_BOX);
+	iV_Box(16 + D_W, MT_Y_POS - 16, pie_GetVideoBufferWidth() - D_W - 16, MT_Y_POS + 256+16, WZCOL_SCORE_BOX_BORDER);
 
 	iV_DrawText( _("Unit Losses"), LC_X + D_W, 80 + 16 + D_H );
 	iV_DrawText( _("Structure Losses"), LC_X + D_W, 140 + 16 + D_H );
@@ -441,6 +442,14 @@ void	dispAdditionalInfo( void )
 	getAsciiTime( (char*)&text2, gameTime );
 	sprintf( text, _("Total Game Time - %s"), text2 );
 	iV_DrawText( text, (pie_GetVideoBufferWidth() - iV_GetTextWidth(text))/2, 340 + D_H );
+	if (Cheated)
+	{
+		// A quick way to flash the text
+		((gameTime2 / 250) % 2) ? iV_SetTextColour(WZCOL_RED) : iV_SetTextColour(WZCOL_YELLOW);
+		sprintf( text, _("You cheated!"));
+		iV_DrawText( text, (pie_GetVideoBufferWidth() - iV_GetTextWidth(text))/2, 360 + D_H );
+		iV_SetTextColour(WZCOL_TEXT_BRIGHT);
+	}
 }
 
 // -----------------------------------------------------------------------------------
