@@ -1,7 +1,7 @@
 /*
 	This file is part of Warzone 2100.
 	Copyright (C) 1999-2004  Eidos Interactive
-	Copyright (C) 2005-2007  Warzone Resurrection Project
+	Copyright (C) 2005-2009  Warzone Resurrection Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -23,14 +23,18 @@
  * Functions for the display of the Escape Sequences (FMV).
  *
  */
+
 #include "lib/framework/frame.h"
+#include "lib/framework/frameint.h"
 
 #include <string.h>
 #include <SDL_timer.h>
 #include <physfs.h>
 
 #include "lib/framework/file.h"
+#include "lib/framework/stdio_ext.h"
 #include "lib/ivis_common/rendmode.h"
+#include "lib/ivis_common/piemode.h"
 #include "lib/ivis_opengl/screen.h"
 #include "lib/sequence/sequence.h"
 #include "lib/sound/audio.h"
@@ -53,16 +57,20 @@
  *	local Definitions
  */
 /***************************************************************************/
-#define STD_FRAME_TIME 40 //milliseconds
-#define VIDEO_PLAYBACK_WIDTH 640
-#define VIDEO_PLAYBACK_HEIGHT 480
 #define MAX_TEXT_OVERLAYS 32
 #define MAX_SEQ_LIST	  6
 #define SUBTITLE_BOX_MIN 430
 #define SUBTITLE_BOX_MAX 480
 
+// NOTE: The original game never had a true fullscreen mode for FMVs on >640x480 screens.
+// They would just use double sized videos, and move the text to that area.
+// Since we *do* offer fullscreen FMVs, this isn't really needed anymore, depending
+// on how we want to handle the text.
+static int D_W2 = 0;	// Text width offset
+static int D_H2 = 0;	// Text height offset
 
-typedef struct {
+typedef struct
+{
 	char pText[MAX_STR_LENGTH];
 	UDWORD x;
 	UDWORD y;
@@ -71,7 +79,8 @@ typedef struct {
 	BOOL	bSubtitle;
 } SEQTEXT;
 
-typedef struct {
+typedef struct
+{
 	const char	*pSeq;						//name of the sequence to play
 	const char	*pAudio;					//name of the wav to play
 	BOOL		bSeqLoop;					//loop this sequence
@@ -87,26 +96,33 @@ typedef struct {
 static BOOL bBackDropWasAlreadyUp;
 static BOOL bAudioPlaying = false;
 static BOOL bHoldSeqForAudio = false;
-static BOOL bCDPath = false;
-static BOOL bHardPath = false;
 static BOOL bSeqSubtitles = true;
-static char aHardPath[MAX_STR_LENGTH];
+static BOOL bSeqPlaying = false;
+static const char aHardPath[] = "sequences/";
 static char aVideoName[MAX_STR_LENGTH];
+static char* pVideoBuffer = NULL;
+static char* pVideoPalette = NULL;
+static SDWORD frameSkip = 1;
 static SEQLIST aSeqList[MAX_SEQ_LIST];
 static SDWORD currentSeq = -1;
 static SDWORD currentPlaySeq = -1;
 static BOOL g_bResumeInGame = false;
 
-static unsigned int time_started = 0;
-
 /***************************************************************************/
 /*
  *	local ProtoTypes
  */
 /***************************************************************************/
 
-static void	seq_SetVideoPath(void);
-static BOOL seq_StartFullScreenVideo(const char* videoName, const char* audioName);
+typedef enum
+{
+	VIDEO_PRESELECTED_RESOLUTION,
+	VIDEO_USER_CHOSEN_RESOLUTION,
+} VIDEO_RESOLUTION;
+
+static bool seq_StartFullScreenVideo(const char* videoName, const char* audioName, VIDEO_RESOLUTION resolution);
+BOOL seq_SetupVideoBuffers(void);
+BOOL seq_ReleaseVideoBuffers(void);
 
 /***************************************************************************/
 /*
@@ -115,65 +131,150 @@ static BOOL seq_StartFullScreenVideo(const char* videoName, const char* audioNam
 /***************************************************************************/
 
  /* Renders a video sequence specified by filename to a buffer*/
-BOOL	seq_RenderVideoToBuffer( iSurface *pSurface, char *sequenceName, int time, int seqCommand)
+bool seq_RenderVideoToBuffer(const char* sequenceName, int seqCommand)
 {
+	static enum
+	{
+		VIDEO_NOT_PLAYING,
+		VIDEO_PLAYING,
+		VIDEO_FINISHED,
+	} videoPlaying = VIDEO_NOT_PLAYING;
+	static enum
+	{
+		VIDEO_LOOP,
+		VIDEO_HOLD_LAST_FRAME,
+	} frameHold = VIDEO_LOOP;
+
 	if (seqCommand == SEQUENCE_KILL)
 	{
 		//stop the movie
 		seq_Shutdown();
+		bSeqPlaying = false;
+		frameHold = VIDEO_LOOP;
+		videoPlaying = VIDEO_NOT_PLAYING;
 		return true;
 	}
 
-	seq_StartFullScreenVideo(sequenceName, NULL);
+	if (!bSeqPlaying
+	 && frameHold == VIDEO_LOOP)
+	{
+		//start the ball rolling
+
+		iV_SetFont(font_regular);
+		iV_SetTextColour(WZCOL_TEXT_BRIGHT);
+
+		/* We do *NOT* want to use the user-choosen resolution when we
+		 * are doing intelligence videos.
+		 */
+		videoPlaying = seq_StartFullScreenVideo(sequenceName, NULL, VIDEO_PRESELECTED_RESOLUTION) ? VIDEO_PLAYING : VIDEO_FINISHED;
+		bSeqPlaying = true;
+	}
+
+	if (videoPlaying != VIDEO_FINISHED)
+	{
+		videoPlaying = seq_Update() ? VIDEO_PLAYING : VIDEO_FINISHED;
+	}
+
+	if (videoPlaying == VIDEO_FINISHED)
+	{
+		seq_Shutdown();
+		bSeqPlaying = false;
+		frameHold = VIDEO_HOLD_LAST_FRAME;
+		videoPlaying = VIDEO_NOT_PLAYING;
+		return false;
+	}
 
 	return true;
 }
 
-static void seq_SetVideoPath(void)
+BOOL seq_ReleaseVideoBuffers(void)
 {
-	// now set up the hard disc path /
-	if (!bHardPath)
+	free(pVideoBuffer);
+	free(pVideoPalette);
+	pVideoBuffer = NULL;
+	pVideoPalette = NULL;
+	return true;
+}
+
+BOOL seq_SetupVideoBuffers(void)
+{
+	return true;
+}
+
+static void seq_SetUserResolution(void)
+{
+	switch (war_GetFMVmode())
 	{
-		strcpy(aHardPath, "sequences/");
-		bHardPath=true;			//yes, always true, as it should be on windows ALSO.
+		case FMV_1X:
+		{
+			// Native (1x)
+			const int x = (screenWidth - 320) / 2;
+			const int y = (screenHeight - 240) / 2;
+			seq_SetDisplaySize(320, 240, x, y);
+			break;
+		}
+		case FMV_2X:
+		{
+			// Double (2x)
+			const int x = (screenWidth - 640) / 2;
+			const int y = (screenHeight - 480) / 2;
+			seq_SetDisplaySize(640, 480, x, y);
+			break;
+		}
+		case FMV_FULLSCREEN:
+			seq_SetDisplaySize(screenWidth, screenHeight, 0, 0);
+			break;
+
+		default:
+			ASSERT(!"invalid FMV mode", "Invalid FMV mode: %u", (unsigned int)war_GetFMVmode());
+			break;
 	}
 }
 
 //full screenvideo functions
-static BOOL seq_StartFullScreenVideo(const char* videoName, const char* audioName)
+static bool seq_StartFullScreenVideo(const char* videoName, const char* audioName, VIDEO_RESOLUTION resolution)
 {
-	const char* aAudioName;
+	const char* aAudioName = NULL;
+	int chars_printed;
+
 	bHoldSeqForAudio = false;
 
-	//set a valid video path if there is one
-	if(!bCDPath && !bHardPath)
-	{
-		seq_SetVideoPath();
-	}
-
-	ASSERT( (strlen(videoName) + strlen(aHardPath))<MAX_STR_LENGTH,"sequence path+name greater than max string" );
-	snprintf(aVideoName, sizeof(aVideoName), "%s%s", aHardPath, videoName);
+	chars_printed = ssprintf(aVideoName, "%s%s", aHardPath, videoName);
+	ASSERT(chars_printed < sizeof(aVideoName), "sequence path + name greater than max string");
 
 	//set audio path
 	if (audioName != NULL)
 	{
-		ASSERT( strlen(audioName) < MAX_STR_LENGTH, "sequence path+name greater than max string" );
 		sasprintf((char**)&aAudioName, "sequenceaudio/%s", audioName);
 	}
 
-	//start video mode
-	if (loop_GetVideoMode() == 0)
+	cdAudio_Pause();
+	iV_SetFont(font_regular);
+	iV_SetTextColour(WZCOL_TEXT_BRIGHT);
+
+	/* We do not want to enter loop_SetVideoPlaybackMode() when we are
+	 * doing intelligence videos.
+	 */
+	if (resolution == VIDEO_USER_CHOSEN_RESOLUTION)
 	{
-		cdAudio_Pause();
-		loop_SetVideoPlaybackMode();
-		iV_SetFont(font_regular);
-		iV_SetTextColour(WZCOL_TEXT_BRIGHT);
+		//start video mode
+		if (loop_GetVideoMode() == 0)
+		{
+			// check to see if we need to pause, and set font each time
+			cdAudio_Pause();
+			loop_SetVideoPlaybackMode();
+			iV_SetFont(font_regular);
+			iV_SetTextColour(WZCOL_TEXT_BRIGHT);
+		}
+
+		// set the dimensions to show full screen or native or ...
+		seq_SetUserResolution();
 	}
 
 	if (!seq_Play(aVideoName))
 	{
-		// don't stop or cancel it as we may have text to display
-		debug( LOG_WARNING,"seq_StartFullScreenVideo: unable to initialise sequence %s",aVideoName );
+		seq_Shutdown();
+		return false;
 	}
 
 	if (audioName == NULL)
@@ -182,44 +283,33 @@ static BOOL seq_StartFullScreenVideo(const char* videoName, const char* audioNam
 	}
 	else
 	{
+		// NOT controlled by sliders for now?
 		static const float maxVolume = 1.f;
 
 		bAudioPlaying = audio_PlayStream(aAudioName, maxVolume, NULL, NULL) ? true : false;
-		ASSERT(bAudioPlaying == true, "seq_StartFullScreenVideo: unable to initialise sound %s", aAudioName);
+		ASSERT(bAudioPlaying == true, "unable to initialise sound %s", aAudioName);
 	}
 
-	time_started = SDL_GetTicks();
-
 	return true;
 }
 
 BOOL seq_UpdateFullScreenVideo(int *pbClear)
 {
-	SDWORD i, x, y, w = 640, h = 480;
-	UDWORD subMin, subMax;
-	UDWORD realFrame;
+	int i;
 	BOOL bMoreThanOneSequenceLine = false;
-	BOOL stillText;
+	bool stillPlaying;
 
-	x = (pie_GetVideoBufferWidth() - w)/2;
-	y = (pie_GetVideoBufferHeight() - h)/2;
+	unsigned int subMin = SUBTITLE_BOX_MAX + D_H2;
+	unsigned int subMax = SUBTITLE_BOX_MIN + D_H2;
 
-	subMin = SUBTITLE_BOX_MAX + D_H;
-	subMax = SUBTITLE_BOX_MIN + D_H;
-
-	stillText = false;
 	//get any text lines over bottom of the video
-	realFrame = (SDL_GetTicks()-time_started)/40; // standard frame is 40 msec
+	unsigned int realFrame = seq_GetFrameNumber();
 	for(i=0;i<MAX_TEXT_OVERLAYS;i++)
 	{
-		if (aSeqList[currentPlaySeq].aText[i].pText[0] != 0)
+		if (aSeqList[currentPlaySeq].aText[i].pText[0] != '\0')
 		{
-			if (aSeqList[currentPlaySeq].aText[i].bSubtitle == true)
+			if (aSeqList[currentPlaySeq].aText[i].bSubtitle)
 			{
-				if(realFrame <= aSeqList[currentPlaySeq].aText[i].endFrame)
-				{
-					stillText = true;
-				}
 				if ((realFrame >= aSeqList[currentPlaySeq].aText[i].startFrame) && (realFrame <= aSeqList[currentPlaySeq].aText[i].endFrame))
 				{
 					if (subMin > aSeqList[currentPlaySeq].aText[i].y)
@@ -249,11 +339,20 @@ BOOL seq_UpdateFullScreenVideo(int *pbClear)
 					}
 				}
 			}
+
+			if (realFrame >= aSeqList[currentPlaySeq].aText[i].endFrame
+			 && realFrame < aSeqList[currentPlaySeq].aText[i].endFrame + frameSkip)
+			{
+				if (pbClear != NULL)
+				{
+					*pbClear = CLEAR_BLACK;
+				}
+			}
 		}
 	}
 
-	subMin -= D_H;//adjust video window here because text is already ofset for big screens
-	subMax -= D_H;
+	subMin -= D_H2;//adjust video window here because text is already ofset for big screens
+	subMax -= D_H2;
 
 	if (subMin < SUBTITLE_BOX_MIN)
 	{
@@ -270,21 +369,19 @@ BOOL seq_UpdateFullScreenVideo(int *pbClear)
 	}
 
 	//call sequence player to download last frame
-	if(seq_Playing())
-	{
-		seq_Update();
-	}
-
+	stillPlaying = seq_Update();
 	//print any text over the video
+	realFrame = seq_GetFrameNumber();//textFrame + 1;
+
 	for(i=0;i<MAX_TEXT_OVERLAYS;i++)
 	{
-		if (aSeqList[currentPlaySeq].aText[i].pText[0] != 0)
+		if (aSeqList[currentPlaySeq].aText[i].pText[0] != '\0')
 		{
 			if ((realFrame >= aSeqList[currentPlaySeq].aText[i].startFrame) && (realFrame <= aSeqList[currentPlaySeq].aText[i].endFrame))
 			{
 				if (bMoreThanOneSequenceLine)
 				{
-					aSeqList[currentPlaySeq].aText[i].x = 20 + D_W;
+					aSeqList[currentPlaySeq].aText[i].x = 20 + D_W2;
 				}
 				iV_DrawText(&(aSeqList[currentPlaySeq].aText[i].pText[0]),
 						aSeqList[currentPlaySeq].aText[i].x, aSeqList[currentPlaySeq].aText[i].y);
@@ -293,7 +390,7 @@ BOOL seq_UpdateFullScreenVideo(int *pbClear)
 			{
 				if (bMoreThanOneSequenceLine)
 				{
-					aSeqList[currentPlaySeq].aText[i].x = 20 + D_W;
+					aSeqList[currentPlaySeq].aText[i].x = 20 + D_W2;
 				}
 				iV_DrawText(&(aSeqList[currentPlaySeq].aText[i].pText[0]),
 						aSeqList[currentPlaySeq].aText[i].x, aSeqList[currentPlaySeq].aText[i].y);
@@ -301,14 +398,15 @@ BOOL seq_UpdateFullScreenVideo(int *pbClear)
 
 		}
 	}
-
-	if ((!seq_Playing() && !stillText) || (bHoldSeqForAudio))
+	if (!stillPlaying
+	 || bHoldSeqForAudio)
 	{
 		if (bAudioPlaying)
 		{
 			if (aSeqList[currentPlaySeq].bSeqLoop)
 			{
 				seq_Shutdown();
+
 				if (!seq_Play(aVideoName))
 				{
 					bHoldSeqForAudio = true;
@@ -325,6 +423,7 @@ BOOL seq_UpdateFullScreenVideo(int *pbClear)
 			return false;//should terminate the video
 		}
 	}
+
 	return true;
 }
 
@@ -351,22 +450,18 @@ BOOL seq_StopFullScreenVideo(void)
 	return true;
 }
 
-#define BUFFER_WIDTH 600
-#define FOLLOW_ON_JUSTIFICATION 160
-#define MIN_JUSTIFICATION 40
-
 // add a string at x,y or add string below last line if x and y are 0
-BOOL seq_AddTextForVideo(char* pText, SDWORD xOffset, SDWORD yOffset, SDWORD startFrame, SDWORD endFrame, SDWORD bJustify)
+BOOL seq_AddTextForVideo(const char* pText, SDWORD xOffset, SDWORD yOffset, SDWORD startFrame, SDWORD endFrame, SEQ_TEXT_POSITIONING textJustification)
 {
 	SDWORD sourceLength, currentLength;
 	char* currentText;
-	SDWORD justification;
 	static SDWORD lastX;
+	// make sure we take xOffset into account, we don't always start at 0
+	const unsigned int BUFFER_WIDTH = pie_GetVideoBufferWidth() - xOffset;
 
 	iV_SetFont(font_regular);
 
-	ASSERT( aSeqList[currentSeq].currentText < MAX_TEXT_OVERLAYS,
-		"seq_AddTextForVideo: too many text lines" );
+	ASSERT(aSeqList[currentSeq].currentText < MAX_TEXT_OVERLAYS, "too many text lines");
 
 	sourceLength = strlen(pText);
 	currentLength = sourceLength;
@@ -389,7 +484,7 @@ BOOL seq_AddTextForVideo(char* pText, SDWORD xOffset, SDWORD yOffset, SDWORD sta
 
 	//check the string is shortenough to print
 	//if not take a word of the end and try again
-	while(iV_GetTextWidth(currentText) > BUFFER_WIDTH)
+	while (iV_GetTextWidth(currentText) > BUFFER_WIDTH)
 	{
 		currentLength--;
 		while((pText[currentLength] != ' ') && (currentLength > 0))
@@ -410,30 +505,35 @@ BOOL seq_AddTextForVideo(char* pText, SDWORD xOffset, SDWORD yOffset, SDWORD sta
 	}
 	else
 	{
-		aSeqList[currentSeq].aText[aSeqList[currentSeq].currentText].x = xOffset + D_W;
-		aSeqList[currentSeq].aText[aSeqList[currentSeq].currentText].y = yOffset + D_H;
+		aSeqList[currentSeq].aText[aSeqList[currentSeq].currentText].x = xOffset + D_W2;
+		aSeqList[currentSeq].aText[aSeqList[currentSeq].currentText].y = yOffset + D_H2;
 	}
 	lastX = aSeqList[currentSeq].aText[aSeqList[currentSeq].currentText].x;
 
-	if ((bJustify) && (currentLength == sourceLength))
+	if (textJustification
+	 && currentLength == sourceLength)
 	{
+		static const int MIN_JUSTIFICATION = 40;
+		static const int FOLLOW_ON_JUSTIFICATION = 160;
 		//justify this text
-		justification = BUFFER_WIDTH - iV_GetTextWidth(currentText);
-		if ((bJustify == SEQ_TEXT_JUSTIFY) && (justification > MIN_JUSTIFICATION))
+		const int justification = BUFFER_WIDTH - iV_GetTextWidth(currentText);
+
+		if (textJustification == SEQ_TEXT_JUSTIFY
+		 && justification > MIN_JUSTIFICATION)
 		{
-			aSeqList[currentSeq].aText[aSeqList[currentSeq].currentText].x += (justification/2);
+			aSeqList[currentSeq].aText[aSeqList[currentSeq].currentText].x += (justification / 2);
 		}
-		else if ((bJustify == SEQ_TEXT_FOLLOW_ON) && (justification > FOLLOW_ON_JUSTIFICATION))
+		else if (textJustification == SEQ_TEXT_FOLLOW_ON
+		      && justification > FOLLOW_ON_JUSTIFICATION)
 		{
 
 		}
 	}
 
-
 	//set start and finish times for the objects
 	aSeqList[currentSeq].aText[aSeqList[currentSeq].currentText].startFrame = startFrame;
 	aSeqList[currentSeq].aText[aSeqList[currentSeq].currentText].endFrame = endFrame;
-	aSeqList[currentSeq].aText[aSeqList[currentSeq].currentText].bSubtitle = bJustify;
+	aSeqList[currentSeq].aText[aSeqList[currentSeq].currentText].bSubtitle = textJustification;
 
 	aSeqList[currentSeq].currentText++;
 	if (aSeqList[currentSeq].currentText >= MAX_TEXT_OVERLAYS)
@@ -445,11 +545,11 @@ BOOL seq_AddTextForVideo(char* pText, SDWORD xOffset, SDWORD yOffset, SDWORD sta
 	if (currentLength < sourceLength)
 	{
 		//RECURSE x= 0 y = 0 for nextLine
-		if (bJustify == SEQ_TEXT_JUSTIFY)
+		if (textJustification == SEQ_TEXT_JUSTIFY)
 		{
-			bJustify = SEQ_TEXT_POSITION;
+			textJustification = SEQ_TEXT_POSITION;
 		}
-		seq_AddTextForVideo(&pText[currentLength + 1], 0, 0, startFrame, endFrame, bJustify);
+		seq_AddTextForVideo(&pText[currentLength + 1], 0, 0, startFrame, endFrame, textJustification);
 	}
 	return true;
 }
@@ -474,7 +574,7 @@ BOOL seq_ClearTextForVideo(void)
 	return true;
 }
 
-static BOOL seq_AddTextFromFile(const char *pTextName, BOOL bJustify)
+static BOOL seq_AddTextFromFile(const char *pTextName, SEQ_TEXT_POSITIONING textJustification)
 {
 	char aTextName[MAX_STR_LENGTH];
 	char *pTextBuffer, *pCurrentLine, *pText;
@@ -482,7 +582,13 @@ static BOOL seq_AddTextFromFile(const char *pTextName, BOOL bJustify)
 	SDWORD xOffset, yOffset, startFrame, endFrame;
 	const char *seps = "\n";
 
-	snprintf(aTextName, sizeof(aTextName), "sequenceaudio/%s", pTextName);
+	// NOTE: The original game never had a fullscreen mode for FMVs on >640x480 screens.
+	// They would just use double sized videos, and move the text to that area.
+	// We just use the full screen for text right now, instead of using offsets.
+	// However, depending on reaction, we may use the old style again.
+	D_H2 = 0;				//( pie_GetVideoBufferHeight()- 480)/2;
+	D_W2 = 0;				//( pie_GetVideoBufferWidth() - 640)/2;
+	ssprintf(aTextName, "sequenceaudio/%s", pTextName);
 
 	if (loadFileToBufferNoError(aTextName, fileLoadBuffer, FILE_LOAD_BUFFER_SIZE, &fileSize) == false)  //Did I mention this is lame? -Q
 	{
@@ -497,18 +603,23 @@ static BOOL seq_AddTextFromFile(const char *pTextName, BOOL bJustify)
 		{
 			if (sscanf(pCurrentLine,"%d %d %d %d", &xOffset, &yOffset, &startFrame, &endFrame) == 4)
 			{
+				// Since all the positioning was hardcoded to specific values, we now calculate the
+				// ratio of our screen, compared to what the game expects and multiply that to x, y.
+				// This makes the text always take up the full screen, instead of original style.
+				xOffset = (double)pie_GetVideoBufferWidth() / 640. * (double)xOffset;
+				yOffset = (double)pie_GetVideoBufferHeight() / 480. * (double)yOffset;
 				//get the text
 				pText = strrchr(pCurrentLine,'"');
-				ASSERT( pText != NULL,"seq_AddTextFromFile error parsing text file" );
+				ASSERT(pText != NULL, "error parsing text file");
 				if (pText != NULL)
 				{
 					*pText = (UBYTE)0;
 				}
 				pText = strchr(pCurrentLine,'"');
-				ASSERT( pText != NULL,"seq_AddTextFromFile error parsing text file" );
+				ASSERT(pText != NULL, "error parsing text file");
 				if (pText != NULL)
 				{
-					seq_AddTextForVideo(&pText[1], xOffset, yOffset, startFrame, endFrame, bJustify);
+					seq_AddTextForVideo(&pText[1], xOffset, yOffset, startFrame, endFrame, textJustification);
 				}
 			}
 		}
@@ -535,13 +646,11 @@ void seq_ClearSeqList(void)
 //add a sequence to the list to be played
 void seq_AddSeqToList(const char *pSeqName, const char *pAudioName, const char *pTextName, BOOL bLoop)
 {
-	SDWORD strLen;
 	currentSeq++;
 
-
-	if ((currentSeq) >=  MAX_SEQ_LIST)
+	ASSERT(currentSeq < MAX_SEQ_LIST, "too many sequences");
+	if (currentSeq >=  MAX_SEQ_LIST)
 	{
-		ASSERT( false, "seq_AddSeqToList: too many sequences" );
 		return;
 	}
 
@@ -551,20 +660,27 @@ void seq_AddSeqToList(const char *pSeqName, const char *pAudioName, const char *
 	aSeqList[currentSeq].bSeqLoop = bLoop;
 	if (pTextName != NULL)
 	{
-		seq_AddTextFromFile(pTextName, false);//SEQ_TEXT_POSITION);//ordinary text not justified
+		// Ordinary text shouldn't be justified
+		seq_AddTextFromFile(pTextName, SEQ_TEXT_POSITION);
 	}
 
 	if (bSeqSubtitles)
 	{
 		char aSubtitleName[MAX_STR_LENGTH];
+		size_t check_len = sstrcpy(aSubtitleName, pSeqName);
+		char* extension;
 
-		//check for a subtitle file
-		strLen = strlen(pSeqName);
-		ASSERT( strLen < MAX_STR_LENGTH,"seq_AddSeqToList: sequence name error" );
-		sstrcpy(aSubtitleName, pSeqName);
-		aSubtitleName[strLen - 4] = 0;
-		sstrcat(aSubtitleName, ".txt");
-		seq_AddTextFromFile(aSubtitleName, true);//SEQ_TEXT_JUSTIFY);//subtitles centre justified
+		ASSERT(check_len < sizeof(aSubtitleName), "given sequence name (%s) longer (%lu) than buffer (%lu)", pSeqName, (unsigned long) check_len, (unsigned long) sizeof(aSubtitleName));
+
+		// check for a subtitle file
+		extension = strrchr(aSubtitleName, '.');
+		if (extension)
+			*extension = '\0';
+		check_len = sstrcat(aSubtitleName, ".txt");
+		ASSERT(check_len < sizeof(aSubtitleName), "sequence name to long to attach an extension too");
+
+		// Subtitles should be center justified
+		seq_AddTextFromFile(aSubtitleName, SEQ_TEXT_JUSTIFY);
 	}
 }
 
@@ -606,7 +722,7 @@ static void seqDispCDOK( void )
 	}
 	else
 	{
-		bPlayedOK = seq_StartFullScreenVideo( aSeqList[currentPlaySeq].pSeq, aSeqList[currentPlaySeq].pAudio );
+		bPlayedOK = seq_StartFullScreenVideo(aSeqList[currentPlaySeq].pSeq, aSeqList[currentPlaySeq].pAudio, VIDEO_USER_CHOSEN_RESOLUTION);
 	}
 
 	if ( bPlayedOK == false )
@@ -615,7 +731,10 @@ static void seqDispCDOK( void )
 		if (!getScriptWinLoseVideo())
 		{
 			debug(LOG_SCRIPT, "*** Called video quit trigger!");
-			eventFireCallbackTrigger((TRIGGER_TYPE)CALL_VIDEO_QUIT);
+			// Not sure this is correct... CHECK, since the callback should ONLY
+			// be called when a video is playing (always?)
+			if (seq_Playing())
+				eventFireCallbackTrigger((TRIGGER_TYPE)CALL_VIDEO_QUIT);
 		}
 		else
 		{
