@@ -1,7 +1,7 @@
 /*
 	This file is part of Warzone 2100.
 	Copyright (C) 1999-2004  Eidos Interactive
-	Copyright (C) 2005-2007  Warzone Resurrection Project
+	Copyright (C) 2005-2009  Warzone Resurrection Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -27,23 +27,15 @@
 #include "lib/framework/strres.h"
 #include "text.h"
 
-/* The numbers of strings that will be loaded */
-#define STRING_INIT		20
-#define STRING_EXT		5
-
-
 /* The string resource object */
-STR_RES *psStringRes;
+struct STR_RES* psStringRes = NULL;
 
 /* Initialise the string system */
 BOOL stringsInitialise(void)
 {
-	if (!strresCreate(&psStringRes, STRING_INIT, STRING_EXT))
-	{
-		return false;
-	}
+	psStringRes = strresCreate();
 
-	return true;
+	return psStringRes != NULL;
 }
 
 
