@@ -1,7 +1,7 @@
 /*
 	This file is part of Warzone 2100.
 	Copyright (C) 1999-2004  Eidos Interactive
-	Copyright (C) 2005-2007  Warzone Resurrection Project
+	Copyright (C) 2005-2009  Warzone Resurrection Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -26,7 +26,7 @@
 
 #include "lib/framework/frame.h"
 #include "lib/framework/strres.h"
-#include "lib/framework/math-help.h"
+#include "lib/framework/math_ext.h"
 #include "lib/widget/label.h"
 #include "lib/widget/widget.h"
 #include "lib/ivis_common/textdraw.h"
@@ -53,6 +53,7 @@
 #include "lib/sound/audio.h"
 #include "lib/sound/audio_id.h"
 // FIXME Direct iVis implementation include!
+#include "lib/framework/fixedpoint.h"
 #include "lib/ivis_opengl/piematrix.h"
 #include "mapgrid.h"
 
@@ -1048,6 +1049,17 @@ UDWORD calcRemainingCapacity(DROID *psTransporter)
 	return (UDWORD)capacity;
 }
 
+bool transporterIsEmpty(const DROID* psTransporter)
+{
+	ASSERT(psTransporter->droidType == DROID_TRANSPORTER, "Non-transporter droid given");
+
+	// Assume dead droids and non-transporter droids to be empty
+	return (isDead((const BASE_OBJECT*)psTransporter)
+	     || psTransporter->droidType != DROID_TRANSPORTER
+
+	     || psTransporter->psGroup->psList == NULL
+	     || psTransporter->psGroup->psList == psTransporter);
+}
 
 #define MAX_TRANSPORTERS	8
 
@@ -1234,6 +1246,7 @@ void intRemoveTrans(void)
 		Form->pUserData = NULL; // Used to signal when the close anim has finished.
 		ClosingTrans = true;
 	}
+
 	intRemoveTransContent();
 	intRemoveTransDroidsAvail();
 	intMode = INT_NORMAL;
@@ -1363,55 +1376,57 @@ void transporterRemoveDroid(UDWORD id)
 	}
 	if (psDroid)
 	{
-        /*if we're offWorld we can't pick a tile without swapping the map
-        pointers - can't be bothered so just do this...*/
-        if (onMission)
-        {
-            psDroid->pos.x = INVALID_XY;
-            psDroid->pos.y = INVALID_XY;
-        }
-        else
-        {
-            if (bMultiPlayer)
-            {
-                //set the units next to the transporter's current location
-                droidX = map_coord(psCurrTransporter->pos.x);
-                droidY = map_coord(psCurrTransporter->pos.y);
-            }
-            else
-            {
-        		//pick a tile because save games won't remember where the droid was when it was loaded
-	        	droidX = map_coord(getLandingX(0));
-		        droidY = map_coord(getLandingY(0));
-            }
-    		if (!pickATileGen(&droidX, &droidY,LOOK_FOR_EMPTY_TILE,zonedPAT))
-	    	{
-		    	ASSERT( false, "transporterRemoveUnit: Unable to find a valid location" );
-    		}
-	    	psDroid->pos.x = (UWORD)world_coord(droidX);
-		    psDroid->pos.y = (UWORD)world_coord(droidY);
-    		psDroid->pos.z = map_Height(psDroid->pos.x, psDroid->pos.y);
+		/*if we're offWorld we can't pick a tile without swapping the map
+		pointers - can't be bothered so just do this...*/
+		if (onMission)
+		{
+			psDroid->pos.x = INVALID_XY;
+			psDroid->pos.y = INVALID_XY;
+		}
+		else
+		{
+			if (bMultiPlayer)
+			{
+				//set the units next to the transporter's current location
+				droidX = map_coord(psCurrTransporter->pos.x);
+				droidY = map_coord(psCurrTransporter->pos.y);
+			}
+			else
+			{
+				//pick a tile because save games won't remember where the droid was when it was loaded
+				droidX = map_coord(getLandingX(0));
+				droidY = map_coord(getLandingY(0));
+			}
+			if (!pickATileGen(&droidX, &droidY,LOOK_FOR_EMPTY_TILE,zonedPAT))
+			{
+				ASSERT( false, "transporterRemoveUnit: Unable to find a valid location" );
+			}
+			psDroid->pos.x = (UWORD)world_coord(droidX);
+			psDroid->pos.y = (UWORD)world_coord(droidY);
+			psDroid->pos.z = map_Height(psDroid->pos.x, psDroid->pos.y);
+		}
 
-        }
 		// remove it from the transporter group
-	    grpLeave(psDroid->psGroup, psDroid);
+		grpLeave(psDroid->psGroup, psDroid);
 
-	    //add it back into apsDroidLists
-	    if (onMission)
-	    {
-		    //addDroid(psDroid, mission.apsBuiltDroids);
-		    addDroid(psDroid, mission.apsDroidLists);
-	    }
-	    else
-	    {
+		//add it back into apsDroidLists
+		if (onMission)
+		{
+			//addDroid(psDroid, mission.apsBuiltDroids);
+			addDroid(psDroid, mission.apsDroidLists);
+		}
+		else
+		{
 			// add the droid back onto the droid list
-		    addDroid(psDroid, apsDroidLists);
+			addDroid(psDroid, apsDroidLists);
+
 			//inform all other players about that
 			if (bMultiPlayer)
 			{
 				sendDroidDisEmbark(psDroid,psCurrTransporter);
 			}
-	    }
+		}
+
 		// We can update the orders now, since everyone has been
 		// notified of the droid exiting the transporter
 		updateDroidOrientation(psDroid);
@@ -1422,21 +1437,22 @@ void transporterRemoveDroid(UDWORD id)
 		gridAddObject((BASE_OBJECT *)psDroid);
 		psDroid->cluster = 0;
 		// check if it is a commander
-   		if (psDroid->droidType == DROID_COMMAND)
-    	{
-	    	if (grpCreate(&psGroup))
-		    {
-			    grpJoin(psGroup, psDroid);
-		    }
-	    }
-   		psDroid->selected = true;
+		if (psDroid->droidType == DROID_COMMAND)
+		{
+			if (grpCreate(&psGroup))
+			{
+				grpJoin(psGroup, psDroid);
+			}
+		}
+		psDroid->selected = true;
 
-        if (calcRemainingCapacity(psCurrTransporter))
-        {
-            //make sure the button isn't flashing
-            stopMissionButtonFlash(IDTRANS_LAUNCH);
-        }
+		if (calcRemainingCapacity(psCurrTransporter))
+		{
+			//make sure the button isn't flashing
+			stopMissionButtonFlash(IDTRANS_LAUNCH);
+		}
 	}
+
 	// we want to sync with all clients *now*.
 	ForceDroidSync(psDroid);
 }
@@ -1472,7 +1488,7 @@ void intTransporterAddDroid(UDWORD id)
 /*Adds a droid to the transporter, removing it from the world */
 void transporterAddDroid(DROID *psTransporter, DROID *psDroidToAdd)
 {
-    BOOL    bDroidRemoved;
+	BOOL    bDroidRemoved;
 
 	ASSERT( psTransporter != NULL, "Was passed a NULL transporter" );
 	ASSERT( psDroidToAdd != NULL, "Was passed a NULL droid, can't add to transporter" );
@@ -1497,23 +1513,24 @@ void transporterAddDroid(DROID *psTransporter, DROID *psDroidToAdd)
 		// removing from droid list
 		bDroidRemoved = droidRemove(psDroidToAdd, apsDroidLists);
 	}
+
 	if (bDroidRemoved)
 	{
 		// adding to transporter unit's group list
 		grpJoin(psTransporter->psGroup, psDroidToAdd);
-
-        if (bMultiPlayer)
-        {
+		
+		if (bMultiPlayer)
+		{
 			//inform all other players to update their local lists
 			sendDroidEmbark(psDroidToAdd,psTransporter);
-        }
-
+		}
 	}
 	else
-    {
+	{
 		debug(LOG_ERROR,"droid %d not found, so nothing added to transporter!",psDroidToAdd->id);
-    }
-    //this is called by droidRemove
+	}
+
+	//this is called by droidRemove
 	//intRefreshScreen();
 }
 
@@ -1873,7 +1890,7 @@ DROID * transporterGetScriptCurrent( void )
 }
 
 /*called when a Transporter has arrived back at the LZ when sending droids to safety*/
-void resetTransporter(DROID *psTransporter)
+void resetTransporter()
 {
      W_CLICKFORM *psForm;
 
